/**
 * Copyright 2014-present Palantir Technologies
 * @license MIT
 */
import * as d3 from "d3";
import { Dataset } from "../core/dataset";
import { SimpleSelection } from "../core/interfaces";
import * as Drawers from "./";
/**
 * A Drawer is responsible for actually committing the DrawSteps to the DOM. You first pass a renderArea
 * to the Drawer, which is the root DOM node holding all the drawing elements. Subclasses set an _svgElementName
 * which is an HTML/SVG tag name. Then you call .draw() with the DrawSteps to draw, and the Drawer will draw
 * to the DOM by clearing old DOM elements, adding new DOM elements, and then passing those DOM elements to
 * the animator, which will set the appropriate attributes on the DOM.
 *
 * "Drawing" in Plottable really means either:
 * (a) "making the DOM elements and their attributes correctly reflect the data being passed in", using SVG.
 * (b) "making draw commands to the Canvas element", using Canvas.
 */
export declare class Drawer {
    private _renderArea;
    private _canvas;
    protected _svgElementName: string;
    protected _className: string;
    protected _dataset: Dataset;
    private _cachedSelectionValid;
    private _cachedSelection;
    private _cachedSelectionNodes;
    /**
     * A Drawer draws svg elements based on the input Dataset.
     *
     * @constructor
     * @param {Dataset} dataset The dataset associated with this Drawer
     */
    constructor(dataset: Dataset);
    canvas(): d3.Selection<HTMLCanvasElement, any, any, any>;
    /**
     * Sets the renderArea() to null and tells this Drawer to draw to the specified canvas instead.
     * @param canvas
     */
    canvas(canvas: d3.Selection<HTMLCanvasElement, any, any, any>): this;
    /**
     * Retrieves the renderArea selection for the Drawer.
     */
    renderArea(): SimpleSelection<void>;
    /**
     * Sets the canvas() to null and tells this Drawer to draw to the specified SVG area instead.
     *
     * @param {d3.Selection} Selection containing the <g> to render to.
     * @returns {Drawer} The calling Drawer.
     */
    renderArea(area: SimpleSelection<void>): this;
    /**
     * Removes the Drawer and its renderArea
     */
    remove(): void;
    /**
     * Binds data to selection
     *
     * @param{any[]} data The data to be drawn
     */
    private _bindSelectionData(data);
    protected _applyDefaultAttributes(selection: SimpleSelection<any>): void;
    /**
     * Draws data using one step
     *
     * @param{AppliedDrawStep} step The step, how data should be drawn.
     */
    private _drawStep(step);
    protected _drawStepCanvas(data: any[], step: Drawers.AppliedDrawStep): void;
    private _appliedProjectors(attrToProjector);
    /**
     * Calculates the total time it takes to use the input drawSteps to draw the input data
     *
     * @param {any[]} data The data that would have been drawn
     * @param {Drawers.DrawStep[]} drawSteps The DrawSteps to use
     * @returns {number} The total time it takes to draw
     */
    totalDrawTime(data: any[], drawSteps: Drawers.DrawStep[]): number;
    /**
     * Draws the data into the renderArea using the spefic steps and metadata
     *
     * @param{any[]} data The data to be drawn
     * @param{DrawStep[]} drawSteps The list of steps, which needs to be drawn
     */
    draw(data: any[], drawSteps: Drawers.DrawStep[]): this;
    selection(): SimpleSelection<any> | null;
    /**
     * Returns the CSS selector for this Drawer's visual elements.
     */
    selector(): string;
    /**
     * Returns the D3 selection corresponding to the datum with the specified index.
     */
    selectionForIndex(index: number): SimpleSelection<any> | null;
    private maybeRefreshCache();
}
