/**
 * Copyright 2014-present Palantir Technologies
 * @license MIT
 */
"use strict";
var d3 = require("d3");
var Utils = require("../utils");
var coerceD3_1 = require("../utils/coerceD3");
/**
 * A Drawer is responsible for actually committing the DrawSteps to the DOM. You first pass a renderArea
 * to the Drawer, which is the root DOM node holding all the drawing elements. Subclasses set an _svgElementName
 * which is an HTML/SVG tag name. Then you call .draw() with the DrawSteps to draw, and the Drawer will draw
 * to the DOM by clearing old DOM elements, adding new DOM elements, and then passing those DOM elements to
 * the animator, which will set the appropriate attributes on the DOM.
 *
 * "Drawing" in Plottable really means either:
 * (a) "making the DOM elements and their attributes correctly reflect the data being passed in", using SVG.
 * (b) "making draw commands to the Canvas element", using Canvas.
 */
var Drawer = (function () {
    /**
     * A Drawer draws svg elements based on the input Dataset.
     *
     * @constructor
     * @param {Dataset} dataset The dataset associated with this Drawer
     */
    function Drawer(dataset) {
        this._cachedSelectionValid = false;
        this._dataset = dataset;
        this._svgElementName = "path";
    }
    Drawer.prototype.canvas = function (canvas) {
        if (canvas === undefined) {
            return this._canvas;
        }
        canvas = coerceD3_1.coerceExternalD3(canvas);
        this._canvas = canvas;
        this._renderArea = null;
        this._cachedSelectionValid = false;
        return this;
    };
    Drawer.prototype.renderArea = function (area) {
        if (area === undefined) {
            return this._renderArea;
        }
        area = coerceD3_1.coerceExternalD3(area);
        this._renderArea = area;
        this._canvas = null;
        this._cachedSelectionValid = false;
        return this;
    };
    /**
     * Removes the Drawer and its renderArea
     */
    Drawer.prototype.remove = function () {
        if (this.renderArea() != null) {
            this.renderArea().remove();
        }
    };
    /**
     * Binds data to selection
     *
     * @param{any[]} data The data to be drawn
     */
    Drawer.prototype._bindSelectionData = function (data) {
        var dataElementsUpdate = this.selection().data(data);
        var dataElements = dataElementsUpdate
            .enter()
            .append(this._svgElementName)
            .merge(dataElementsUpdate);
        dataElementsUpdate.exit().remove();
        this._applyDefaultAttributes(dataElements);
    };
    Drawer.prototype._applyDefaultAttributes = function (selection) {
        if (this._className != null) {
            selection.classed(this._className, true);
        }
    };
    /**
     * Draws data using one step
     *
     * @param{AppliedDrawStep} step The step, how data should be drawn.
     */
    Drawer.prototype._drawStep = function (step) {
        var selection = this.selection();
        var colorAttributes = ["fill", "stroke"];
        colorAttributes.forEach(function (colorAttribute) {
            if (step.attrToAppliedProjector[colorAttribute] != null) {
                selection.attr(colorAttribute, step.attrToAppliedProjector[colorAttribute]);
            }
        });
        step.animator.animate(selection, step.attrToAppliedProjector);
        if (this._className != null) {
            this.selection().classed(this._className, true);
        }
    };
    Drawer.prototype._drawStepCanvas = function (data, step) {
        Utils.Window.warn("canvas rendering not yet implemented on " + this.constructor.name);
    };
    Drawer.prototype._appliedProjectors = function (attrToProjector) {
        var _this = this;
        var modifiedAttrToProjector = {};
        Object.keys(attrToProjector).forEach(function (attr) {
            modifiedAttrToProjector[attr] =
                function (datum, index) { return attrToProjector[attr](datum, index, _this._dataset); };
        });
        return modifiedAttrToProjector;
    };
    /**
     * Calculates the total time it takes to use the input drawSteps to draw the input data
     *
     * @param {any[]} data The data that would have been drawn
     * @param {Drawers.DrawStep[]} drawSteps The DrawSteps to use
     * @returns {number} The total time it takes to draw
     */
    Drawer.prototype.totalDrawTime = function (data, drawSteps) {
        var delay = 0;
        drawSteps.forEach(function (drawStep, i) {
            delay += drawStep.animator.totalTime(data.length);
        });
        return delay;
    };
    /**
     * Draws the data into the renderArea using the spefic steps and metadata
     *
     * @param{any[]} data The data to be drawn
     * @param{DrawStep[]} drawSteps The list of steps, which needs to be drawn
     */
    Drawer.prototype.draw = function (data, drawSteps) {
        var _this = this;
        var appliedDrawSteps = drawSteps.map(function (dr) {
            var attrToAppliedProjector = _this._appliedProjectors(dr.attrToProjector);
            return {
                attrToAppliedProjector: attrToAppliedProjector,
                animator: dr.animator,
            };
        });
        if (this._renderArea != null) {
            this._bindSelectionData(data);
            this._cachedSelectionValid = false;
            var delay_1 = 0;
            appliedDrawSteps.forEach(function (drawStep, i) {
                Utils.Window.setTimeout(function () { return _this._drawStep(drawStep); }, delay_1);
                delay_1 += drawStep.animator.totalTime(data.length);
            });
        }
        else if (this._canvas != null) {
            // don't support animations for now; just draw the last draw step immediately
            var lastDrawStep_1 = appliedDrawSteps[appliedDrawSteps.length - 1];
            Utils.Window.setTimeout(function () { return _this._drawStepCanvas(data, lastDrawStep_1); }, 0);
        }
        else {
            throw new Error("Drawer's canvas and renderArea are both null!");
        }
        return this;
    };
    Drawer.prototype.selection = function () {
        this.maybeRefreshCache();
        return this._cachedSelection;
    };
    /**
     * Returns the CSS selector for this Drawer's visual elements.
     */
    Drawer.prototype.selector = function () {
        return this._svgElementName;
    };
    /**
     * Returns the D3 selection corresponding to the datum with the specified index.
     */
    Drawer.prototype.selectionForIndex = function (index) {
        this.maybeRefreshCache();
        if (this._cachedSelectionNodes != null) {
            return d3.select(this._cachedSelectionNodes[index]);
        }
        else {
            return null;
        }
    };
    Drawer.prototype.maybeRefreshCache = function () {
        if (!this._cachedSelectionValid) {
            if (this._renderArea != null) {
                this._cachedSelection = this.renderArea().selectAll(this.selector());
                this._cachedSelectionNodes = this._cachedSelection.nodes();
            }
            else {
                this._cachedSelection = null;
                this._cachedSelectionNodes = null;
            }
            this._cachedSelectionValid = true;
        }
    };
    return Drawer;
}());
exports.Drawer = Drawer;
