"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Axes", function () {
    describe("NumericAxis", function () {
        function assertBoxInside(inner, outer, epsilon, message) {
            if (epsilon === void 0) { epsilon = 0; }
            if (message === void 0) { message = ""; }
            chai_1.assert.operator(inner.left, ">", outer.left - epsilon, message + " (box inside (left))");
            chai_1.assert.operator(inner.right, "<", outer.right + epsilon, message + " (box inside (right))");
            chai_1.assert.operator(inner.top, ">", outer.top - epsilon, message + " (box inside (top))");
            chai_1.assert.operator(inner.bottom, "<", outer.bottom + epsilon, message + " (box inside (bottom))");
        }
        function applyVisibleFilter(selection) {
            return selection.filter(function () {
                var visibilityAttr = d3.select(this).style("visibility");
                return visibilityAttr === "visible" || visibilityAttr === "inherit";
            });
        }
        var horizontalOrientations = ["top", "bottom"];
        var verticalOrientations = ["left", "right"];
        var orientations = horizontalOrientations.concat(verticalOrientations);
        var isHorizontalOrientation = function (orientation) { return horizontalOrientations.indexOf(orientation) >= 0; };
        describe("managing tick labels", function () {
            var verticalTickLabelPositions = ["top", "bottom"];
            var horizontalTickLabelPositions = ["left", "right"];
            horizontalOrientations.forEach(function (orientation) {
                it("throws an error when setting an invalid position on orientation " + orientation, function () {
                    var scale = new Plottable.Scales.Linear();
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    verticalTickLabelPositions.forEach(function (position) {
                        chai_1.assert.throws(function () { return axis.tickLabelPosition(position); }, "horizontal", "cannot set position to " + position);
                    });
                });
            });
            verticalOrientations.forEach(function (orientation) {
                it("throws an error when setting an invalid position on orientation " + orientation, function () {
                    var scale = new Plottable.Scales.Linear();
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    horizontalTickLabelPositions.forEach(function (position) {
                        chai_1.assert.throws(function () { return axis.tickLabelPosition(position); }, "vertical", "cannot set position to " + position);
                    });
                });
            });
            orientations.forEach(function (orientation) {
                var labelPositions = isHorizontalOrientation(orientation) ? horizontalTickLabelPositions : verticalTickLabelPositions;
                labelPositions.forEach(function (labelPosition) {
                    it("draws tick labels " + labelPosition + " of the corresponding mark if specified for orientation " + orientation, function () {
                        var scale = new Plottable.Scales.Linear();
                        var axis = new Plottable.Axes.Numeric(scale, orientation);
                        var div = TestMethods.generateDiv();
                        axis.tickLabelPosition(labelPosition);
                        axis.renderTo(div);
                        var tickLabels = axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS);
                        var tickMarks = axis.content().selectAll("." + Plottable.Axis.TICK_MARK_CLASS);
                        chai_1.assert.operator(tickLabels.size(), ">=", 2, "at least two tick labels were drawn");
                        chai_1.assert.strictEqual(tickLabels.size(), tickMarks.size(), "there is one label per mark");
                        tickLabels.each(function (d, i) {
                            TestMethods.assertBBoxNonIntersection(d3.select(this), d3.select(tickMarks.nodes()[i]));
                        });
                        div.remove();
                    });
                });
            });
            orientations.forEach(function (orientation) {
                it("does not overlap tick labels in a constrained space for orientation " + orientation, function () {
                    var div = TestMethods.generateDiv();
                    var constrainedWidth = 50;
                    var constrainedHeight = 50;
                    div.style("width", constrainedWidth + "px");
                    div.style("height", constrainedHeight + "px");
                    var scale = new Plottable.Scales.Linear();
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    axis.renderTo(div);
                    var visibconstickLabels = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS));
                    visibconstickLabels.each(function (d, i) {
                        var labelRect = this.getBoundingClientRect();
                        visibconstickLabels.nodes().slice(i + 1).forEach(function (otherVisibconstickLabel, i2) {
                            var labelRect2 = otherVisibconstickLabel.getBoundingClientRect();
                            var rectOverlap = Plottable.Utils.DOM.clientRectsOverlap(labelRect, labelRect2);
                            chai_1.assert.isFalse(rectOverlap, "tick label " + i + " does not overlap with tick label " + i2);
                        });
                    });
                    div.remove();
                });
            });
            orientations.forEach(function (orientation) {
                it("separates tick labels with the same spacing for orientation " + orientation, function () {
                    var div = TestMethods.generateDiv();
                    var scale = new Plottable.Scales.Linear();
                    scale.domain([-2500000, 2500000]);
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    axis.renderTo(div);
                    var visibconstickLabels = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS));
                    var visibconstickLabelRects = visibconstickLabels.nodes().map(function (label) { return label.getBoundingClientRect(); });
                    function getClientRectCenter(rect) {
                        return isHorizontalOrientation(orientation) ? rect.left + rect.width / 2 : rect.top + rect.height / 2;
                    }
                    var interval = getClientRectCenter(visibconstickLabelRects[1]) - getClientRectCenter(visibconstickLabelRects[0]);
                    d3.pairs(visibconstickLabelRects).forEach(function (rects, i) {
                        chai_1.assert.closeTo(getClientRectCenter(rects[1]) - getClientRectCenter(rects[0]), interval, window.Pixel_CloseTo_Requirement, "tick label pair " + i + " is spaced the same as the first pair");
                    });
                    div.remove();
                });
            });
            orientations.forEach(function (orientation) {
                it("renders numbers in order with a reversed domain for orientation " + orientation, function () {
                    var div = TestMethods.generateDiv();
                    var scale = new Plottable.Scales.Linear();
                    scale.domain([3, 0]);
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    axis.renderTo(div);
                    var tickLabels = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS));
                    chai_1.assert.operator(tickLabels.size(), ">", 1, "tick labels are shown");
                    div.remove();
                });
            });
            verticalOrientations.forEach(function (orientation) {
                it("ensures that long labels are contained for vertical orientation " + orientation, function () {
                    var scale = new Plottable.Scales.Linear().domain([400000000, 500000000]);
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    var div = TestMethods.generateDiv();
                    axis.renderTo(div);
                    var tickLabelContainerClass = "tick-label-container";
                    var labelContainer = axis.content().select("." + tickLabelContainerClass);
                    axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS).each(function () {
                        TestMethods.assertBBoxInclusion(labelContainer, d3.select(this));
                    });
                    div.remove();
                });
            });
            orientations.forEach(function (orientation) {
                it("draws ticks labels centered with the corresponding tick mark for orientation " + orientation, function () {
                    var div = TestMethods.generateDiv();
                    var scale = new Plottable.Scales.Linear();
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    axis.renderTo(div);
                    var tickLabels = axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS);
                    chai_1.assert.operator(tickLabels.size(), ">=", 2, "at least two tick labels were drawn");
                    var tickMarks = axis.content().selectAll("." + Plottable.Axis.TICK_MARK_CLASS);
                    chai_1.assert.strictEqual(tickLabels.size(), tickMarks.size(), "there is one label per mark");
                    tickLabels.each(function (d, i) {
                        var tickLabelClientRect = this.getBoundingClientRect();
                        var tickMarkClientRect = tickMarks.nodes()[i].getBoundingClientRect();
                        var labelCenter = isHorizontalOrientation(orientation) ?
                            (tickLabelClientRect.left + tickLabelClientRect.right) / 2 :
                            (tickLabelClientRect.top + tickLabelClientRect.bottom) / 2;
                        var markCenter = isHorizontalOrientation(orientation) ?
                            (tickMarkClientRect.left + tickMarkClientRect.right) / 2 :
                            (tickMarkClientRect.top + tickMarkClientRect.bottom) / 2;
                        chai_1.assert.closeTo(labelCenter, markCenter, 1.5, "tick label " + i + " is centered on mark");
                    });
                    div.remove();
                });
            });
            it("does not overlap tick marks with tick labels", function () {
                var div = TestMethods.generateDiv();
                var scale = new Plottable.Scales.Linear();
                scale.domain([175, 185]);
                var axis = new Plottable.Axes.Numeric(scale, "left")
                    .innerTickLength(50);
                axis.renderTo(div);
                var tickLabels = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS));
                var tickMarks = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_MARK_CLASS));
                tickLabels.each(function (d, i) {
                    var tickLabelRect = this.getBoundingClientRect();
                    tickMarks.each(function (d2, i2) {
                        var tickMarkRect = this.getBoundingClientRect();
                        chai_1.assert.isFalse(Plottable.Utils.DOM.clientRectsOverlap(tickLabelRect, tickMarkRect), "tickMark " + i + " and tickLabel " + i2 + " should not overlap");
                    });
                });
                div.remove();
            });
        });
        describe("formatting the labels", function () {
            it("formats to the specified formatter", function () {
                var div = TestMethods.generateDiv();
                var scale = new Plottable.Scales.Linear();
                var formatter = Plottable.Formatters.fixed(2);
                var axis = new Plottable.Axes.Numeric(scale, "left");
                axis.formatter(formatter);
                axis.renderTo(div);
                var tickLabels = axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS);
                tickLabels.each(function (d, i) {
                    var labelText = d3.select(this).text();
                    chai_1.assert.strictEqual(labelText, formatter(d), "formatter used to format tick label " + i);
                });
                div.remove();
            });
        });
        describe("allocating space", function () {
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv();
            });
            afterEach(function () {
                if (this.currentTest.state === "passed") {
                    div.remove();
                }
            });
            orientations.forEach(function (orientation) {
                it("allocates enough height to show all tick labels for orientation " + orientation, function () {
                    var scale = new Plottable.Scales.Linear();
                    scale.domain([5, -5]);
                    var axis = new Plottable.Axes.Numeric(scale, orientation);
                    axis.renderTo(div);
                    var visibconstickLabels = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS));
                    var boundingBox = axis.element().node().getBoundingClientRect();
                    visibconstickLabels.each(function (d, i) {
                        var visibconstickLabelRect = this.getBoundingClientRect();
                        assertBoxInside(visibconstickLabelRect, boundingBox, 0.5, "tick label " + i + " is inside the bounding box");
                    });
                });
            });
            verticalOrientations.forEach(function (verticalOrientation) {
                it("allocates enough width to show long tick labels for orientation " + verticalOrientation, function () {
                    var scale = new Plottable.Scales.Linear();
                    scale.domain([50000000000, -50000000000]);
                    var axis = new Plottable.Axes.Numeric(scale, verticalOrientation);
                    axis.renderTo(div);
                    var visibconstickLabels = applyVisibleFilter(axis.content().selectAll("." + Plottable.Axis.TICK_LABEL_CLASS));
                    var boundingBox = axis.element().node().getBoundingClientRect();
                    visibconstickLabels.each(function (d, i) {
                        var visibconstickLabelRect = this.getBoundingClientRect();
                        assertBoxInside(visibconstickLabelRect, boundingBox, 0, "long tick label " + i + " is inside the bounding box");
                    });
                });
            });
        });
        describe("drawing tick marks", function () {
            it("does not draw ticks marks outside of the div", function () {
                var div = TestMethods.generateDiv();
                var scale = new Plottable.Scales.Linear();
                var domainMin = 0;
                var domainMax = 3;
                scale.domain([domainMin, domainMax]);
                scale.tickGenerator(function () {
                    return Plottable.Utils.Math.range(domainMin, domainMax + 4);
                });
                var axis = new Plottable.Axes.Numeric(scale, "bottom");
                axis.renderTo(div);
                var tickMarks = axis.content().selectAll(".tick-mark");
                tickMarks.each(function (d, i) {
                    var tickMark = d3.select(this);
                    var tickMarkPosition = TestMethods.numAttr(tickMark, "x1");
                    chai_1.assert.operator(tickMarkPosition, ">=", 0, "tick mark " + i + " drawn right of left edge");
                    chai_1.assert.operator(tickMarkPosition, "<=", axis.width(), "tick mark " + i + " drawn left of right edge");
                });
                div.remove();
            });
        });
        describe("using width approximation", function () {
            it("reasonably approximates tick label sizes with approximate measuring", function () {
                var div = TestMethods.generateDiv();
                var testDomains = [
                    [-1, 1],
                    [0, 10],
                    [0, 999999999],
                ];
                var maxErrorFactor = 1.4;
                testDomains.forEach(function (testDomain) {
                    var scale = new Plottable.Scales.Linear();
                    scale.domain(testDomain);
                    var axis = new Plottable.Axes.Numeric(scale, "left");
                    axis.usesTextWidthApproximation(true);
                    axis.renderTo(div);
                    var widthApprox = axis.width();
                    axis.usesTextWidthApproximation(false);
                    axis.redraw();
                    var widthExact = axis.width();
                    chai_1.assert.operator(widthApprox, "<", (widthExact * maxErrorFactor), "approximate domain of [" + testDomain[0] + "," + testDomain[1] + "] less than " + maxErrorFactor + " times larger than exact scale");
                    chai_1.assert.operator(widthApprox, ">=", widthExact, "approximate domain of [" + testDomain[0] + "," + testDomain[1] + "] greater than an exact scale");
                    axis.destroy();
                });
                div.remove();
            });
        });
    });
});
