"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("SelectionBoxLayer", function () {
    describe("DragBoxLayer", function () {
        describe("Basics usage", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var quarterPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
            var halfPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
            var DRAG_BOX_LAYER_EVENTS = ["dragStart", "drag", "dragEnd"];
            var div;
            var dbl;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dbl = new Plottable.Components.DragBoxLayer();
            });
            function onEvent(event, callback) {
                switch (event) {
                    case "dragStart":
                        dbl.onDragStart(callback);
                        break;
                    case "drag":
                        dbl.onDragStart(callback);
                        break;
                    case "dragEnd":
                        dbl.onDragEnd(callback);
                        break;
                }
            }
            function offEvent(event, callback) {
                switch (event) {
                    case "dragStart":
                        dbl.offDragStart(callback);
                        break;
                    case "drag":
                        dbl.offDragStart(callback);
                        break;
                    case "dragEnd":
                        dbl.offDragEnd(callback);
                        break;
                }
            }
            it("draws box on drag", function () {
                dbl.renderTo(div);
                chai_1.assert.isFalse(dbl.boxVisible(), "box is hidden initially");
                var target = dbl.background();
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isTrue(dbl.boxVisible(), "box is drawn on drag");
                var bounds = dbl.bounds();
                chai_1.assert.deepEqual(bounds.topLeft, quarterPoint, "top-left point was set correctly");
                chai_1.assert.deepEqual(bounds.bottomRight, halfPoint, "bottom-right point was set correctly");
                div.remove();
            });
            it("dismisses on click", function () {
                dbl.renderTo(div);
                var target = dbl.background();
                chai_1.assert.isFalse(dbl.boxVisible(), "box is hidden initially");
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isTrue(dbl.boxVisible(), "box is drawn on drag");
                TestMethods.triggerFakeDragSequence(target, halfPoint, halfPoint);
                chai_1.assert.isFalse(dbl.boxVisible(), "box is hidden on click");
                div.remove();
            });
            it("can get and set the detection radius", function () {
                chai_1.assert.strictEqual(dbl.detectionRadius(), 3, "there is a default detection radius");
                chai_1.assert.doesNotThrow(function () { return dbl.detectionRadius(4); }, Error, "can set detection radius before anchoring");
                dbl.renderTo(div);
                chai_1.assert.strictEqual(dbl.detectionRadius(), 4, "detection radius did not change upon rendering");
                chai_1.assert.strictEqual(dbl.detectionRadius(5), dbl, "setting the detection radius returns the drag box layer");
                chai_1.assert.strictEqual(dbl.detectionRadius(), 5, "can retrieve the detection radius");
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return dbl.detectionRadius(-1); }, Error, "detection radius cannot be negative", "fails to set negative radius");
                div.remove();
            });
            it("applies the given detection radius property", function () {
                dbl.renderTo(div);
                var radius = 5;
                dbl.detectionRadius(radius);
                TestMethods.triggerFakeDragSequence(dbl.background(), quarterPoint, halfPoint);
                var edges = dbl.content().selectAll("line");
                chai_1.assert.strictEqual(edges.size(), 4, "the edges of a rectangle are drawn");
                edges.each(function () {
                    var edge = d3.select(this);
                    var strokeWidth = parseFloat(edge.style("stroke-width"));
                    chai_1.assert.strictEqual(strokeWidth, 2 * radius, "edge width was set correctly");
                });
                var corners = dbl.content().selectAll("circle");
                chai_1.assert.strictEqual(corners.size(), 4, "the corners of a rectangle are drawn");
                corners.each(function () {
                    var corner = d3.select(this);
                    var cornerRadius = parseFloat(corner.attr("r"));
                    chai_1.assert.strictEqual(cornerRadius, radius, "corner radius was set correctly");
                });
                div.remove();
            });
            it("does not error on destroy if scales are not added", function () {
                chai_1.assert.doesNotThrow(function () { return dbl.destroy(); }, Error, "can destroy without scales");
                div.remove();
            });
            it("does not call callbacks when dragBoxLayer is destroyed", function () {
                // rendered in a Group so that drag sequence can be simulated on Group background after DragBoxLayer is destroyed
                var group = new Plottable.Components.Group([dbl]).renderTo(div);
                var target = group.background();
                var onDragStartCallbackCalled = false;
                var onDragCallbackCalled = false;
                var onDragEndcallbackCalled = false;
                dbl.onDragStart(function () { return onDragStartCallbackCalled = true; });
                dbl.onDrag(function () { return onDragCallbackCalled = true; });
                dbl.onDragEnd(function () { return onDragEndcallbackCalled = true; });
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isTrue(onDragStartCallbackCalled, "onDragStart callback is called");
                chai_1.assert.isTrue(onDragCallbackCalled, "onDrag callback is called");
                chai_1.assert.isTrue(onDragEndcallbackCalled, "onDragEnd callback is called");
                onDragStartCallbackCalled = false;
                onDragCallbackCalled = false;
                onDragEndcallbackCalled = false;
                dbl.destroy();
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isFalse(onDragStartCallbackCalled, "onDragStart callback is not called after destroyed");
                chai_1.assert.isFalse(onDragCallbackCalled, "onDrag callback is not called after destroyed");
                chai_1.assert.isFalse(onDragEndcallbackCalled, "onDragEnd callback is not called after destroyed");
                div.remove();
            });
            it("does not call callbacks when dragBoxLayer is detached", function () {
                // rendered in a Group so that drag sequence can be simulated on Group background after DragBoxLayer is detached
                var group = new Plottable.Components.Group([dbl]);
                group.renderTo(div);
                var target = group.background();
                var onDragStartCallbackCalled = false;
                var onDragCallbackCalled = false;
                var onDragEndcallbackCalled = false;
                dbl.onDragStart(function () { return onDragStartCallbackCalled = true; });
                dbl.onDrag(function () { return onDragCallbackCalled = true; });
                dbl.onDragEnd(function () { return onDragEndcallbackCalled = true; });
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isTrue(onDragStartCallbackCalled, "onDragStart callback is called");
                chai_1.assert.isTrue(onDragCallbackCalled, "onDrag callback is called");
                chai_1.assert.isTrue(onDragEndcallbackCalled, "onDragEnd callback is called");
                onDragStartCallbackCalled = false;
                onDragCallbackCalled = false;
                onDragEndcallbackCalled = false;
                dbl.detach();
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isFalse(onDragStartCallbackCalled, "onDragStart callback is not called after detached");
                chai_1.assert.isFalse(onDragCallbackCalled, "onDrag callback is not called after detached");
                chai_1.assert.isFalse(onDragEndcallbackCalled, "onDragEnd callback is not called after detached");
                group.append(dbl);
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isTrue(onDragStartCallbackCalled, "onDragStart callback is called when re-anchor()");
                chai_1.assert.isTrue(onDragCallbackCalled, "onDrag callback is called when re-anchor()");
                chai_1.assert.isTrue(onDragEndcallbackCalled, "onDragEnd callback is called when re-anchor()");
                dbl.destroy();
                div.remove();
            });
            DRAG_BOX_LAYER_EVENTS.forEach(function (event) {
                it("calls the " + event + " callback", function () {
                    dbl.renderTo(div);
                    var receivedBounds;
                    var callbackCalled = false;
                    var callback = function (b) {
                        receivedBounds = b;
                        callbackCalled = true;
                    };
                    onEvent(event, callback);
                    var target = dbl.background();
                    TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                    chai_1.assert.isTrue(callbackCalled, "the callback was called");
                    chai_1.assert.deepEqual(receivedBounds.topLeft, quarterPoint, "top-left point was set correctly");
                    var endPoint = event === "dragEnd" ? halfPoint : quarterPoint;
                    chai_1.assert.deepEqual(receivedBounds.bottomRight, endPoint, "bottom-right point was set correctly");
                    offEvent(event, callback);
                    callbackCalled = false;
                    TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                    chai_1.assert.isFalse(callbackCalled, "the callback was detached from the dragBoxLayer and not called");
                    div.remove();
                });
                it("can register two onDragStart callbacks on the same component", function () {
                    dbl.renderTo(div);
                    var callbackDragStart1Called = false;
                    var callbackDragStart2Called = false;
                    var callbackDragStart1 = function () { return callbackDragStart1Called = true; };
                    var callbackDragStart2 = function () { return callbackDragStart2Called = true; };
                    onEvent(event, callbackDragStart1);
                    onEvent(event, callbackDragStart2);
                    var target = dbl.background();
                    TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                    chai_1.assert.isTrue(callbackDragStart1Called, "the callback 1 for drag start was called");
                    chai_1.assert.isTrue(callbackDragStart2Called, "the callback 2 for drag start was called");
                    offEvent(event, callbackDragStart1);
                    callbackDragStart1Called = false;
                    callbackDragStart2Called = false;
                    TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                    chai_1.assert.isFalse(callbackDragStart1Called, "the callback 1 for drag start was disconnected");
                    chai_1.assert.isTrue(callbackDragStart2Called, "the callback 2 for drag start is still connected");
                    div.remove();
                });
            });
            it("calls all the drag interaction callbacks when needed", function () {
                dbl.renderTo(div);
                var callbackDragStartCalled = false;
                var callbackDragCalled = false;
                var callbackDragEndCalled = false;
                var callbackDragStart = function () { return callbackDragStartCalled = true; };
                var callbackDrag = function () { return callbackDragCalled = true; };
                var callbackDragEnd = function () { return callbackDragEndCalled = true; };
                dbl.onDragStart(callbackDragStart);
                dbl.onDrag(callbackDrag);
                dbl.onDragEnd(callbackDragEnd);
                var target = dbl.background();
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isTrue(callbackDragStartCalled, "the callback for drag start was called");
                chai_1.assert.isTrue(callbackDragCalled, "the callback for drag was called");
                chai_1.assert.isTrue(callbackDragEndCalled, "the callback for drag end was called");
                dbl.offDragStart(callbackDragStart);
                dbl.offDrag(callbackDrag);
                callbackDragStartCalled = false;
                callbackDragCalled = false;
                callbackDragEndCalled = false;
                TestMethods.triggerFakeDragSequence(target, quarterPoint, halfPoint);
                chai_1.assert.isFalse(callbackDragStartCalled, "the callback for drag start was disconnected");
                chai_1.assert.isFalse(callbackDragCalled, "the callback for drag was disconnected");
                chai_1.assert.isTrue(callbackDragEndCalled, "the callback for drag end is still connected");
                div.remove();
            });
        });
        describe("Enabling and disabling", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var div;
            var dbl;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dbl = new Plottable.Components.DragBoxLayer();
            });
            it("can enable and disable the DragBoxLayer", function () {
                chai_1.assert.isTrue(dbl.enabled(), "drag box layer is enabled by default");
                chai_1.assert.strictEqual(dbl.enabled(false), dbl, "the setter returns the drag box layer");
                chai_1.assert.isFalse(dbl.enabled(), "drag box layer can be disabled");
                chai_1.assert.strictEqual(dbl.enabled(true), dbl, "the setter returns the drag box layer");
                chai_1.assert.isTrue(dbl.enabled(), "drag box layer can be re-enabled");
                div.remove();
            });
            it("does not draw box when disabled", function () {
                dbl.enabled(false);
                dbl.renderTo(div);
                chai_1.assert.isFalse(dbl.boxVisible(), "box is hidden initially");
                var startPoint = {
                    x: DIV_WIDTH / 4,
                    y: DIV_HEIGHT / 4,
                };
                var endPoint = {
                    x: DIV_WIDTH / 2,
                    y: DIV_HEIGHT / 2,
                };
                var target = dbl.background();
                TestMethods.triggerFakeDragSequence(target, startPoint, endPoint);
                chai_1.assert.isFalse(dbl.boxVisible(), "box is not shown when disabled");
                dbl.enabled(true);
                TestMethods.triggerFakeDragSequence(target, startPoint, endPoint);
                chai_1.assert.isTrue(dbl.boxVisible(), "box is shown when enabled");
                div.remove();
            });
        });
        describe("Resizing", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var midPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
            var div;
            var dbl;
            var target;
            var initialBounds;
            function resetBox() {
                dbl.bounds({
                    topLeft: { x: 0, y: 0 },
                    bottomRight: { x: 0, y: 0 },
                });
                TestMethods.triggerFakeDragSequence(target, { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 }, { x: DIV_WIDTH * 3 / 4, y: DIV_HEIGHT * 3 / 4 });
                initialBounds = dbl.bounds();
            }
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dbl = new Plottable.Components.DragBoxLayer();
                dbl.renderTo(div);
                target = dbl.background();
                resetBox();
            });
            it("can get and set the resizable property", function () {
                chai_1.assert.isFalse(dbl.resizable(), "defaults to false");
                chai_1.assert.strictEqual(dbl.resizable(true), dbl, "the setter returns the drag box layer");
                chai_1.assert.isTrue(dbl.resizable(), "successfully set to true");
                chai_1.assert.strictEqual(dbl.resizable(false), dbl, "the setter returns the drag box layer");
                chai_1.assert.isFalse(dbl.resizable(), "successfully set back to false");
                div.remove();
            });
            it("correctly sets pointer-events for resizable DragBoxLayer", function () {
                dbl.resizable(true);
                var edges = dbl.content().selectAll("line");
                chai_1.assert.strictEqual(edges.size(), 4, "there are 4 edges per box");
                edges.each(function () {
                    var edge = d3.select(this);
                    var computedStyle = window.getComputedStyle(edge.node());
                    chai_1.assert.strictEqual(computedStyle.pointerEvents.toLowerCase(), "visiblestroke", "pointer-events set correctly on edges");
                });
                var corners = dbl.content().selectAll("circle");
                chai_1.assert.strictEqual(corners.size(), 4, "there are 4 corners per box");
                corners.each(function () {
                    var corner = d3.select(this);
                    var computedStyle = window.getComputedStyle(corner.node());
                    chai_1.assert.strictEqual(computedStyle.pointerEvents.toLowerCase(), "visiblefill", "pointer-events set correctly on corners");
                });
                div.remove();
            });
            it("can resize from top edge", function () {
                dbl.resizable(true);
                var newY = 0;
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.topLeft.y }, { x: midPoint.x, y: newY });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, newY, "top edge was repositioned");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y, "bottom edge was not moved");
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x, "right edge was not moved");
                resetBox();
                newY = DIV_HEIGHT;
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.topLeft.y }, { x: midPoint.x, y: newY });
                bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.bottomRight.y, newY, "can drag through to other side");
                div.remove();
            });
            it("can resize from bottom edge", function () {
                dbl.resizable(true);
                var newY = DIV_HEIGHT;
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.bottomRight.y }, { x: midPoint.x, y: newY });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, newY, "bottom edge was repositioned");
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x, "right edge was not moved");
                resetBox();
                newY = 0;
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.bottomRight.y }, { x: midPoint.x, y: newY });
                bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, newY, "can drag through to other side");
                div.remove();
            });
            it("can resize from left edge", function () {
                dbl.resizable(true);
                var newX = 0;
                TestMethods.triggerFakeDragSequence(target, { x: initialBounds.topLeft.x, y: midPoint.y }, { x: newX, y: midPoint.y });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y, "bottom edge was not moved");
                chai_1.assert.strictEqual(bounds.topLeft.x, newX, "left edge was repositioned");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x, "right edge was not moved");
                resetBox();
                newX = DIV_WIDTH;
                TestMethods.triggerFakeDragSequence(target, { x: initialBounds.topLeft.x, y: midPoint.y }, { x: newX, y: midPoint.y });
                bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.bottomRight.x, newX, "can drag through to other side");
                div.remove();
            });
            it("can resize from right edge", function () {
                dbl.resizable(true);
                var newX = DIV_WIDTH;
                TestMethods.triggerFakeDragSequence(target, { x: initialBounds.bottomRight.x, y: midPoint.y }, { x: newX, y: midPoint.y });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y, "bottom edge was not moved");
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, newX, "right edge was repositioned");
                resetBox();
                newX = 0;
                TestMethods.triggerFakeDragSequence(target, { x: initialBounds.bottomRight.x, y: midPoint.y }, { x: newX, y: midPoint.y });
                bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, newX, "can drag through to other side");
                div.remove();
            });
            it("resizes if grabbed within detectionRadius", function () {
                dbl.resizable(true);
                var detectionRadius = dbl.detectionRadius();
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.bottomRight.y + detectionRadius - 1 }, { x: midPoint.x, y: DIV_HEIGHT });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, DIV_HEIGHT, "bottom edge was repositioned");
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x, "right edge was not moved");
                resetBox();
                var startYOutside = initialBounds.bottomRight.y + detectionRadius + 1;
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: startYOutside }, { x: midPoint.x, y: DIV_HEIGHT });
                bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, midPoint.x, "new box was started at the drag start position");
                chai_1.assert.strictEqual(bounds.topLeft.y, startYOutside, "new box was started at the drag start position");
                chai_1.assert.strictEqual(bounds.bottomRight.x, midPoint.x, "new box was started at the drag start position");
                chai_1.assert.strictEqual(bounds.bottomRight.y, DIV_HEIGHT, "new box was started at the drag start position");
                div.remove();
            });
            it("doesn't dismiss on no-op resize", function () {
                dbl.resizable(true);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.topLeft.y }, { x: midPoint.x, y: initialBounds.topLeft.y });
                chai_1.assert.isTrue(dbl.boxVisible(), "box was not dismissed");
                chai_1.assert.deepEqual(dbl.bounds(), initialBounds, "bounds did not change");
                div.remove();
            });
            it("can't resize if hidden", function () {
                dbl.resizable(true);
                dbl.boxVisible(false);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: initialBounds.bottomRight.y }, { x: midPoint.x, y: DIV_HEIGHT });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, midPoint.x, "new box was started at the drag start position");
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.bottomRight.y, "new box was started at the drag start position");
                chai_1.assert.strictEqual(bounds.bottomRight.x, midPoint.x, "new box was ended at the drag end position");
                chai_1.assert.strictEqual(bounds.bottomRight.y, DIV_HEIGHT, "new box was ended at the drag end position");
                div.remove();
            });
            it("has resizable CSS classes only when it is enabled and resizable", function () {
                var xResizableClass = "x-resizable";
                var yResizableClass = "y-resizable";
                dbl.resizable(true);
                chai_1.assert.isTrue(dbl.hasClass(xResizableClass), "carries \"x-resizable\" class if resizable");
                chai_1.assert.isTrue(dbl.hasClass(yResizableClass), "carries \"y-resizable\" class if resizable");
                dbl.enabled(false);
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if resizable, but not enabled");
                chai_1.assert.isFalse(dbl.hasClass(yResizableClass), "does not carry \"y-resizable\" class if resizable, but not enabled");
                dbl.resizable(false);
                dbl.enabled(true);
                chai_1.assert.isFalse(dbl.hasClass(xResizableClass), "does not carry \"x-resizable\" class if enabled, but not resizable");
                chai_1.assert.isFalse(dbl.hasClass(yResizableClass), "does not carry \"y-resizable\" class if enabled, but not resizable");
                div.remove();
            });
        });
        describe("Moving", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var midPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
            var dragDistance = 10;
            var div;
            var dbl;
            var target;
            var initialBounds;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                dbl = new Plottable.Components.DragBoxLayer();
                dbl.renderTo(div);
                target = dbl.background();
                dbl.bounds({
                    topLeft: { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 },
                    bottomRight: { x: DIV_WIDTH * 3 / 4, y: DIV_HEIGHT * 3 / 4 },
                });
                dbl.boxVisible(true);
                initialBounds = dbl.bounds();
            });
            it("can get and set the movable property", function () {
                chai_1.assert.isFalse(dbl.movable(), "defaults to false");
                chai_1.assert.strictEqual(dbl.movable(true), dbl, "setter returns DragBoxLayer");
                chai_1.assert.isTrue(dbl.movable(), "can set to true");
                chai_1.assert.strictEqual(dbl.movable(false), dbl, "setter returns DragBoxLayer");
                chai_1.assert.isFalse(dbl.movable(), "can set back to false");
                div.remove();
            });
            it("can move left", function () {
                dbl.movable(true);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: midPoint.y }, { x: midPoint.x - dragDistance, y: midPoint.y });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x - dragDistance, "left edge moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x - dragDistance, "right edge moved");
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge did not move");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y, "bottom edge did not move");
                div.remove();
            });
            it("can move right", function () {
                dbl.movable(true);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: midPoint.y }, { x: midPoint.x + dragDistance, y: midPoint.y });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x + dragDistance, "left edge moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x + dragDistance, "right edge moved");
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge did not move");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y, "bottom edge did not move");
                div.remove();
            });
            it("can move up", function () {
                dbl.movable(true);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: midPoint.y }, { x: midPoint.x, y: midPoint.y - dragDistance });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge did not move");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x, "right edge did not move");
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y - dragDistance, "top edge moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y - dragDistance, "bottom edge moved");
                div.remove();
            });
            it("can move down", function () {
                dbl.movable(true);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: midPoint.y }, { x: midPoint.x, y: midPoint.y + dragDistance });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge did not move");
                chai_1.assert.strictEqual(bounds.bottomRight.x, initialBounds.bottomRight.x, "right edge did not move");
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y + dragDistance, "top edge moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y + dragDistance, "bottom edge moved");
                div.remove();
            });
            it("does not move if grabbed within detectionRadius while resizable", function () {
                dbl.movable(true);
                dbl.resizable(true);
                TestMethods.triggerFakeDragSequence(target, { x: initialBounds.bottomRight.x, y: midPoint.y }, { x: DIV_WIDTH, y: midPoint.y });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.y, initialBounds.topLeft.y, "top edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.y, initialBounds.bottomRight.y, "bottom edge was not moved");
                chai_1.assert.strictEqual(bounds.topLeft.x, initialBounds.topLeft.x, "left edge was not moved");
                chai_1.assert.strictEqual(bounds.bottomRight.x, DIV_WIDTH, "right edge was repositioned");
                div.remove();
            });
            it("doesn't dismiss on no-op move", function () {
                dbl.movable(true);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: midPoint.y }, { x: midPoint.x, y: midPoint.y });
                chai_1.assert.isTrue(dbl.boxVisible(), "box remains visible");
                chai_1.assert.deepEqual(dbl.bounds(), initialBounds, "bounds did not change");
                div.remove();
            });
            it("dismisses on click outside of box", function () {
                dbl.movable(true);
                var origin = { x: 0, y: 0 };
                TestMethods.triggerFakeDragSequence(target, origin, origin);
                chai_1.assert.isFalse(dbl.boxVisible(), "box is no longer visible");
                div.remove();
            });
            it("starts new box if hidden instead of moving", function () {
                dbl.movable(true);
                dbl.boxVisible(false);
                TestMethods.triggerFakeDragSequence(target, { x: midPoint.x, y: midPoint.y }, { x: midPoint.x, y: midPoint.y + dragDistance });
                var bounds = dbl.bounds();
                chai_1.assert.strictEqual(bounds.topLeft.x, midPoint.x, "new box was started at the drag start position (x)");
                chai_1.assert.strictEqual(bounds.topLeft.y, midPoint.y, "new box was started at the drag start position (y)");
                chai_1.assert.strictEqual(bounds.bottomRight.x, midPoint.x, "new box was ended at the drag end position (x)");
                chai_1.assert.strictEqual(bounds.bottomRight.y, midPoint.y + dragDistance, "new box was ended at the drag end position (y)");
                div.remove();
            });
            it("has movable CSS classes only when enabled and movable", function () {
                var movableClass = "movable";
                chai_1.assert.isFalse(dbl.hasClass(movableClass), "initially does not have \"movable\" CSS class");
                dbl.movable(true);
                chai_1.assert.isTrue(dbl.hasClass(movableClass), "carries \"movable\" class if movable");
                dbl.enabled(false);
                chai_1.assert.isFalse(dbl.hasClass(movableClass), "does not carry \"movable\" class if movable, but not enabled");
                dbl.movable(false);
                dbl.enabled(true);
                chai_1.assert.isFalse(dbl.hasClass(movableClass), "does not carry \"movable\" class if enabled, but not movable");
                div.remove();
            });
        });
    });
});
