"use strict";
var chai_1 = require("chai");
var Typesetter = require("typesettable");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var domUtils_1 = require("../../src/utils/domUtils");
describe("Labels", function () {
    var LABEL_CLASS = "label";
    var CLOSETO_REQUIRMENT;
    before(function () {
        CLOSETO_REQUIRMENT = window.Pixel_CloseTo_Requirement;
        // HACKHACK #2422: use an open source web font by default for Plottable
        window.Pixel_CloseTo_Requirement = 3;
    });
    after(function () {
        window.Pixel_CloseTo_Requirement = CLOSETO_REQUIRMENT;
    });
    describe("Label", function () {
        describe("Basic Usage", function () {
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv();
            });
            it("can update text after label is created", function () {
                var label = new Plottable.Components.Label("a");
                label.renderTo(div);
                chai_1.assert.strictEqual(label.content().select("text").text(), "a", "the text starts at the specified string");
                chai_1.assert.operator(label.height(), ">", 0, "height is positive for non-empty string");
                label.text("hello world");
                label.renderTo(div);
                chai_1.assert.strictEqual(label.content().select("text").text(), "hello world", "the label text updated properly");
                chai_1.assert.operator(label.height(), ">", 0, "height is positive for non-empty string");
                div.remove();
            });
            it("can change label angle after label is created", function () {
                var label = new Plottable.Components.Label("CHANGING ORIENTATION");
                label.renderTo(div);
                var content = label.content();
                var text = content.select("text");
                var bbox = Plottable.Utils.DOM.elementBBox(text);
                chai_1.assert.closeTo(bbox.height, label.height(), 1, "label is in horizontal position");
                label.angle(90);
                text = content.select("text");
                bbox = Plottable.Utils.DOM.elementBBox(text);
                TestMethods.assertBBoxInclusion(label.element(), text);
                chai_1.assert.closeTo(bbox.height, label.width(), window.Pixel_CloseTo_Requirement, "label is in vertical position");
                div.remove();
            });
            it("positions centered text in the middle of horizontal space", function () {
                var DIV_WIDTH = Plottable.Utils.DOM.elementWidth(div);
                var label = new Plottable.Components.Label("X").renderTo(div);
                var textTranslate = domUtils_1.getTranslateValues(label.content().select("g"));
                var eleLeft = parseFloat(label.element().style("left"));
                var textWidth = Plottable.Utils.DOM.elementBBox(label.content().select("text")).width;
                var midPoint = eleLeft + textTranslate[0] + textWidth / 2;
                chai_1.assert.closeTo(midPoint, DIV_WIDTH / 2, window.Pixel_CloseTo_Requirement, "label is centered");
                div.remove();
            });
            it("truncates text that is too long to be render", function () {
                var DIV_WIDTH = Plottable.Utils.DOM.elementWidth(div);
                var label = new Plottable.Components.Label("THIS LABEL IS SO LONG WHOEVER WROTE IT WAS PROBABLY DERANGED");
                label.renderTo(div);
                var content = label.content();
                var text = content.select("text");
                var bbox = Plottable.Utils.DOM.elementBBox(text);
                chai_1.assert.strictEqual(bbox.height, label.height(), "text height === label.minimumHeight()");
                chai_1.assert.operator(bbox.width, "<=", DIV_WIDTH, "the text is not wider than the SVG width");
                div.remove();
            });
            it("truncates text to empty string if space given is too small", function () {
                div.style("width", "5px");
                var label = new Plottable.Components.Label("Yeah, not gonna fit...");
                label.renderTo(div);
                var text = label.content().select("text");
                chai_1.assert.strictEqual(text.text(), "", "text was truncated to empty string");
                div.remove();
            });
            it("sets width to 0 if a label text is changed to empty string", function () {
                var label = new Plottable.Components.Label("foo");
                label.renderTo(div);
                label.text("");
                chai_1.assert.strictEqual(label.width(), 0, "width updated to 0");
                div.remove();
            });
            it("renders left-rotated text properly", function () {
                var str = "LEFT-ROTATED LABEL";
                var label = new Plottable.Components.Label(str, -90);
                label.renderTo(div);
                var content = label.content();
                var text = content.select("text");
                var textBBox = Plottable.Utils.DOM.elementBBox(text);
                TestMethods.assertBBoxInclusion(label.element(), text);
                chai_1.assert.closeTo(textBBox.height, label.width(), window.Pixel_CloseTo_Requirement, "text height");
                // OS/Browser specific kerning messes with typesetter's per-character measurement
                var widthFudge = str.length;
                chai_1.assert.closeTo(textBBox.width, label.height(), window.Pixel_CloseTo_Requirement + widthFudge, "text width");
                div.remove();
            });
            it("renders right-rotated text properly", function () {
                var str = "RIGHT-ROTATED LABEL";
                var label = new Plottable.Components.Label(str, 90);
                label.renderTo(div);
                var content = label.content();
                var text = content.select("text");
                var textBBox = Plottable.Utils.DOM.elementBBox(text);
                TestMethods.assertBBoxInclusion(label.element(), text);
                chai_1.assert.closeTo(textBBox.height, label.width(), window.Pixel_CloseTo_Requirement, "text height");
                // OS/Browser specific kerning messes with typesetter's per-character measurement
                var widthFudge = str.length;
                chai_1.assert.closeTo(textBBox.width, label.height(), window.Pixel_CloseTo_Requirement + widthFudge, "text width");
                div.remove();
            });
        });
        describe("Input validation", function () {
            it("only accepts strings as input for text", function () {
                var label = new Plottable.Components.Label();
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return label.text(23); }, Error, "Label.text() only takes strings as input", "error on inputing number");
                chai_1.assert.throws(function () { return label.text(new Date()); }, Error, "Label.text() only takes strings as input", "error on inputing Date");
                chai_1.assert.doesNotThrow(function () { return label.text("string"); }, Error, "text() accepts strings");
                chai_1.assert.strictEqual(label.text(null), "string", "text(null) returns text string");
            });
            it("converts angle to stay within -180 and 180", function () {
                var label360 = new Plottable.Components.Label("noScope", 360);
                chai_1.assert.strictEqual(label360.angle(), 0, "angles are converted to range [-180, 180] (360 -> 0)");
                var label270 = new Plottable.Components.Label("turnRight", 270);
                chai_1.assert.strictEqual(label270.angle(), -90, "angles are converted to range [-180, 180] (270 -> -90)");
                var labelNeg270 = new Plottable.Components.Label("turnRight", -270);
                chai_1.assert.strictEqual(labelNeg270.angle(), 90, "angles are converted to range [-180, 180] (-270 -> 90)");
            });
            it("throws error on invalid angles", function () {
                var badAngle = 10;
                chai_1.assert.throws(function () { return new Plottable.Components.Label("foo").angle(badAngle); }, Error, badAngle + " is not a valid angle for Label", "only accept -90/0/90 for angle");
                chai_1.assert.throws(function () { return new Plottable.Components.Label("foo", badAngle); }, Error, badAngle + " is not a valid angle for Label", "only accept -90/0/90 for angle");
            });
            it("errors on negative padding", function () {
                var testLabel = new Plottable.Components.Label("testing label");
                chai_1.assert.throws(function () { return testLabel.padding(-10); }, Error, "Cannot be less than 0", "error on negative input");
            });
        });
        describe("Padding", function () {
            var PADDING = 30;
            var div;
            var label;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                label = new Plottable.Components.Label("testing label").padding(PADDING);
            });
            it("adds padding for label accordingly under left alignment", function () {
                label.xAlignment("left").renderTo(div);
                var testTextRect = label.content().select("text").node().getBoundingClientRect();
                var elementRect = label.element().node().getBoundingClientRect();
                chai_1.assert.closeTo(testTextRect.left, elementRect.left + PADDING, window.Pixel_CloseTo_Requirement, "left difference by padding amount");
                div.remove();
            });
            it("adds padding for label accordingly under right alignment", function () {
                label.xAlignment("right").renderTo(div);
                var testTextRect = label.content().select("text").node().getBoundingClientRect();
                var elementRect = label.element().node().getBoundingClientRect();
                chai_1.assert.closeTo(testTextRect.right, elementRect.right - PADDING, window.Pixel_CloseTo_Requirement, "right difference by padding amount");
                div.remove();
            });
            it("adds padding for label accordingly under top alignment", function () {
                label.yAlignment("top").renderTo(div);
                var testTextRect = label.content().select("text").node().getBoundingClientRect();
                var elementRect = label.element().node().getBoundingClientRect();
                chai_1.assert.closeTo(testTextRect.top, elementRect.top + PADDING, window.Pixel_CloseTo_Requirement, "top difference by padding amount");
                div.remove();
            });
            it("adds padding for label accordingly under bottom alignment", function () {
                label.yAlignment("bottom").renderTo(div);
                var testTextRect = label.content().select("text").node().getBoundingClientRect();
                var elementRect = label.element().node().getBoundingClientRect();
                chai_1.assert.closeTo(testTextRect.bottom, elementRect.bottom - PADDING, window.Pixel_CloseTo_Requirement, "bottom difference by padding amount");
                div.remove();
            });
            it("puts space around the label", function () {
                label.renderTo(div);
                var context = new Typesetter.SvgContext(label.content().node());
                var measurer = new Typesetter.CacheMeasurer(context);
                var measure = measurer.measure("testing label");
                chai_1.assert.operator(label.width(), ">", measure.width, "padding increases size of the component");
                chai_1.assert.operator(label.width(), "<=", measure.width + 2 * PADDING, "width at most incorporates full padding amount");
                chai_1.assert.operator(label.height(), ">", measure.height, "padding increases size of the component");
                chai_1.assert.operator(label.height(), ">=", measure.height + 2 * PADDING, "height at most incorporates full padding amount");
                div.remove();
            });
        });
    });
    describe("TitleLabel", function () {
        it("has appropriate css class", function () {
            var label = new Plottable.Components.TitleLabel("A CHART TITLE");
            var div = TestMethods.generateDiv();
            label.renderTo(div);
            chai_1.assert.isTrue(label.hasClass(LABEL_CLASS), "element has label css class");
            chai_1.assert.isTrue(label.hasClass(Plottable.Components.TitleLabel.TITLE_LABEL_CLASS), "element has title-label css class");
            div.remove();
        });
    });
    describe("AxisLabel", function () {
        it("has appropriate css class", function () {
            var label = new Plottable.Components.AxisLabel("Axis Label");
            var div = TestMethods.generateDiv();
            label.renderTo(div);
            chai_1.assert.isTrue(label.hasClass(LABEL_CLASS), "element has label css class");
            chai_1.assert.isTrue(label.hasClass(Plottable.Components.AxisLabel.AXIS_LABEL_CLASS), "title element has axis-label css class");
            div.remove();
        });
    });
});
