"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("SelectionBoxLayer", function () {
    describe("Basic Usage", function () {
        var DIV_WIDTH = 500;
        var DIV_HEIGHT = 500;
        var SELECTION_BOX_CLASSNAME = ".selection-box";
        var div;
        var sbl;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            sbl = new Plottable.Components.SelectionBoxLayer();
        });
        it("can set the boxVisible property", function () {
            chai_1.assert.strictEqual(sbl.boxVisible(), false, "The box is not visible by default");
            sbl.renderTo(div);
            chai_1.assert.strictEqual(sbl.boxVisible(), false, "The box is not visible by default after rendering to div");
            chai_1.assert.strictEqual(sbl.boxVisible(true), sbl, "Setting the boxVisible attribute returns the selection box layer");
            chai_1.assert.strictEqual(sbl.boxVisible(), true, "Setting the boxVisible to true attribute works");
            sbl.boxVisible(false);
            chai_1.assert.strictEqual(sbl.boxVisible(), false, "Setting the boxVisible attribute to false works");
            sbl.destroy();
            div.remove();
        });
        it("renders the box in accordance to boxVisible property", function () {
            sbl.renderTo(div);
            var selectionBox = div.select(SELECTION_BOX_CLASSNAME);
            chai_1.assert.isTrue(selectionBox.empty(), "initilizes without box in DOM");
            sbl.boxVisible(true);
            selectionBox = div.select(SELECTION_BOX_CLASSNAME);
            chai_1.assert.isFalse(selectionBox.empty(), "box is inserted in DOM when showing");
            sbl.boxVisible(false);
            selectionBox = div.select(SELECTION_BOX_CLASSNAME);
            chai_1.assert.isTrue(selectionBox.empty(), "box is removed from DOM when not showing");
            sbl.destroy();
            div.remove();
        });
        it("turns _overflowHidden on", function () {
            sbl.renderTo(div);
            chai_1.assert.isTrue(sbl._overflowHidden, "overflowHidden is enabled");
            div.remove();
        });
        it("can set the bounds property", function () {
            var defaultBounds = sbl.bounds();
            chai_1.assert.strictEqual(defaultBounds.topLeft.x, 0, "top-left bound is correct (x)");
            chai_1.assert.strictEqual(defaultBounds.topLeft.y, 0, "top-left bound is correct (y)");
            chai_1.assert.strictEqual(defaultBounds.bottomRight.x, 0, "bottom-right bound is correct (x)");
            chai_1.assert.strictEqual(defaultBounds.bottomRight.y, 0, "bottom-right bound is correct (y)");
            var topLeft = {
                x: 100,
                y: 100,
            };
            var bottomRight = {
                x: 300,
                y: 300,
            };
            chai_1.assert.doesNotThrow(function () { return sbl.bounds({
                topLeft: topLeft,
                bottomRight: bottomRight,
            }); }, Error, "can set bounds before anchoring");
            sbl.boxVisible(true);
            sbl.renderTo(div);
            assertCorrectRendering(topLeft, bottomRight, "rendered correctly");
            var queriedBounds = sbl.bounds();
            chai_1.assert.deepEqual(queriedBounds.topLeft, topLeft, "returns correct top-left position");
            chai_1.assert.deepEqual(queriedBounds.bottomRight, bottomRight, "returns correct bottom-right position");
            chai_1.assert.strictEqual(sbl.bounds({
                topLeft: bottomRight,
                bottomRight: topLeft,
            }), sbl, "Setting the bounds property returns the selection box layer");
            assertCorrectRendering(topLeft, bottomRight, "rendered correctly with reversed bounds");
            queriedBounds = sbl.bounds();
            chai_1.assert.deepEqual(queriedBounds.topLeft, topLeft, "returns correct top-left position");
            chai_1.assert.deepEqual(queriedBounds.bottomRight, bottomRight, "returns correct bottom-right position");
            sbl.destroy();
            div.remove();
            function assertCorrectRendering(expectedTL, expectedBR, msg) {
                var selectionBox = div.select(SELECTION_BOX_CLASSNAME);
                var bbox = Plottable.Utils.DOM.elementBBox(selectionBox);
                chai_1.assert.strictEqual(bbox.x, expectedTL.x, msg + " (x-origin)");
                chai_1.assert.strictEqual(bbox.x, expectedTL.y, msg + " (y-origin)");
                chai_1.assert.strictEqual(bbox.width, expectedBR.x - expectedTL.x, msg + " (width)");
                chai_1.assert.strictEqual(bbox.height, expectedBR.y - expectedTL.y, msg + " (height)");
            }
        });
        it("throws an error if the bounds have not been set properly before rendering", function () {
            var topLeft = {
                x: "a",
                y: "b",
            };
            var bottomRight = {
                x: 300,
                y: 300,
            };
            sbl.bounds({
                topLeft: topLeft,
                bottomRight: bottomRight,
            });
            sbl.boxVisible(true);
            sbl.anchor(div);
            // HACKHACK #2614: chai-assert.d.ts has the wrong signature
            chai_1.assert.throws(function () { return sbl.renderImmediately(); }, "bounds have not been properly set", "cannot set invalid bounds");
            sbl.destroy();
            div.remove();
        });
        it("uses the pixel values sides if they were set last", function () {
            var xExtent = [0, 100];
            var yExtent = [0, 100];
            sbl.xExtent(xExtent);
            sbl.yExtent(yExtent);
            chai_1.assert.deepEqual(sbl.xExtent(), xExtent, "x extent set");
            chai_1.assert.deepEqual(sbl.yExtent(), yExtent, "y extent set");
            var topLeft = {
                x: 100,
                y: 100,
            };
            var bottomRight = {
                x: 300,
                y: 300,
            };
            chai_1.assert.doesNotThrow(function () { return sbl.bounds({
                topLeft: topLeft,
                bottomRight: bottomRight,
            }); }, Error, "can set bounds before anchoring");
            var queriedBounds = sbl.bounds();
            chai_1.assert.deepEqual(queriedBounds.topLeft, topLeft, "returns correct top-left position");
            chai_1.assert.deepEqual(queriedBounds.bottomRight, bottomRight, "returns correct bottom-right position");
            sbl.destroy();
            div.remove();
        });
        it("has an effective size of 0, but will occupy all offered space", function () {
            var request = sbl.requestedSpace(400, 400);
            TestMethods.verifySpaceRequest(request, 0, 0, "does not request any space");
            chai_1.assert.isTrue(sbl.fixedWidth(), "fixed width");
            chai_1.assert.isTrue(sbl.fixedHeight(), "fixed height");
            sbl.destroy();
            div.remove();
        });
    });
    describe("X scale binding", function () {
        var DIV_WIDTH = 500;
        var DIV_HEIGHT = 500;
        var SELECTION_AREA_CLASSNAME = ".selection-area";
        var div;
        var sbl;
        var xScale;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            sbl = new Plottable.Components.SelectionBoxLayer();
            xScale = new Plottable.Scales.Linear();
            xScale.domain([0, 2000]);
            xScale.range([0, DIV_HEIGHT]);
        });
        it("can set the xScale property", function () {
            chai_1.assert.isUndefined(sbl.xScale(), "no xScale is specified by default");
            chai_1.assert.strictEqual(sbl.xScale(xScale), sbl, "setting the xScale returns the selection box layer");
            chai_1.assert.strictEqual(sbl.xScale(), xScale, "The getter returns the correct scale");
            sbl.renderTo(div);
            sbl.xExtent([0, 1000]);
            sbl.boxVisible(true);
            var selectionBox = div.select(SELECTION_AREA_CLASSNAME);
            chai_1.assert.strictEqual(selectionBox.attr("x"), "0", "box starts at left edge");
            xScale.domain([-1000, 1000]);
            selectionBox = div.select(SELECTION_AREA_CLASSNAME);
            chai_1.assert.strictEqual(selectionBox.attr("x"), "250", "domain change moves box");
            sbl.destroy();
            div.remove();
        });
        it("can set the data values of the left and right sides directly", function () {
            sbl.boxVisible(true);
            sbl.xScale(xScale);
            chai_1.assert.deepEqual(sbl.xExtent(), [undefined, undefined], "xExtent is not set");
            var xExtent = [100, 250];
            chai_1.assert.strictEqual(sbl.xExtent(xExtent), sbl, "returns calling object");
            chai_1.assert.deepEqual(sbl.xExtent(), xExtent, "xExtent set");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.x, xScale.scale(xExtent[0]), "left pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.x, xScale.scale(xExtent[1]), "right pixel position adjusts accordingly");
            sbl.renderTo(div);
            var box = sbl.content().select(SELECTION_AREA_CLASSNAME);
            chai_1.assert.closeTo(TestMethods.numAttr(box, "x"), xScale.scale(xExtent[0]), window.Pixel_CloseTo_Requirement, "box x attribute starts at start of x extent");
            chai_1.assert.closeTo(TestMethods.numAttr(box, "width"), xScale.scale(xExtent[1]) - xScale.scale(xExtent[0]), window.Pixel_CloseTo_Requirement, "box width attribute extends for width of x extent");
            sbl.destroy();
            div.remove();
        });
        it("uses the data values for the left and right sides if they were set last", function () {
            sbl.xScale(xScale);
            var bounds = {
                topLeft: {
                    x: 100,
                    y: 100,
                },
                bottomRight: {
                    x: 200,
                    y: 200,
                },
            };
            sbl.bounds(bounds);
            chai_1.assert.deepEqual(sbl.bounds(), bounds, "bounds set");
            var xExtent = [100, 250];
            chai_1.assert.strictEqual(sbl.xExtent(xExtent), sbl, "returns calling object");
            chai_1.assert.deepEqual(sbl.xExtent(), xExtent, "xExtent set");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.x, xScale.scale(xExtent[0]), "left pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.x, xScale.scale(xExtent[1]), "right pixel position adjusts accordingly");
            sbl.destroy();
            div.remove();
        });
        it("updates left and right edge pixel positions if in VALUE mode and xScale is switched", function () {
            sbl.xScale(xScale);
            var xExtent = [100, 250];
            sbl.xExtent(xExtent);
            var leftPosition = sbl.bounds().topLeft.x;
            var rightPosition = sbl.bounds().bottomRight.x;
            chai_1.assert.strictEqual(leftPosition, xScale.scale(xExtent[0]), "left pixel position adjusts accordingly");
            chai_1.assert.strictEqual(rightPosition, xScale.scale(xExtent[1]), "right pixel position adjusts accordingly");
            var xScale2 = new Plottable.Scales.ModifiedLog();
            xScale2.domain([0, 1000]);
            xScale2.range([0, DIV_HEIGHT]);
            sbl.xScale(xScale2);
            chai_1.assert.notStrictEqual(sbl.bounds().topLeft.x, leftPosition, "left pixel position changed");
            chai_1.assert.notStrictEqual(sbl.bounds().bottomRight.x, rightPosition, "right pixel position changed");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.x, xScale2.scale(xExtent[0]), "left pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.x, xScale2.scale(xExtent[1]), "right pixel position adjusts accordingly");
            sbl.destroy();
            div.remove();
        });
        it("updates left and right edge pixel positions if in VALUE mode and xScale updates", function () {
            sbl.xScale(xScale);
            var xExtent = [100, 250];
            sbl.xExtent(xExtent);
            var leftPosition = sbl.bounds().topLeft.x;
            var rightPosition = sbl.bounds().bottomRight.x;
            chai_1.assert.strictEqual(leftPosition, xScale.scale(xExtent[0]), "left pixel position adjusts accordingly");
            chai_1.assert.strictEqual(rightPosition, xScale.scale(xExtent[1]), "right pixel position adjusts accordingly");
            xScale.domain([0, 1000]);
            chai_1.assert.notStrictEqual(sbl.bounds().topLeft.x, leftPosition, "left pixel position changed");
            chai_1.assert.notStrictEqual(sbl.bounds().bottomRight.x, rightPosition, "right pixel position changed");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.x, xScale.scale(xExtent[0]), "left pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.x, xScale.scale(xExtent[1]), "right pixel position adjusts accordingly");
            sbl.destroy();
            div.remove();
        });
    });
    describe("Y scale binding", function () {
        var DIV_WIDTH = 500;
        var DIV_HEIGHT = 500;
        var SELECTION_AREA_CLASSNAME = ".selection-area";
        var div;
        var sbl;
        var yScale;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            sbl = new Plottable.Components.SelectionBoxLayer();
            yScale = new Plottable.Scales.Linear();
            yScale.domain([0, 2000]);
            yScale.range([0, DIV_HEIGHT]);
        });
        it("can set the yScale property", function () {
            chai_1.assert.isUndefined(sbl.yScale(), "no yScale is specified by default");
            chai_1.assert.strictEqual(sbl.yScale(yScale), sbl, "setting the yScale returns the selection box layer");
            chai_1.assert.strictEqual(sbl.yScale(), yScale, "The getter returns the correct scale");
            sbl.renderTo(div);
            sbl.yExtent([0, 1000]);
            sbl.boxVisible(true);
            var selectionBox = div.select(SELECTION_AREA_CLASSNAME);
            chai_1.assert.strictEqual(selectionBox.attr("y"), "0", "box starts at top edge");
            yScale.domain([-1000, 1000]);
            selectionBox = div.select(SELECTION_AREA_CLASSNAME);
            chai_1.assert.strictEqual(selectionBox.attr("y"), "250", "domain change moves box");
            sbl.destroy();
            div.remove();
        });
        it("can set the data values of the top and bottom sides directly", function () {
            sbl.boxVisible(true);
            sbl.yScale(yScale);
            chai_1.assert.deepEqual(sbl.yExtent(), [undefined, undefined], "yExtent is not set");
            var yExtent = [0, 300];
            chai_1.assert.strictEqual(sbl.yExtent(yExtent), sbl, "returns calling object");
            chai_1.assert.deepEqual(sbl.yExtent(), yExtent, "yExtent set");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.y, yScale.scale(yExtent[0]), "top pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.y, yScale.scale(yExtent[1]), "bottom pixel position adjusts accordingly");
            sbl.renderTo(div);
            var box = sbl.content().select(SELECTION_AREA_CLASSNAME);
            chai_1.assert.closeTo(TestMethods.numAttr(box, "y"), yScale.scale(yExtent[0]), window.Pixel_CloseTo_Requirement, "box y attribute starts at start of y extent");
            chai_1.assert.closeTo(TestMethods.numAttr(box, "height"), yScale.scale(yExtent[1]) - yScale.scale(yExtent[0]), window.Pixel_CloseTo_Requirement, "box height attribute extends for width of y extent");
            sbl.destroy();
            div.remove();
        });
        it("uses the data values for the top and bottom sides if they were set last", function () {
            sbl.yScale(yScale);
            var bounds = {
                topLeft: {
                    x: 100,
                    y: 100,
                },
                bottomRight: {
                    x: 200,
                    y: 200,
                },
            };
            sbl.bounds(bounds);
            chai_1.assert.deepEqual(sbl.bounds(), bounds, "bounds set");
            var yExtent = [100, 250];
            chai_1.assert.strictEqual(sbl.yExtent(yExtent), sbl, "returns calling object");
            chai_1.assert.deepEqual(sbl.yExtent(), yExtent, "yExtent set");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.y, yScale.scale(yExtent[0]), "top pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.y, yScale.scale(yExtent[1]), "bottom pixel position adjusts accordingly");
            sbl.destroy();
            div.remove();
        });
        it("updates top and bottom edge pixel positions if in VALUE mode and yScale is switched", function () {
            sbl.yScale(yScale);
            var yExtent = [100, 250];
            sbl.yExtent(yExtent);
            var topPosition = sbl.bounds().topLeft.y;
            var bottomPosition = sbl.bounds().bottomRight.y;
            chai_1.assert.strictEqual(topPosition, yScale.scale(yExtent[0]), "top pixel position adjusts accordingly");
            chai_1.assert.strictEqual(bottomPosition, yScale.scale(yExtent[1]), "bottom pixel position adjusts accordingly");
            var yScale2 = new Plottable.Scales.ModifiedLog();
            yScale2.domain([0, 1000]);
            yScale2.range([0, DIV_HEIGHT]);
            sbl.yScale(yScale2);
            chai_1.assert.notStrictEqual(sbl.bounds().topLeft.y, topPosition, "top pixel position changed");
            chai_1.assert.notStrictEqual(sbl.bounds().bottomRight.y, bottomPosition, "bottom pixel position changed");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.y, yScale2.scale(yExtent[0]), "top pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.y, yScale2.scale(yExtent[1]), "bottom pixel position adjusts accordingly");
            sbl.destroy();
            div.remove();
        });
        it("updates top and bottom edge pixel positions if in VALUE mode and yScale updates", function () {
            sbl.yScale(yScale);
            var yExtent = [100, 250];
            sbl.yExtent(yExtent);
            var topPosition = sbl.bounds().topLeft.y;
            var bottomPosition = sbl.bounds().bottomRight.y;
            chai_1.assert.strictEqual(topPosition, yScale.scale(yExtent[0]), "top pixel position adjusts accordingly");
            chai_1.assert.strictEqual(bottomPosition, yScale.scale(yExtent[1]), "bottom pixel position adjusts accordingly");
            yScale.domain([0, 1000]);
            chai_1.assert.notStrictEqual(sbl.bounds().topLeft.y, topPosition, "top pixel position changed");
            chai_1.assert.notStrictEqual(sbl.bounds().bottomRight.y, bottomPosition, "bottom pixel position changed");
            chai_1.assert.strictEqual(sbl.bounds().topLeft.y, yScale.scale(yExtent[0]), "top pixel position adjusts accordingly");
            chai_1.assert.strictEqual(sbl.bounds().bottomRight.y, yScale.scale(yExtent[1]), "bottom pixel position adjusts accordingly");
            sbl.destroy();
            div.remove();
        });
    });
});
