"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var Mocks = require("../mocks");
var TestMethods = require("../testMethods");
describe("Tables", function () {
    function assertTableRows(table, expectedRows, message) {
        expectedRows.forEach(function (row, rowIndex) {
            row.forEach(function (component, columnIndex) {
                chai_1.assert.strictEqual(table.componentAt(rowIndex, columnIndex), component, message + " contains the correct entry at [" + rowIndex + ", " + columnIndex + "]");
            });
        });
    }
    describe("constructor behavior", function () {
        it("adds the \"table\" CSS class", function () {
            var table = new Plottable.Components.Table();
            chai_1.assert.isTrue(table.hasClass("table"));
        });
        it("can take a 2-D array of Components in the constructor", function () {
            var c00 = new Plottable.Component();
            var c11 = new Plottable.Component();
            var rows = [
                [c00, null],
                [null, c11],
            ];
            var table = new Plottable.Components.Table(rows);
            assertTableRows(table, rows, "constructor initialized Table correctly");
        });
    });
    describe("checking Table contents", function () {
        it("can check if a given Component is in the Table", function () {
            var c0 = new Plottable.Component();
            var table = new Plottable.Components.Table([[c0]]);
            chai_1.assert.isTrue(table.has(c0), "correctly checks that Component is in the Table");
            table.remove(c0);
            chai_1.assert.isFalse(table.has(c0), "correctly checks that Component is no longer in the Table");
            table.add(c0, 1, 1);
            chai_1.assert.isTrue(table.has(c0), "correctly checks that Component is in the Table again");
        });
        it("can retrieve the Component at a given row, column index", function () {
            var c00 = new Plottable.Component();
            var c01 = new Plottable.Component();
            var c10 = new Plottable.Component();
            var c11 = new Plottable.Component();
            var table = new Plottable.Components.Table([
                [c00, c01],
                [c10, c11],
            ]);
            chai_1.assert.strictEqual(table.componentAt(0, 0), c00, "retrieves the Component at [0, 0]");
            chai_1.assert.strictEqual(table.componentAt(0, 1), c01, "retrieves the Component at [0, 1]");
            chai_1.assert.strictEqual(table.componentAt(1, 0), c10, "retrieves the Component at [1, 0]");
            chai_1.assert.strictEqual(table.componentAt(1, 1), c11, "retrieves the Component at [1, 1]");
        });
        it("returns null when no Component exists at the specified row, column index", function () {
            var c00 = new Plottable.Component();
            var c11 = new Plottable.Component();
            var table = new Plottable.Components.Table();
            table.add(c00, 0, 0);
            table.add(c11, 1, 1);
            chai_1.assert.isNull(table.componentAt(0, 1), "returns null if an empty cell is queried");
            chai_1.assert.isNull(table.componentAt(-1, 0), "returns null if a negative row index is passed in");
            chai_1.assert.isNull(table.componentAt(0, -1), "returns null if a negative column index is passed in");
            chai_1.assert.isNull(table.componentAt(9001, 0), "returns null if a row index larger than the number of rows is passed in");
            chai_1.assert.isNull(table.componentAt(0, 9001), "returns null if a column index larger than the number of columns is passed in");
        });
    });
    describe("adding Components", function () {
        it("adds the Component and pads out other empty cells with null", function () {
            var table = new Plottable.Components.Table();
            var c11 = new Plottable.Component();
            table.add(c11, 1, 1);
            chai_1.assert.strictEqual(table.componentAt(1, 1), c11, "Component was added at the correct position");
            chai_1.assert.isNull(table.componentAt(0, 0), "Table padded [0, 0] with null");
            chai_1.assert.isNull(table.componentAt(0, 1), "Table padded [0, 1] with null");
            chai_1.assert.isNull(table.componentAt(1, 0), "Table padded [1, 0] with null");
        });
        it("throws an Error on trying to add a Component to an occupied cell", function () {
            var c1 = new Plottable.Component();
            var table = new Plottable.Components.Table([[c1]]);
            var c2 = new Plottable.Component();
            chai_1.assert.throws(function () { return table.add(c2, 0, 0); }, Error, "occupied");
        });
        it("throws an Error on trying to add null to the Table", function () {
            var table = new Plottable.Components.Table();
            chai_1.assert.throws(function () { return table.add(null, 0, 0); }, "Cannot add null to a table cell");
        });
        it("detaches a Component from its previous location before adding it", function () {
            var component = new Plottable.Component();
            var div = TestMethods.generateDiv();
            component.renderTo(div);
            var table = new Plottable.Components.Table();
            table.add(component, 0, 0);
            chai_1.assert.isFalse(div.node().hasChildNodes(), "Component was detach()-ed");
            div.remove();
        });
    });
    describe("removing Components", function () {
        var c00;
        var c01;
        var c10;
        var c11;
        var table;
        beforeEach(function () {
            c00 = new Plottable.Component();
            c01 = new Plottable.Component();
            c10 = new Plottable.Component();
            c11 = new Plottable.Component();
        });
        it("removes the specified Component", function () {
            var tableRows = [
                [c00, c01],
                [c10, c11],
            ];
            table = new Plottable.Components.Table(tableRows);
            table.remove(c11);
            var expectedRows = [
                [c00, c01],
                [c10, null],
            ];
            assertTableRows(table, expectedRows, "the requested element was removed");
            chai_1.assert.isNull(c11.parent(), "Component disconnected from the Table");
        });
        it("does nothing when component is not found", function () {
            var tableRows = [
                [c00, c01],
                [c10, c11],
            ];
            table = new Plottable.Components.Table(tableRows);
            var expectedRows = tableRows;
            var notInTable = new Plottable.Component();
            table.remove(notInTable);
            assertTableRows(table, expectedRows, "removing a nonexistent Component does not affect the table");
        });
        it("has no further effect when called a second time with the same Component", function () {
            var tableRows = [
                [c00, c01],
                [c10, c11],
            ];
            table = new Plottable.Components.Table(tableRows);
            var expectedRows = [
                [null, c01],
                [c10, c11],
            ];
            table.remove(c00);
            assertTableRows(table, expectedRows, "Component was removed");
            table.remove(c00);
            assertTableRows(table, expectedRows, "removing Component again has no further effect");
        });
        it("removes a Component from the Table if the Component becomes detached", function () {
            table = new Plottable.Components.Table([[c00]]);
            c00.detach();
            chai_1.assert.isNull(table.componentAt(0, 0), "calling detach() on the Component removes it from the Table");
            chai_1.assert.isNull(c00.parent(), "Component disconnected from the Table");
        });
    });
    describe("requesting space", function () {
        it("is fixed-width if all columns contain only fixed-width Components, non-fixed otherwise", function () {
            var fixedNullsTable = new Plottable.Components.Table([
                [new Mocks.FixedSizeComponent(), null],
                [null, new Mocks.FixedSizeComponent()],
            ]);
            chai_1.assert.isTrue(fixedNullsTable.fixedWidth(), "width is fixed if all columns contain fixed-width Components or null");
            var notAllFixedTable = new Plottable.Components.Table([
                [new Mocks.FixedSizeComponent(), new Mocks.FixedSizeComponent()],
                [new Plottable.Component(), null],
            ]);
            chai_1.assert.isFalse(notAllFixedTable.fixedWidth(), "width is not fixed if any column contains a non-fixed-width Component");
        });
        it("is fixed-height if all rows contain only fixed-height Components, non-fixed otherwise", function () {
            var fixedNullsTable = new Plottable.Components.Table([
                [new Mocks.FixedSizeComponent(), null],
                [null, new Mocks.FixedSizeComponent()],
            ]);
            chai_1.assert.isTrue(fixedNullsTable.fixedHeight(), "height is fixed if all columns contain fixed-height Components or null");
            var notAllFixedTable = new Plottable.Components.Table([
                [new Mocks.FixedSizeComponent(), new Plottable.Component()],
                [new Mocks.FixedSizeComponent(), null],
            ]);
            chai_1.assert.isFalse(notAllFixedTable.fixedHeight(), "height is not fixed if any row contains a non-fixed-height Component");
        });
        it("requests enough space for largest fixed-size Component in each row and column", function () {
            var BIG_COMPONENT_SIZE = 50;
            var SMALL_COMPONENT_SIZE = 20;
            var unfixedComponent = new Plottable.Component();
            var bigFixedComponent01 = new Mocks.FixedSizeComponent(BIG_COMPONENT_SIZE, BIG_COMPONENT_SIZE);
            var bigFixedComponent10 = new Mocks.FixedSizeComponent(BIG_COMPONENT_SIZE, BIG_COMPONENT_SIZE);
            var smallFixedComponent = new Mocks.FixedSizeComponent(SMALL_COMPONENT_SIZE, SMALL_COMPONENT_SIZE);
            var table = new Plottable.Components.Table([
                [unfixedComponent, bigFixedComponent01],
                [bigFixedComponent10, smallFixedComponent],
            ]);
            var expectedMinWidth = 2 * BIG_COMPONENT_SIZE;
            var expectedMinHeight = 2 * BIG_COMPONENT_SIZE;
            var constrainedSpaceRequest = table.requestedSpace(0, 0);
            TestMethods.verifySpaceRequest(constrainedSpaceRequest, expectedMinWidth, expectedMinHeight, "requests enough space for all fixed-size Components when space is constrained");
            var exactlyEnoughSpaceRequest = table.requestedSpace(expectedMinWidth, expectedMinHeight);
            TestMethods.verifySpaceRequest(exactlyEnoughSpaceRequest, expectedMinWidth, expectedMinHeight, "requests enough space for all fixed-size Components when given exactly enough space");
            var extraSpaceRequest = table.requestedSpace(2 * expectedMinWidth, 2 * expectedMinHeight);
            TestMethods.verifySpaceRequest(extraSpaceRequest, expectedMinWidth, expectedMinHeight, "requests enough space for all fixed-size Components when given extra space");
        });
    });
    describe("error checking on padding and weights", function () {
        var table;
        beforeEach(function () {
            table = new Plottable.Components.Table([
                [new Plottable.Component(), new Plottable.Component()],
                [new Plottable.Component(), new Plottable.Component()],
            ]);
        });
        afterEach(function () {
            table.destroy();
        });
        it("rejects invalid input to rowPadding", function () {
            // HACKHACK #2661: Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return table.rowPadding(-1); }, Error, "rowPadding must be a non-negative finite value", "rowPadding rejects negative numbers");
            chai_1.assert.throws(function () { return table.rowPadding(Infinity); }, Error, "rowPadding must be a non-negative finite value", "rowPadding rejects Infinity");
            chai_1.assert.throws(function () { return table.rowPadding(NaN); }, Error, "rowPadding must be a non-negative finite value", "rowPadding rejects NaN");
            chai_1.assert.throws(function () { return table.rowPadding("4"); }, Error, "rowPadding must be a non-negative finite value", "rowPadding rejects string numbers");
        });
        it("rejects invalid input to columnPadding", function () {
            // HACKHACK #2661: Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return table.columnPadding(-1); }, Error, "columnPadding must be a non-negative finite value", "columnPadding rejects negative numbers");
            chai_1.assert.throws(function () { return table.columnPadding(Infinity); }, Error, "columnPadding must be a non-negative finite value", "columnPadding rejects Infinity");
            chai_1.assert.throws(function () { return table.columnPadding(NaN); }, Error, "columnPadding must be a non-negative finite value", "columnPadding rejects NaN");
            chai_1.assert.throws(function () { return table.columnPadding("4"); }, Error, "columnPadding must be a non-negative finite value", "columnPadding rejects string numbers");
        });
        it("rejects invalid input to rowWeight", function () {
            // HACKHACK #2661: Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return table.rowWeight(0, -1); }, Error, "rowWeight must be a non-negative finite value", "rowWeight rejects negative numbers");
            chai_1.assert.throws(function () { return table.rowWeight(1, Infinity); }, Error, "rowWeight must be a non-negative finite value", "rowWeight rejects Infinity");
            chai_1.assert.throws(function () { return table.rowWeight(0, NaN); }, Error, "rowWeight must be a non-negative finite value", "rowWeight rejects NaN");
            chai_1.assert.throws(function () { return table.rowWeight(1, "4"); }, Error, "rowWeight must be a non-negative finite value", "rowWeight rejects string numbers");
        });
        it("rejects invalid input to columnWeight", function () {
            // HACKHACK #2661: Cannot assert errors being thrown with description
            chai_1.assert.throws(function () { return table.columnWeight(1, -1); }, Error, "columnWeight must be a non-negative finite value", "columnWeight rejects negative numbers");
            chai_1.assert.throws(function () { return table.columnWeight(0, Infinity); }, Error, "columnWeight must be a non-negative finite value", "columnWeight rejects Infinity");
            chai_1.assert.throws(function () { return table.columnWeight(1, NaN); }, Error, "columnWeight must be a non-negative finite value", "columnWeight rejects NaN");
            chai_1.assert.throws(function () { return table.columnWeight(0, "4"); }, Error, "columnWeight must be a non-negative finite value", "columnWeight rejects string numbers");
        });
    });
    describe("layout of constituent Components", function () {
        var FIXED_COMPONENT_SIZE = 50;
        function verifyOrigins(rows, rowPadding, columnPadding) {
            if (rowPadding === void 0) { rowPadding = 0; }
            if (columnPadding === void 0) { columnPadding = 0; }
            var expectedOrigin = {
                x: 0,
                y: 0,
            };
            rows.forEach(function (row, r) {
                var maxHeight = 0;
                expectedOrigin.x = 0;
                row.forEach(function (component, c) {
                    TestMethods.assertPointsClose(component.origin(), expectedOrigin, 0, "Component at [" + r + ", " + c + "] has the correct origin");
                    expectedOrigin.x += component.width() + columnPadding;
                    maxHeight = maxHeight > component.height() ? maxHeight : component.height();
                });
                expectedOrigin.y += maxHeight + rowPadding;
            });
        }
        it("divides available width evenly between non-fixed-width Components", function () {
            var div = TestMethods.generateDiv();
            var component1 = new Plottable.Component();
            var component2 = new Plottable.Component();
            var tableRows = [[component1, component2]];
            var table = new Plottable.Components.Table(tableRows);
            table.renderTo(div);
            var twoColumnExpectedWidth = Plottable.Utils.DOM.elementWidth(div) / 2;
            chai_1.assert.strictEqual(component1.width(), twoColumnExpectedWidth, "first Component received half the available width");
            chai_1.assert.strictEqual(component2.width(), twoColumnExpectedWidth, "second Component received half the available width");
            verifyOrigins(tableRows);
            var component3 = new Plottable.Component();
            table.add(component3, 0, 2);
            tableRows[0].push(component3);
            var threeColumnExpectedWidth = Plottable.Utils.DOM.elementWidth(div) / 3;
            chai_1.assert.strictEqual(component1.width(), threeColumnExpectedWidth, "first Component received one-third of the available width");
            chai_1.assert.strictEqual(component2.width(), threeColumnExpectedWidth, "second Component received one-third of the available width");
            chai_1.assert.strictEqual(component3.width(), threeColumnExpectedWidth, "third Component received one-third of the available width");
            verifyOrigins(tableRows);
            table.destroy();
            div.remove();
        });
        it("gives width to fixed-width Components, then divides remainder between non-fixed-width Components", function () {
            var div = TestMethods.generateDiv();
            var fixedSizeComponent = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var unfixedComponent1 = new Plottable.Component();
            var unfixedComponent2 = new Plottable.Component();
            var table = new Plottable.Components.Table([[fixedSizeComponent, unfixedComponent1, unfixedComponent2]]);
            table.renderTo(div);
            var expectedUnfixedWidth = (Plottable.Utils.DOM.elementWidth(div) - FIXED_COMPONENT_SIZE) / 2;
            chai_1.assert.strictEqual(unfixedComponent1.width(), expectedUnfixedWidth, "first non-fixed-width Component received half the remaining width");
            chai_1.assert.strictEqual(unfixedComponent2.width(), expectedUnfixedWidth, "second non-fixed-width Component received half the remaining width");
            table.destroy();
            div.remove();
        });
        it("divides available height evenly between non-fixed-height Components", function () {
            var div = TestMethods.generateDiv();
            var component1 = new Plottable.Component();
            var component2 = new Plottable.Component();
            var tableRows = [
                [component1],
                [component2],
            ];
            var table = new Plottable.Components.Table(tableRows);
            table.renderTo(div);
            var twoRowExpectedHeight = Plottable.Utils.DOM.elementHeight(div) / 2;
            chai_1.assert.strictEqual(component1.height(), twoRowExpectedHeight, "first Component received half the available height");
            chai_1.assert.strictEqual(component2.height(), twoRowExpectedHeight, "second Component received half the available height");
            verifyOrigins(tableRows);
            var component3 = new Plottable.Component();
            table.add(component3, 2, 0);
            tableRows.push([component3]);
            var threeRowExpectedHeight = Plottable.Utils.DOM.elementHeight(div) / 3;
            chai_1.assert.strictEqual(component1.height(), threeRowExpectedHeight, "first Component received one-third of the available height");
            chai_1.assert.strictEqual(component2.height(), threeRowExpectedHeight, "second Component received one-third of the available height");
            chai_1.assert.strictEqual(component3.height(), threeRowExpectedHeight, "third Component received one-third of the available height");
            verifyOrigins(tableRows);
            table.destroy();
            div.remove();
        });
        it("gives height to fixed-height Components, then divides remainder between non-fixed-height Components", function () {
            var div = TestMethods.generateDiv();
            var fixedSizeComponent = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var unfixedComponent1 = new Plottable.Component();
            var unfixedComponent2 = new Plottable.Component();
            var table = new Plottable.Components.Table([
                [fixedSizeComponent],
                [unfixedComponent1],
                [unfixedComponent2],
            ]);
            table.renderTo(div);
            var expectedUnfixedHeight = (Plottable.Utils.DOM.elementHeight(div) - FIXED_COMPONENT_SIZE) / 2;
            chai_1.assert.strictEqual(unfixedComponent1.height(), expectedUnfixedHeight, "first non-fixed-height Component received half the remaining height");
            chai_1.assert.strictEqual(unfixedComponent2.height(), expectedUnfixedHeight, "second non-fixed-height Component received half the remaining height");
            table.destroy();
            div.remove();
        });
        it("divides width evenly if there isn't enough width to go around", function () {
            var div = TestMethods.generateDiv(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var fixedComponent1 = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var fixedComponent2 = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var tableRows = [[fixedComponent1, fixedComponent2]];
            var table = new Plottable.Components.Table(tableRows);
            table.renderTo(div);
            var expectedWidth = FIXED_COMPONENT_SIZE / 2;
            chai_1.assert.strictEqual(fixedComponent1.width(), expectedWidth, "first fixed-width Component received half the available width");
            chai_1.assert.strictEqual(fixedComponent2.width(), expectedWidth, "second fixed-width Component received half the available width");
            verifyOrigins(tableRows);
            table.destroy();
            div.remove();
        });
        it("divides height evenly if there isn't enough hieght to go around", function () {
            var div = TestMethods.generateDiv(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var fixedComponent1 = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var fixedComponent2 = new Mocks.FixedSizeComponent(FIXED_COMPONENT_SIZE, FIXED_COMPONENT_SIZE);
            var tableRows = [
                [fixedComponent1],
                [fixedComponent2],
            ];
            var table = new Plottable.Components.Table(tableRows);
            table.renderTo(div);
            var expectedHeight = FIXED_COMPONENT_SIZE / 2;
            chai_1.assert.strictEqual(fixedComponent1.height(), expectedHeight, "first fixed-height Component received half the available height");
            chai_1.assert.strictEqual(fixedComponent2.height(), expectedHeight, "second fixed-height Component received half the available height");
            verifyOrigins(tableRows);
            table.destroy();
            div.remove();
        });
        describe("layout with padding and weights", function () {
            it("adds row padding between rows", function () {
                var div = TestMethods.generateDiv();
                var component1 = new Plottable.Component();
                var component2 = new Plottable.Component();
                var tableRows = [
                    [component1],
                    [component2],
                ];
                var table = new Plottable.Components.Table(tableRows);
                var rowPadding = 10;
                table.rowPadding(rowPadding);
                table.renderTo(div);
                var expectedHeight = (Plottable.Utils.DOM.elementHeight(div) - rowPadding) / 2;
                chai_1.assert.strictEqual(component1.height(), expectedHeight, "first non-fixed-height Component received half the remaining height");
                chai_1.assert.strictEqual(component2.height(), expectedHeight, "second non-fixed-height Component received half the remaining height");
                verifyOrigins(tableRows, rowPadding, 0);
                table.destroy();
                div.remove();
            });
            it("adds column padding between columns", function () {
                var div = TestMethods.generateDiv();
                var component1 = new Plottable.Component();
                var component2 = new Plottable.Component();
                var tableRows = [[component1, component2]];
                var table = new Plottable.Components.Table(tableRows);
                var columnPadding = 10;
                table.columnPadding(columnPadding);
                table.renderTo(div);
                var expectedWidth = (Plottable.Utils.DOM.elementHeight(div) - columnPadding) / 2;
                chai_1.assert.strictEqual(component1.width(), expectedWidth, "first non-fixed-width Component received half the remaining width");
                chai_1.assert.strictEqual(component2.width(), expectedWidth, "second non-fixed-width Component received half the remaining width");
                verifyOrigins(tableRows, 0, columnPadding);
                table.destroy();
                div.remove();
            });
            it("allocates height to unfixed rows according to row weights", function () {
                var div = TestMethods.generateDiv();
                var component0 = new Plottable.Component();
                var component1 = new Plottable.Component();
                var table = new Plottable.Components.Table([
                    [component0],
                    [component1],
                ]);
                var row0Weight = 1;
                table.rowWeight(0, row0Weight);
                var row1Weight = 2;
                table.rowWeight(1, row1Weight);
                var totalRowWeight = row0Weight + row1Weight;
                table.renderTo(div);
                chai_1.assert.strictEqual(component0.height(), Plottable.Utils.DOM.elementWidth(div) * row0Weight / totalRowWeight, "row 0 received height according to its weight");
                chai_1.assert.strictEqual(component1.height(), Plottable.Utils.DOM.elementWidth(div) * row1Weight / totalRowWeight, "row 1 received height according to its weight");
                table.destroy();
                div.remove();
            });
            it("allocates width to unfixed columns according to column weights", function () {
                var div = TestMethods.generateDiv();
                var component0 = new Plottable.Component();
                var component1 = new Plottable.Component();
                var table = new Plottable.Components.Table([[component0, component1]]);
                var column0Weight = 1;
                table.columnWeight(0, column0Weight);
                var column1Weight = 2;
                table.columnWeight(1, column1Weight);
                var totalColumnWeight = column0Weight + column1Weight;
                table.renderTo(div);
                chai_1.assert.strictEqual(component0.width(), Plottable.Utils.DOM.elementWidth(div) * column0Weight / totalColumnWeight, "column 0 received width according to its weight");
                chai_1.assert.strictEqual(component1.width(), Plottable.Utils.DOM.elementWidth(div) * column1Weight / totalColumnWeight, "column 1 received width according to its weight");
                table.destroy();
                div.remove();
            });
        });
    });
});
