"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Dispatchers", function () {
    describe("Touch Dispatcher", function () {
        describe("Basic usage", function () {
            it("creates only one Dispatcher.Touch per element", function () {
                var div = TestMethods.generateDiv();
                var component = new Plottable.Component();
                component.anchor(div);
                var td1 = Plottable.Dispatchers.Touch.getDispatcher(component);
                chai_1.assert.isNotNull(td1, "created a new Dispatcher on an SVG");
                var td2 = Plottable.Dispatchers.Touch.getDispatcher(component);
                chai_1.assert.strictEqual(td1, td2, "returned the existing Dispatcher if called again with same <div>");
                div.remove();
            });
        });
        describe("Callbacks", function () {
            var div;
            var touchDispatcher;
            var callbackWasCalled;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var targetXs = [17, 18, 12, 23, 44];
            var targetYs = [77, 78, 52, 43, 14];
            var ids = targetXs.map(function (targetX, i) { return i; });
            var expectedPoints = targetXs.map(function (targetX, i) {
                return {
                    x: targetX,
                    y: targetYs[i],
                };
            });
            var callbackWithPositionAssertion = function (_ids, points, event) {
                callbackWasCalled = true;
                _ids.forEach(function (id) {
                    TestMethods.assertPointsClose(points[id], expectedPoints[id], 0.5, "touch position is correct");
                });
                chai_1.assert.isNotNull(event, "TouchEvent was passed to the Dispatcher");
            };
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.anchor(div);
                touchDispatcher = Plottable.Dispatchers.Touch.getDispatcher(component);
                callbackWasCalled = false;
            });
            it("calls the touchStart callback", function () {
                chai_1.assert.strictEqual(touchDispatcher.onTouchStart(callbackWithPositionAssertion), touchDispatcher, "setting the touchStart callback returns the dispatcher");
                TestMethods.triggerFakeTouchEvent("touchstart", div, expectedPoints, ids);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on touchstart");
                chai_1.assert.strictEqual(touchDispatcher.offTouchStart(callbackWithPositionAssertion), touchDispatcher, "unsetting the touchStart callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeTouchEvent("touchstart", div, expectedPoints, ids);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls the touchMove callback", function () {
                chai_1.assert.strictEqual(touchDispatcher.onTouchMove(callbackWithPositionAssertion), touchDispatcher, "setting the touchMove callback returns the dispatcher");
                TestMethods.triggerFakeTouchEvent("touchmove", div, expectedPoints, ids);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on touchmove");
                chai_1.assert.strictEqual(touchDispatcher.offTouchMove(callbackWithPositionAssertion), touchDispatcher, "unsetting the touchMove callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeTouchEvent("touchmove", div, expectedPoints, ids);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls the touchEnd callback", function () {
                chai_1.assert.strictEqual(touchDispatcher.onTouchEnd(callbackWithPositionAssertion), touchDispatcher, "setting the touchEnd callback returns the dispatcher");
                TestMethods.triggerFakeTouchEvent("touchend", div, expectedPoints, ids);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on touchend");
                chai_1.assert.strictEqual(touchDispatcher.offTouchEnd(callbackWithPositionAssertion), touchDispatcher, "unsetting the touchEnd callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeTouchEvent("touchend", div, expectedPoints, ids);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("calls the touchCancel callback", function () {
                chai_1.assert.strictEqual(touchDispatcher.onTouchCancel(callbackWithPositionAssertion), touchDispatcher, "setting the touchCancel callback returns the dispatcher");
                TestMethods.triggerFakeTouchEvent("touchcancel", div, expectedPoints, ids);
                chai_1.assert.isTrue(callbackWasCalled, "callback was called on touchend");
                chai_1.assert.strictEqual(touchDispatcher.offTouchCancel(callbackWithPositionAssertion), touchDispatcher, "unsetting the touchCancel callback returns the dispatcher");
                callbackWasCalled = false;
                TestMethods.triggerFakeTouchEvent("touchcancel", div, expectedPoints, ids);
                chai_1.assert.isFalse(callbackWasCalled, "callback was disconnected from the dispatcher");
                div.remove();
            });
            it("can register two callbacks for the same touch dispatcher", function () {
                var callback1WasCalled = false;
                var callback1 = function () { return callback1WasCalled = true; };
                var callback2WasCalled = false;
                var callback2 = function () { return callback2WasCalled = true; };
                touchDispatcher.onTouchStart(callback1);
                touchDispatcher.onTouchStart(callback2);
                TestMethods.triggerFakeTouchEvent("touchstart", div, expectedPoints, ids);
                chai_1.assert.isTrue(callback1WasCalled, "callback 1 was called on touchstart");
                chai_1.assert.isTrue(callback2WasCalled, "callback 2 was called on touchstart");
                touchDispatcher.offTouchStart(callback1);
                callback1WasCalled = false;
                callback2WasCalled = false;
                TestMethods.triggerFakeTouchEvent("touchstart", div, expectedPoints, ids);
                chai_1.assert.isFalse(callback1WasCalled, "callback 1 was disconnected from the dispatcher");
                chai_1.assert.isTrue(callback2WasCalled, "callback 2 is still connected to the dispatcher");
                div.remove();
            });
            it("doesn't call callbacks if not in the DOM", function () {
                var customCallbackWasCalled = false;
                var callback = function () { return customCallbackWasCalled = true; };
                touchDispatcher.onTouchMove(callback);
                TestMethods.triggerFakeTouchEvent("touchmove", div, expectedPoints, ids);
                chai_1.assert.isTrue(customCallbackWasCalled, "callback was called on touchmove");
                div.remove();
                customCallbackWasCalled = false;
                TestMethods.triggerFakeTouchEvent("touchmove", div, expectedPoints, ids);
                chai_1.assert.isFalse(customCallbackWasCalled, "callback was not called after <div> was removed from DOM");
                touchDispatcher.offTouchMove(callback);
            });
        });
    });
});
