"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Interactions", function () {
    describe("Click Interaction", function () {
        var DIV_WIDTH = 400;
        var DIV_HEIGHT = 400;
        var clickedPoint;
        var div;
        var component;
        var clickInteraction;
        beforeEach(function () {
            div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
            component = new Plottable.Component();
            component.renderTo(div);
            clickInteraction = new Plottable.Interactions.Click();
            clickInteraction.attachTo(component);
            clickedPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
        });
        afterEach(function () {
            if (this.currentTest.state === "passed") {
                div.remove();
            }
        });
        function makeClickCallback() {
            var callback = function (p) {
                callback.lastPoint = p;
                callback.called = true;
                callback.calledCount += 1;
            };
            callback.called = false;
            callback.calledCount = 0;
            callback.reset = function () {
                callback.lastPoint = undefined;
                callback.called = false;
            };
            return callback;
        }
        function clickPoint(point, mode) {
            if (mode === void 0) { mode = TestMethods.InteractionMode.Mouse; }
            clickPointWithMove(point, point, mode);
        }
        function clickPointWithMove(clickStartPoint, clickEndPoint, mode) {
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Start, component.content(), clickStartPoint.x, clickStartPoint.y);
            TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.End, component.content(), clickEndPoint.x, clickEndPoint.y);
        }
        function doubleClickPoint(point, mode) {
            if (mode === void 0) { mode = TestMethods.InteractionMode.Mouse; }
            doubleClickPointWithMove(point, point, mode);
        }
        function doubleClickPointWithMove(firstClickPoint, secondClickPoint, mode) {
            clickPoint(firstClickPoint, mode);
            clickPoint(secondClickPoint, mode);
            TestMethods.triggerFakeMouseEvent("dblclick", component.content(), secondClickPoint.x, secondClickPoint.y);
        }
        // All the tests require setTimouts becuase of the internal logic in click interactions
        function runAsserts(callback, done) {
            setTimeout(function () {
                callback();
                done();
            }, 0);
        }
        describe("registering click callbacks", function () {
            it("registers callback using onClick", function (done) {
                var callback = makeClickCallback();
                chai_1.assert.strictEqual(clickInteraction.onClick(callback), clickInteraction, "registration returns the calling Interaction");
                clickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isTrue(callback.called, "Interaction should trigger the callback");
                }, done);
            });
            it("deregisters callback using offClick", function (done) {
                var callback = makeClickCallback();
                clickInteraction.onClick(callback);
                chai_1.assert.strictEqual(clickInteraction.offClick(callback), clickInteraction, "deregistration returns the calling Interaction");
                clickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isFalse(callback.called, "Callback should be disconnected from the Interaction");
                }, done);
            });
            it("can register multiple onClick callbacks", function (done) {
                var callback1 = makeClickCallback();
                var callback2 = makeClickCallback();
                clickInteraction.onClick(callback1);
                clickInteraction.onClick(callback2);
                clickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isTrue(callback1.called, "Interaction should trigger the first callback");
                    chai_1.assert.isTrue(callback2.called, "Interaction should trigger the second callback");
                }, done);
            });
            it("can deregister a callback without affecting the other ones", function (done) {
                var callback1 = makeClickCallback();
                var callback2 = makeClickCallback();
                clickInteraction.onClick(callback1);
                clickInteraction.onClick(callback2);
                clickInteraction.offClick(callback1);
                clickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isFalse(callback1.called, "Callback1 should be disconnected from the click interaction");
                    chai_1.assert.isTrue(callback2.called, "Callback2 should still exist on the click interaction");
                }, done);
            });
        });
        describe("registering double click callbacks", function () {
            it("registers callback using onDoubleClick", function (done) {
                var callback = makeClickCallback();
                chai_1.assert.strictEqual(clickInteraction.onDoubleClick(callback), clickInteraction, "registration returns the calling Interaction");
                doubleClickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isTrue(callback.called, "Interaction should trigger the callback");
                }, done);
            });
            it("deregisters callback using offDoubleClick", function (done) {
                var callback = makeClickCallback();
                clickInteraction.onDoubleClick(callback);
                chai_1.assert.strictEqual(clickInteraction.offDoubleClick(callback), clickInteraction, "deregistration returns the calling Interaction");
                doubleClickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isFalse(callback.called, "Callback should be disconnected from the Interaction");
                }, done);
            });
            it("can register multiple onDoubleClick callbacks", function (done) {
                var callback1 = makeClickCallback();
                var callback2 = makeClickCallback();
                clickInteraction.onDoubleClick(callback1);
                clickInteraction.onDoubleClick(callback2);
                doubleClickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isTrue(callback1.called, "Interaction should trigger the first callback");
                    chai_1.assert.isTrue(callback2.called, "Interaction should trigger the second callback");
                }, done);
            });
            it("can deregister a callback without affecting the other ones", function (done) {
                var callback1 = makeClickCallback();
                var callback2 = makeClickCallback();
                clickInteraction.onDoubleClick(callback1);
                clickInteraction.onDoubleClick(callback2);
                clickInteraction.offDoubleClick(callback1);
                doubleClickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isFalse(callback1.called, "Callback1 should be disconnected from the click interaction");
                    chai_1.assert.isTrue(callback2.called, "Callback2 should still exist on the click interaction");
                }, done);
            });
        });
        [TestMethods.InteractionMode.Mouse, TestMethods.InteractionMode.Touch].forEach(function (mode) {
            describe("invoking click callbacks with " + TestMethods.InteractionMode[mode] + " events", function () {
                var callback;
                var quarterPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var halfPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var outsidePoint = { x: DIV_WIDTH * 2, y: DIV_HEIGHT * 2 };
                beforeEach(function () {
                    callback = makeClickCallback();
                    clickInteraction.onClick(callback);
                });
                it("invokes onClick callback on single location click", function (done) {
                    clickPoint(halfPoint, mode);
                    runAsserts(function () {
                        chai_1.assert.isTrue(callback.called, "callback called on clicking Component without moving pointer");
                        chai_1.assert.deepEqual(callback.lastPoint, halfPoint, "was passed correct point");
                    }, done);
                });
                it("provides correct release point to onClick callback on click", function (done) {
                    clickPointWithMove(halfPoint, halfPoint, mode);
                    runAsserts(function () {
                        chai_1.assert.isTrue(callback.called, "callback called on clicking and releasing inside the Component");
                        chai_1.assert.deepEqual(callback.lastPoint, halfPoint, "was passed mouseup point");
                    }, done);
                });
                it("does not invoke callback if interaction start and end in different locations", function (done) {
                    clickPointWithMove(halfPoint, quarterPoint, mode);
                    runAsserts(function () {
                        chai_1.assert.isFalse(callback.called, "callback not called if click is released at a different location");
                    }, done);
                });
                it("does not invoke callback if click is released outside Component", function (done) {
                    clickPointWithMove(halfPoint, outsidePoint, mode);
                    runAsserts(function () {
                        chai_1.assert.isFalse(callback.called, "callback not called if click is released outside Component");
                    }, done);
                });
                it("does not invoke callback if click is started outside Component", function (done) {
                    clickPointWithMove(outsidePoint, halfPoint, mode);
                    runAsserts(function () {
                        chai_1.assert.isFalse(callback.called, "callback not called if click was started outside Component");
                    }, done);
                });
                it("invokes callback if the pointer is moved out then back inside the Component before releasing", function (done) {
                    TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Start, component.content(), DIV_WIDTH / 2, DIV_HEIGHT / 2);
                    TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.Move, component.content(), DIV_WIDTH * 2, DIV_HEIGHT * 2);
                    TestMethods.triggerFakeInteractionEvent(mode, TestMethods.InteractionType.End, component.content(), DIV_WIDTH / 2, DIV_HEIGHT / 2);
                    runAsserts(function () {
                        chai_1.assert.isTrue(callback.called, "callback still called if the pointer is moved out then back inside the Component before releasing");
                    }, done);
                });
                if (mode === TestMethods.InteractionMode.Touch) {
                    it("does not trigger callback if touch event is cancelled", function (done) {
                        TestMethods.triggerFakeTouchEvent("touchstart", component.content(), [halfPoint]);
                        TestMethods.triggerFakeTouchEvent("touchcancel", component.content(), [halfPoint]);
                        TestMethods.triggerFakeTouchEvent("touchend", component.content(), [halfPoint]);
                        runAsserts(function () {
                            chai_1.assert.isFalse(callback.called, "callback not called if touch was cancelled");
                        }, done);
                    });
                }
            });
        });
        describe("invoking double click callbacks with mouse events", function () {
            // The current implementation assumes there will be at least some delay between clicks
            it("silences click event associated with a double click event", function (done) {
                var singleClickCallback = makeClickCallback();
                var doubleClickCallback = makeClickCallback();
                clickInteraction.onClick(singleClickCallback);
                clickInteraction.onDoubleClick(doubleClickCallback);
                clickPoint(clickedPoint);
                setTimeout(function () {
                    clickPoint(clickedPoint);
                    TestMethods.triggerFakeMouseEvent("dblclick", component.content(), clickedPoint.x, clickedPoint.y);
                    runAsserts(function () {
                        chai_1.assert.equal(singleClickCallback.calledCount, 1, "single click callback only called once on doubleclick");
                        chai_1.assert.isTrue(doubleClickCallback.called, "double click callback called on doubleclick");
                    }, done);
                }, 0);
            });
            it("does not invoke callback if clicking in the same place without doubleclick event", function (done) {
                var callback = makeClickCallback();
                clickInteraction.onDoubleClick(callback);
                clickPoint(clickedPoint);
                clickPoint(clickedPoint);
                runAsserts(function () {
                    chai_1.assert.isFalse(callback.called, "double click callback not called without double click event");
                }, done);
            });
        });
    });
});
