"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Interactions", function () {
    describe("PanZoom Interaction", function () {
        describe("Scale setting", function () {
            var xScale;
            var yScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                panZoomInteraction = new Plottable.Interactions.PanZoom();
            });
            it("can set scales", function () {
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), [], "no xScales exist by default");
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), [], "no yScales exist by default");
                chai_1.assert.strictEqual(panZoomInteraction.addXScale(xScale), panZoomInteraction, "setting the xScale returns the interaction");
                chai_1.assert.strictEqual(panZoomInteraction.addYScale(yScale), panZoomInteraction, "setting the yScale returns the interaction");
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), [xScale], "xScale has been added");
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), [yScale], "yScale has been added");
            });
            it("can remove scales", function () {
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.addYScale(yScale);
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), [xScale], "xScale has been added");
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), [yScale], "yScale has been added");
                chai_1.assert.strictEqual(panZoomInteraction.removeXScale(xScale), panZoomInteraction, "removing the xScale returns the interaction");
                chai_1.assert.strictEqual(panZoomInteraction.removeYScale(yScale), panZoomInteraction, "removing the yScale returns the interaction");
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), [], "xScale has been removed");
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), [], "yScale has been removed");
            });
            it("can set the xScales in batch", function () {
                var xScale2 = new Plottable.Scales.Linear();
                var expectedXScales = [xScale, xScale2];
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.addXScale(xScale2);
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), expectedXScales, "interaction contains the 2 xScales added");
                chai_1.assert.deepEqual(panZoomInteraction.xScales([]), panZoomInteraction, "returns the calling interaction");
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), [], "scales can be removed with the xScales call");
                panZoomInteraction.xScales([xScale, xScale2]);
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), expectedXScales, "setting and adding x scales result in the same behavior");
            });
            it("can set the yScales in batch", function () {
                var yScale2 = new Plottable.Scales.Linear();
                var expectedYScales = [yScale, yScale2];
                panZoomInteraction.addYScale(yScale);
                panZoomInteraction.addYScale(yScale2);
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), expectedYScales, "interaction contains the 2 yScales added");
                chai_1.assert.deepEqual(panZoomInteraction.yScales([]), panZoomInteraction, "returns the calling interaction");
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), [], "scales can be removed with the yScales call");
                panZoomInteraction.yScales([yScale, yScale2]);
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), expectedYScales, "setting and adding y scales result in the same behavior");
            });
            it("does not create duplicate for an already existent xScale", function () {
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.addXScale(xScale);
                chai_1.assert.deepEqual(panZoomInteraction.xScales(), [xScale], "there is exactly one copy of xScale");
            });
            it("does not create duplicate for an already existent yScale", function () {
                panZoomInteraction.addYScale(yScale);
                panZoomInteraction.addYScale(yScale);
                chai_1.assert.deepEqual(panZoomInteraction.yScales(), [yScale], "there is exactly one copy of yScale");
            });
        });
        describe("Panning", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var yScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]).range([0, DIV_WIDTH]);
                yScale = new Plottable.Scales.Linear();
                yScale.domain([0, DIV_HEIGHT / 2]).range([0, DIV_HEIGHT]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.addYScale(yScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            it("translates the scale correctly on dragging (mouse)", function () {
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var endPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT * 3 / 4 };
                var expectedXDomain = domainAfterPan(startPoint, endPoint, xScale, true);
                var expectedYDomain = domainAfterPan(startPoint, endPoint, yScale, false);
                TestMethods.triggerFakeMouseEvent("mousedown", eventTarget, startPoint.x, startPoint.y);
                TestMethods.triggerFakeMouseEvent("mousemove", eventTarget, endPoint.x, endPoint.y);
                TestMethods.triggerFakeMouseEvent("mouseend", eventTarget, endPoint.x, endPoint.y);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale pans to the correct domain via drag (mouse)");
                chai_1.assert.deepEqual(yScale.domain(), expectedYDomain, "yScale pans to the correct domain via drag (mouse)");
                div.remove();
            });
            it("translates the scale correctly on dragging to outside of the component (mouse)", function () {
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var expectedXDomain = domainAfterPan(startPoint, endPoint, xScale, true);
                var expectedYDomain = domainAfterPan(startPoint, endPoint, yScale, false);
                TestMethods.triggerFakeMouseEvent("mousedown", eventTarget, startPoint.x, startPoint.y);
                TestMethods.triggerFakeMouseEvent("mousemove", eventTarget, endPoint.x, endPoint.y);
                TestMethods.triggerFakeMouseEvent("mouseend", eventTarget, endPoint.x, endPoint.y);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale pans to the correct domain via drag (mouse)");
                chai_1.assert.deepEqual(yScale.domain(), expectedYDomain, "yScale pans to the correct domain via drag (mouse)");
                div.remove();
            });
            it("translates multiple scales correctly on dragging (mouse)", function () {
                var xScale2 = new Plottable.Scales.Linear();
                xScale2.domain([0, 2 * DIV_WIDTH]).range([0, DIV_WIDTH]);
                panZoomInteraction.addXScale(xScale2);
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var expectedXDomain = domainAfterPan(startPoint, endPoint, xScale, true);
                var expectedXDomain2 = domainAfterPan(startPoint, endPoint, xScale2, true);
                TestMethods.triggerFakeMouseEvent("mousedown", eventTarget, startPoint.x, startPoint.y);
                TestMethods.triggerFakeMouseEvent("mousemove", eventTarget, endPoint.x, endPoint.y);
                TestMethods.triggerFakeMouseEvent("mouseend", eventTarget, endPoint.x, endPoint.y);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale pans to the correct domain via drag (mouse)");
                chai_1.assert.deepEqual(xScale2.domain(), expectedXDomain2, "xScale2 pans to the correct domain via drag (mouse)");
                div.remove();
            });
            it("translates the scale correctly on dragging (touch)", function () {
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var endPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT * 3 / 4 };
                var expectedXDomain = domainAfterPan(startPoint, endPoint, xScale, true);
                var expectedYDomain = domainAfterPan(startPoint, endPoint, yScale, false);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale pans to the correct domain via drag (touch)");
                chai_1.assert.deepEqual(yScale.domain(), expectedYDomain, "yScale pans to the correct domain via drag (touch)");
                div.remove();
            });
            it("translates the scale correctly on dragging to outside of the component (touch)", function () {
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var expectedXDomain = domainAfterPan(startPoint, endPoint, xScale, true);
                var expectedYDomain = domainAfterPan(startPoint, endPoint, yScale, false);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale pans to the correct domain via drag (touch)");
                chai_1.assert.deepEqual(yScale.domain(), expectedYDomain, "yScale pans to the correct domain via drag (touch)");
                div.remove();
            });
            it("translates multiple scales correctly on dragging (touch)", function () {
                var xScale2 = new Plottable.Scales.Linear();
                xScale2.domain([0, 2 * DIV_WIDTH]).range([0, DIV_WIDTH]);
                panZoomInteraction.addXScale(xScale2);
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var expectedXDomain = domainAfterPan(startPoint, endPoint, xScale, true);
                var expectedXDomain2 = domainAfterPan(startPoint, endPoint, xScale2, false);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale pans to the correct domain via drag (touch)");
                chai_1.assert.deepEqual(xScale2.domain(), expectedXDomain2, "xScale2 pans to the correct domain via drag (touch)");
                div.remove();
            });
            function domainAfterPan(startPoint, endPoint, scale, isHorizontal) {
                var delta = isHorizontal ? endPoint.x - startPoint.x : endPoint.y - startPoint.y;
                var domain = scale.domain();
                var range = isHorizontal ? DIV_WIDTH : DIV_HEIGHT;
                var diff = delta / range * (domain[1] - domain[0]);
                return domain.map(function (v) { return v - diff; });
            }
        });
        describe("Zooming", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var component;
            var eventTarget;
            var xScale;
            var yScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]).range([0, DIV_WIDTH]);
                yScale = new Plottable.Scales.Linear();
                yScale.domain([0, DIV_HEIGHT / 2]).range([0, DIV_HEIGHT]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.addYScale(yScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            it("magnifies the scale correctly (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    div.remove();
                    return;
                }
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = 500;
                var expectedXDomain = domainAfterWheel(deltaY, scrollPoint, xScale, true);
                var expectedYDomain = domainAfterWheel(deltaY, scrollPoint, yScale, true);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale zooms to the correct domain via scroll");
                chai_1.assert.deepEqual(yScale.domain(), expectedYDomain, "yScale zooms to the correct domain via scroll");
                div.remove();
            });
            it("magnifies multiple scales correctly (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    div.remove();
                    return;
                }
                var xScale2 = new Plottable.Scales.Linear();
                xScale2.domain([0, 2 * DIV_WIDTH]).range([0, DIV_WIDTH]);
                panZoomInteraction.addXScale(xScale2);
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = 500;
                var expectedXDomain = domainAfterWheel(deltaY, scrollPoint, xScale, true);
                var expectedXDomain2 = domainAfterWheel(deltaY, scrollPoint, xScale2, true);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale zooms to the correct domain via scroll");
                chai_1.assert.deepEqual(xScale2.domain(), expectedXDomain2, "xScale2 zooms to the correct domain via scroll");
                div.remove();
            });
            it("magnifies the scale correctly (pinching)", function () {
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var startPoint2 = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: DIV_WIDTH * 3 / 4, y: DIV_HEIGHT * 3 / 4 };
                var expectedXDomain = domainAfterPinch(startPoint, startPoint2, startPoint, endPoint, xScale, true);
                var expectedYDomain = domainAfterPinch(startPoint, startPoint2, startPoint, endPoint, yScale, true);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale transforms to the correct domain via pinch");
                chai_1.assert.deepEqual(yScale.domain(), expectedYDomain, "yScale transforms to the correct domain via pinch");
                div.remove();
            });
            it("magnifies multiple scales correctly (pinching)", function () {
                var xScale2 = new Plottable.Scales.Linear();
                xScale2.domain([0, 2 * DIV_WIDTH]).range([0, DIV_WIDTH]);
                panZoomInteraction.addXScale(xScale2);
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var startPoint2 = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: DIV_WIDTH * 3 / 4, y: DIV_HEIGHT * 3 / 4 };
                var expectedXDomain = domainAfterPinch(startPoint, startPoint2, startPoint, endPoint, xScale, true);
                var expectedXDomain2 = domainAfterPinch(startPoint, startPoint2, startPoint, endPoint, xScale2, true);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale transforms to the correct domain via pinch");
                chai_1.assert.deepEqual(xScale2.domain(), expectedXDomain2, "xScale2 transforms to the correct domain via pinch");
                div.remove();
            });
            it("can pinch inside one component and not affect another component", function () {
                var xScale2 = new Plottable.Scales.Linear();
                var initialDomain = [0, DIV_WIDTH / 2];
                xScale2.domain(initialDomain).range([0, DIV_WIDTH]);
                var component2 = new Plottable.Component();
                var panZoomInteraction2 = new Plottable.Interactions.PanZoom();
                panZoomInteraction2.addXScale(xScale2);
                panZoomInteraction2.attachTo(component2);
                var table = new Plottable.Components.Table([[component, component2]]);
                table.renderTo(div);
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 2 };
                var startPoint2 = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: DIV_WIDTH * 3 / 4, y: DIV_HEIGHT / 2 };
                var expectedXDomain = domainAfterPinch(startPoint, startPoint2, startPoint, endPoint, xScale, true);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.deepEqual(xScale.domain(), expectedXDomain, "xScale inside target component transforms via pinch");
                chai_1.assert.deepEqual(xScale2.domain(), initialDomain, "xScale outside of target component does not transform via pinch");
                div.remove();
            });
            function domainAfterPinch(startPoint, startPoint2, endPoint, endPoint2, scale, isHorizontal) {
                var oldDistance = isHorizontal ? startPoint2.x - startPoint.x : startPoint2.y - startPoint.y;
                var newDistance = isHorizontal ? endPoint2.x - endPoint.x : endPoint2.y - endPoint.y;
                var delta = isHorizontal ? endPoint2.x - startPoint2.x : endPoint2.y - startPoint2.y;
                var zoomAmount = oldDistance / newDistance;
                var domain = scale.domain();
                var range = isHorizontal ? DIV_WIDTH : DIV_HEIGHT;
                var diff = delta / range * (domain[1] - domain[0]);
                return domain.map(function (v, i) { return (v - domain[0] + diff) * zoomAmount + domain[0]; });
            }
            function domainAfterWheel(deltaY, scrollPoint, scale, isHorizontal) {
                var zoomAmount = Math.pow(2, deltaY * .002);
                var domain = scale.domain();
                var diff = (isHorizontal ? scrollPoint.x / DIV_WIDTH : scrollPoint.y / DIV_HEIGHT) * (domain[1] - domain[0]);
                return domain.map(function (v, i) { return (v - domain[0] - diff) * zoomAmount + domain[0] + diff; });
            }
        });
        describe("Setting minDomainExtent", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            it("can set minDomainExtent", function () {
                var minimumDomainExtent = DIV_WIDTH / 4;
                chai_1.assert.strictEqual(panZoomInteraction.minDomainExtent(xScale, minimumDomainExtent), panZoomInteraction, "setting the minDomainExtent returns the interaction");
                chai_1.assert.strictEqual(panZoomInteraction.minDomainExtent(xScale), minimumDomainExtent, "returns the correct minDomainExtent");
                div.remove();
            });
            it("rejects negative extents", function () {
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return panZoomInteraction.minDomainExtent(xScale, -1); }, Error, "extent must be non-negative", "Correctly rejects -1");
                div.remove();
            });
            it("can't set minDomainExtent() be larger than maxDomainExtent() for the same Scale", function () {
                var maximumDomainExtent = DIV_WIDTH / 2;
                panZoomInteraction.maxDomainExtent(xScale, maximumDomainExtent);
                var tooBigMinimumDomainExtent = maximumDomainExtent * 2;
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return panZoomInteraction.minDomainExtent(xScale, tooBigMinimumDomainExtent); }, Error, "minDomainExtent must be smaller than maxDomainExtent for the same Scale", "cannot have minDomainExtent larger than maxDomainExtent");
                div.remove();
            });
            it("cannot go beyond the specified domainExtent (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    div.remove();
                    return;
                }
                var minimumDomainExtent = DIV_WIDTH / 4;
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = -3000;
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                var domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.operator(domainExtent, "<", minimumDomainExtent, "there is no zoom limit before setting minimun extent via scroll");
                panZoomInteraction.minDomainExtent(xScale, minimumDomainExtent);
                xScale.domain([0, DIV_WIDTH / 2]);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.strictEqual(domainExtent, minimumDomainExtent, "xScale zooms to the correct domain via scroll");
                div.remove();
            });
            it("cannot go beyond the specified domainExtent (pinching)", function () {
                var minimumDomainExtent = DIV_WIDTH / 4;
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var startPoint2 = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var zoomAmount = 6;
                var endX = (startPoint2.x - startPoint.x) * zoomAmount + startPoint.x;
                var endY = (startPoint2.y - startPoint.y) * zoomAmount + startPoint.y;
                var endPoint = { x: endX, y: endY };
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                var domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.operator(domainExtent, "<", minimumDomainExtent, "there is no zoom limit before setting minimun extent via pinch");
                panZoomInteraction.minDomainExtent(xScale, minimumDomainExtent);
                xScale.domain([0, DIV_WIDTH / 2]);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.strictEqual(domainExtent, minimumDomainExtent, "xScale zooms to the correct domain via pinch");
                div.remove();
            });
        });
        describe("Setting maxDomainExtent", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            it("can set maxDomainExtent", function () {
                var maximumDomainExtent = DIV_WIDTH;
                chai_1.assert.strictEqual(panZoomInteraction.maxDomainExtent(xScale, maximumDomainExtent), panZoomInteraction, "setting the maxDomainExtent returns the interaction");
                chai_1.assert.strictEqual(panZoomInteraction.maxDomainExtent(xScale), maximumDomainExtent, "returns the correct maxDomainExtent");
                div.remove();
            });
            it("rejects non-positive extents", function () {
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return panZoomInteraction.maxDomainExtent(xScale, -1); }, Error, "extent must be positive", "Correctly rejects -1");
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return panZoomInteraction.maxDomainExtent(xScale, 0); }, Error, "extent must be positive", "Correctly rejects 0");
                div.remove();
            });
            it("can't set maxDomainExtent() to be smaller than minDomainExtent() for the same Scale", function () {
                var minimumDomainExtent = DIV_WIDTH / 2;
                panZoomInteraction.minDomainExtent(xScale, minimumDomainExtent);
                var tooSmallMaximumDomainExtent = minimumDomainExtent / 2;
                // HACKHACK #2661: Cannot assert errors being thrown with description
                chai_1.assert.throws(function () { return panZoomInteraction.maxDomainExtent(xScale, tooSmallMaximumDomainExtent); }, Error, "maxDomainExtent must be larger than minDomainExtent for the same Scale", "cannot have maxDomainExtent smaller than minDomainExtent");
                div.remove();
            });
            it("cannot go beyond the specified domainExtent (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    div.remove();
                    return;
                }
                var maximumDomainExtent = DIV_WIDTH;
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = 3000;
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                var domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.operator(domainExtent, ">", maximumDomainExtent, "there is no zoom limit before setting maximun extent via scroll");
                xScale.domain([0, DIV_WIDTH / 2]);
                panZoomInteraction.maxDomainExtent(xScale, maximumDomainExtent);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.strictEqual(domainExtent, maximumDomainExtent, "xScale zooms to the correct domain via scroll");
                div.remove();
            });
            it("cannot go beyond the specified domainExtent (pinching)", function () {
                var maximumDomainExtent = DIV_WIDTH;
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var startPoint2 = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var zoomAmount = 1 / 6;
                var endX = (startPoint2.x - startPoint.x) * zoomAmount + startPoint.x;
                var endY = (startPoint2.y - startPoint.y) * zoomAmount + startPoint.y;
                var endPoint = { x: endX, y: endY };
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                var domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.operator(domainExtent, ">", maximumDomainExtent, "there is no zoom limit before setting maximun extent via pinch");
                xScale.domain([0, DIV_WIDTH / 2]);
                panZoomInteraction.maxDomainExtent(xScale, maximumDomainExtent);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                domainExtent = Math.abs(xScale.domain()[1] - xScale.domain()[0]);
                chai_1.assert.strictEqual(domainExtent, maximumDomainExtent, "xScale zooms to the correct domain via pinch");
                div.remove();
            });
        });
        describe("Setting minDomainValue", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]).range([0, DIV_WIDTH]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            afterEach(function () {
                div.remove();
            });
            it("can set minDomainValue", function () {
                var domainValue = DIV_WIDTH / 4;
                panZoomInteraction.minDomainValue(xScale, domainValue);
                chai_1.assert.strictEqual(panZoomInteraction.minDomainValue(xScale), domainValue, "returns the correct minDomainValue");
            });
            it("cannot go beyond the specified minDomainValue (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    return;
                }
                var domainValue = DIV_WIDTH / 4;
                // simulate massive scroll zoom out
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = 3000;
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.operator(xScale.domain()[0], "<", domainValue, "no initial limit");
                // add limit
                panZoomInteraction.minDomainValue(xScale, domainValue);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.strictEqual(xScale.domain()[0], domainValue, "limit works");
            });
            it("cannot go beyond the specified minDomainValue (pinching)", function () {
                var domainValue = DIV_WIDTH / 4;
                var startPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var startPoint2 = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var zoomAmount = 1 / 6;
                var endX = (startPoint2.x - startPoint.x) * zoomAmount + startPoint.x;
                var endY = (startPoint2.y - startPoint.y) * zoomAmount + startPoint.y;
                var endPoint = { x: endX, y: endY };
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.operator(xScale.domain()[0], "<", domainValue, "no initial limit");
                // add limit
                panZoomInteraction.minDomainValue(xScale, domainValue);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.strictEqual(xScale.domain()[0], domainValue, "limit works");
            });
        });
        describe("Setting maxDomainValue", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]).range([0, DIV_WIDTH]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            afterEach(function () {
                div.remove();
            });
            it("can set maxDomainValue", function () {
                var domainValue = DIV_WIDTH / 4;
                panZoomInteraction.maxDomainValue(xScale, domainValue);
                chai_1.assert.strictEqual(panZoomInteraction.maxDomainValue(xScale), domainValue, "returns the correct minDomainValue");
            });
            it("cannot go beyond the specified maxDomainValue (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    return;
                }
                var domainValue = DIV_WIDTH / 2;
                // simulate massive scroll zoom out
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = 3000;
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.operator(xScale.domain()[1], ">", domainValue, "no initial limit");
                // add limit
                panZoomInteraction.maxDomainValue(xScale, domainValue);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.strictEqual(xScale.domain()[1], domainValue, "limit works");
            });
            it("cannot go beyond the specified maxDomainValue (pinching)", function () {
                var domainValue = DIV_WIDTH / 2;
                var fromCenter = function (dx, dy) {
                    return { x: DIV_WIDTH / 4 + dx, y: DIV_HEIGHT / 4 + dy };
                };
                var startPoint1 = fromCenter(-60, 0);
                var startPoint2 = fromCenter(60, 0);
                var endPoint1 = fromCenter(-10, 0);
                var endPoint2 = fromCenter(10, 0);
                // zoom out pinch
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint1, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint1, endPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint1, endPoint2], [0, 1]);
                chai_1.assert.operator(xScale.domain()[1], ">", domainValue, "no initial limit " + xScale.domain());
                // add limit
                panZoomInteraction.maxDomainValue(xScale, domainValue);
                // zoom out pinch
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint1, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint1, endPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint1, endPoint2], [0, 1]);
                chai_1.assert.strictEqual(xScale.domain()[1], domainValue, "limit works");
            });
        });
        describe("Setting both minDomainValue and maxDomainValue", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var panZoomInteraction;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]).range([0, DIV_WIDTH]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            afterEach(function () {
                div.remove();
            });
            it("cannot go beyond the specified maxDomainValue (mousewheel)", function () {
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    return;
                }
                var domainMinValue = DIV_WIDTH / 4;
                var domainMaxValue = domainMinValue + DIV_WIDTH / 2;
                // simulate massive scroll zoom out
                var scrollPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 4 };
                var deltaY = 3000;
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.operator(xScale.domain()[0], "<", domainMinValue, "no initial min limit");
                chai_1.assert.operator(xScale.domain()[1], ">", domainMaxValue, "no initial max limit");
                // add limit
                xScale.domain([domainMinValue, domainMaxValue]);
                panZoomInteraction.setMinMaxDomainValuesTo(xScale);
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.strictEqual(xScale.domain()[0], domainMinValue, "min limit works");
                chai_1.assert.strictEqual(xScale.domain()[1], domainMaxValue, "max limit works");
            });
            it("cannot go beyond the specified minDomainValue (pinching)", function () {
                var fromCenter = function (dx, dy) {
                    return { x: DIV_WIDTH / 4 + dx, y: DIV_HEIGHT / 4 + dy };
                };
                var domainMinValue = DIV_WIDTH / 4;
                var domainMaxValue = domainMinValue + DIV_WIDTH / 2;
                var startPoint1 = fromCenter(-60, 0);
                var startPoint2 = fromCenter(60, 0);
                var endPoint1 = fromCenter(-10, 0);
                var endPoint2 = fromCenter(10, 0);
                // zoom out pinch
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint1, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint1, endPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint1, endPoint2], [0, 1]);
                chai_1.assert.operator(xScale.domain()[0], "<", domainMinValue, "no initial min limit");
                chai_1.assert.operator(xScale.domain()[1], ">", domainMaxValue, "no initial max limit");
                // add limit
                xScale.domain([domainMinValue, domainMaxValue]);
                panZoomInteraction.setMinMaxDomainValuesTo(xScale);
                // zoom out pinch
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint1, startPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint1, endPoint2], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint1, endPoint2], [0, 1]);
                chai_1.assert.strictEqual(xScale.domain()[0], domainMinValue, "min limit works");
                chai_1.assert.strictEqual(xScale.domain()[1], domainMaxValue, "max limit works");
            });
        });
        describe("Registering and deregistering Pan and Zoom event callbacks", function () {
            var div;
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 500;
            var eventTarget;
            var xScale;
            var panZoomInteraction;
            function makeCallback() {
                var callback = function (e) {
                    callback.called = true;
                };
                callback.called = false;
                callback.reset = function () {
                    callback.called = false;
                };
                return callback;
            }
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                xScale.domain([0, DIV_WIDTH / 2]);
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var component = new Plottable.Component();
                component.renderTo(div);
                panZoomInteraction = new Plottable.Interactions.PanZoom();
                panZoomInteraction.addXScale(xScale);
                panZoomInteraction.attachTo(component);
                eventTarget = component.background();
            });
            it("registers callback using onPanEnd", function () {
                var callback = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                chai_1.assert.strictEqual(panZoomInteraction.onPanEnd(callback), panZoomInteraction, "registration returns the calling Interaction");
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.isTrue(callback.called, "Interaction correctly triggers the callback (touch)");
                callback.reset();
                TestMethods.triggerFakeMouseEvent("mousedown", eventTarget, startPoint.x, startPoint.y);
                TestMethods.triggerFakeMouseEvent("mousemove", eventTarget, endPoint.x, endPoint.y);
                TestMethods.triggerFakeMouseEvent("mouseend", eventTarget, endPoint.x, endPoint.y);
                TestMethods.triggerFakeMouseEvent("mouseup", eventTarget, endPoint.x, endPoint.y);
                chai_1.assert.isTrue(callback.called, "Interaction correctly triggers the callback (mouse)");
                div.remove();
            });
            it("registers callback using onZoomEnd", function () {
                var callback = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                var deltaY = 3000;
                chai_1.assert.strictEqual(panZoomInteraction.onZoomEnd(callback), panZoomInteraction, "registration returns the calling Interaction");
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, scrollPoint], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.isTrue(callback.called, "Interaction correctly triggers the callback (touch)");
                callback.reset();
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.isFalse(callback.called, "Interaction does not trigger zoom callback on pan event (touch)");
                callback.reset();
                // HACKHACK PhantomJS doesn't implement fake creation of WheelEvents
                // https://github.com/ariya/phantomjs/issues/11289
                if (window.PHANTOMJS) {
                    div.remove();
                    return;
                }
                TestMethods.triggerFakeWheelEvent("wheel", div, scrollPoint.x, scrollPoint.y, deltaY);
                chai_1.assert.isTrue(callback.called, "Interaction correctly triggers the callback (mouse)");
                div.remove();
            });
            it("deregisters callback using offPanEnd", function () {
                var callback = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                panZoomInteraction.onPanEnd(callback);
                chai_1.assert.strictEqual(panZoomInteraction.offPanEnd(callback), panZoomInteraction, "deregistration returns the calling Interaction");
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.isFalse(callback.called, "callback should be disconnected from the Interaction");
                div.remove();
            });
            it("deregisters callback using offZoomEnd", function () {
                var callback = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                panZoomInteraction.onZoomEnd(callback);
                chai_1.assert.strictEqual(panZoomInteraction.offZoomEnd(callback), panZoomInteraction, "deregistration returns the calling Interaction");
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, scrollPoint], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.isFalse(callback.called, "callback should be disconnected from the Interaction");
                div.remove();
            });
            it("can register multiple onPanEnd callbacks", function () {
                var callback1 = makeCallback();
                var callback2 = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                panZoomInteraction.onPanEnd(callback1);
                panZoomInteraction.onPanEnd(callback2);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.isTrue(callback1.called, "Interaction should trigger the first callback");
                chai_1.assert.isTrue(callback1.called, "Interaction should trigger the second callback");
                div.remove();
            });
            it("can register multiple onZoomEnd callbacks", function () {
                var callback1 = makeCallback();
                var callback2 = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                panZoomInteraction.onZoomEnd(callback1);
                panZoomInteraction.onZoomEnd(callback2);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, scrollPoint], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.isTrue(callback1.called, "Interaction should trigger the first callback");
                chai_1.assert.isTrue(callback1.called, "Interaction should trigger the second callback");
                div.remove();
            });
            it("can deregister a onPanEnd callback without affecting the other ones", function () {
                var callback1 = makeCallback();
                var callback2 = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                panZoomInteraction.onPanEnd(callback1);
                panZoomInteraction.onPanEnd(callback2);
                panZoomInteraction.offPanEnd(callback1);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint]);
                chai_1.assert.isFalse(callback1.called, "Callback1 should be disconnected from the Interaction");
                chai_1.assert.isTrue(callback2.called, "Callback2 should still exist on the Interaction");
                div.remove();
            });
            it("can deregister a onZoomEnd callback without affecting the other ones", function () {
                var callback1 = makeCallback();
                var callback2 = makeCallback();
                var startPoint = { x: DIV_WIDTH / 2, y: DIV_HEIGHT / 2 };
                var endPoint = { x: -DIV_WIDTH / 2, y: -DIV_HEIGHT / 2 };
                var scrollPoint = { x: DIV_WIDTH / 4, y: DIV_HEIGHT / 4 };
                panZoomInteraction.onZoomEnd(callback1);
                panZoomInteraction.onZoomEnd(callback2);
                panZoomInteraction.offZoomEnd(callback1);
                TestMethods.triggerFakeTouchEvent("touchstart", eventTarget, [startPoint, scrollPoint], [0, 1]);
                TestMethods.triggerFakeTouchEvent("touchmove", eventTarget, [endPoint], [1]);
                TestMethods.triggerFakeTouchEvent("touchend", eventTarget, [endPoint], [1]);
                chai_1.assert.isFalse(callback1.called, "Callback1 should be disconnected from the Interaction");
                chai_1.assert.isTrue(callback2.called, "Callback2 should still exist on the Interaction");
                div.remove();
            });
        });
    });
});
