"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("AreaPlot", function () {
        describe("Basic Usage", function () {
            var div;
            var xScale;
            var yScale;
            var plot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Area();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
            });
            it("does not throw error when rendering without data", function () {
                chai_1.assert.doesNotThrow(function () { return plot.renderTo(div); }, Error, "rendering without data does not throw an error");
                div.remove();
            });
            it("adds a padding exception to the y scale at the constant y0 value", function () {
                yScale.padProportion(0.1);
                var constantY0 = 30;
                yScale.addIncludedValuesProvider(function (scale) { return [constantY0, constantY0 + 10]; });
                plot.y0(constantY0);
                plot.addDataset(new Plottable.Dataset([{ x: 0, y: constantY0 + 5 }]));
                plot.renderTo(div);
                chai_1.assert.strictEqual(yScale.domain()[0], constantY0, "y Scale doesn't pad beyond 0 when used in a Plots.Area");
                div.remove();
            });
        });
        describe("Rendering", function () {
            var div;
            var xScale;
            var yScale;
            var areaPlot;
            var fill = "steelblue";
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                div = TestMethods.generateDiv();
                var twoPointData = [{ x: 0, y: 0 }, { x: 1, y: 1 }];
                var simpleDataset = new Plottable.Dataset(twoPointData);
                areaPlot = new Plottable.Plots.Area();
                areaPlot.addDataset(simpleDataset);
                areaPlot.x(function (d) { return d.x; }, xScale)
                    .y(function (d) { return d.y; }, yScale);
                areaPlot.y0(function () { return 0; })
                    .attr("fill", function () { return fill; })
                    .attr("stroke", function (d, i) { return d3.rgb(d.x, d.y, i).toString(); })
                    .renderTo(div);
            });
            it("draws area and line with correct data points and correct fill and stroke attributes", function () {
                var content = areaPlot.content();
                var areaPath = content.select(".area");
                areaPlot.datasets().forEach(function (dataset) {
                    var data = dataset.data();
                    var pointsInArea = getPointsInArea(areaPlot, data);
                    TestMethods.assertPathEqualToDataPoints(areaPath.attr("d"), pointsInArea, xScale, yScale);
                    chai_1.assert.strictEqual(areaPath.attr("fill"), fill, "area fill was set correctly");
                    chai_1.assert.strictEqual(areaPath.style("stroke"), "none", "area stroke renders as \"none\"");
                    var linePath = content.select(".line");
                    TestMethods.assertPathEqualToDataPoints(linePath.attr("d"), data, xScale, yScale);
                    chai_1.assert.strictEqual(linePath.attr("stroke"), "rgb(0, 0, 0)", "line stroke was set correctly");
                    chai_1.assert.strictEqual(linePath.style("fill"), "none", "line fill renders as \"none\"");
                });
                div.remove();
            });
            it("draws area appropriately with non-constant y0 values", function () {
                areaPlot.y0(function (d) { return d.y / 2; });
                areaPlot.datasets().forEach(function (dataset) {
                    var data = dataset.data();
                    var areaPath = areaPlot.content().select(".area");
                    var pointsInArea = getPointsInArea(areaPlot, data);
                    TestMethods.assertPathEqualToDataPoints(areaPath.attr("d"), pointsInArea, xScale, yScale);
                });
                div.remove();
            });
            it("places the area before line", function () {
                var content = areaPlot.content();
                var paths = content.selectAll("path").nodes();
                var areaElement = content.select(".area").node();
                var lineElement = content.select(".line").node();
                chai_1.assert.operator(paths.indexOf(areaElement), "<", paths.indexOf(lineElement), "area appended before line");
                div.remove();
            });
            it("removes the plot svg elements when removing Datasets", function () {
                areaPlot.renderTo(div);
                var paths = areaPlot.content().selectAll("path");
                var pathSize = paths.size();
                var dataset = areaPlot.datasets()[0];
                areaPlot.removeDataset(dataset);
                paths = areaPlot.content().selectAll("path");
                chai_1.assert.strictEqual(paths.size(), pathSize - 2, "removing a Dataset cleans up both <path>s associated with it");
                div.remove();
            });
            it("skips data points consisting of NaN and undefined x and y values", function () {
                var areaData = [
                    { x: 0.0, y: 0.0 },
                    { x: 0.2, y: 0.2 },
                    { x: 0.4, y: 0.4 },
                    { x: 0.6, y: 0.6 },
                    { x: 0.8, y: 0.8 },
                ];
                var areaPath = areaPlot.content().select(".area");
                var dataset = areaPlot.datasets()[0];
                var dataWithNaN = areaData.slice();
                dataWithNaN[2] = { x: 0.4, y: NaN };
                dataset.data(dataWithNaN);
                var pointsInArea = getPointsInArea(areaPlot, dataWithNaN.slice(0, 2)).concat(getPointsInArea(areaPlot, dataWithNaN.slice(3, 5)));
                TestMethods.assertPathEqualToDataPoints(areaPath.attr("d"), pointsInArea, xScale, yScale);
                dataWithNaN[2] = { x: NaN, y: 0.4 };
                dataset.data(dataWithNaN);
                TestMethods.assertPathEqualToDataPoints(areaPath.attr("d"), pointsInArea, xScale, yScale);
                var dataWithUndefined = areaData.slice();
                dataWithUndefined[2] = { x: 0.4, y: undefined };
                dataset.data(dataWithUndefined);
                TestMethods.assertPathEqualToDataPoints(areaPath.attr("d"), pointsInArea, xScale, yScale);
                dataWithUndefined[2] = { x: undefined, y: 0.4 };
                dataset.data(dataWithUndefined);
                TestMethods.assertPathEqualToDataPoints(areaPath.attr("d"), pointsInArea, xScale, yScale);
                div.remove();
            });
            it("retains the area class when setting css class via attr", function () {
                var cssClass = "pink";
                areaPlot.attr("class", cssClass);
                areaPlot.renderTo(div);
                var areaPath = areaPlot.content().select(".area");
                chai_1.assert.isTrue(areaPath.classed(cssClass), "it has css class set via attr");
                chai_1.assert.isTrue(areaPath.classed("area"), "it has default css class");
                div.remove();
            });
        });
        describe("Selections", function () {
            var div;
            var xScale;
            var yScale;
            var areaPlot;
            beforeEach(function () {
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                div = TestMethods.generateDiv();
                var simpleDataset = new Plottable.Dataset([{ x: 0, y: 0 }, { x: 1, y: 1 }]);
                areaPlot = new Plottable.Plots.Area();
                areaPlot.addDataset(simpleDataset);
                areaPlot.x(function (d) { return d.x; }, xScale)
                    .y(function (d) { return d.y; }, yScale);
                areaPlot.y0(0)
                    .attr("fill", "steelblue")
                    .attr("stroke", function (d, i, m) { return d3.rgb(d.foo, d.bar, i).toString(); })
                    .renderTo(div);
            });
            it("retrieves all selections with no args", function () {
                var newTwoPointData = [{ x: 2, y: 1 }, { x: 3, y: 2 }];
                areaPlot.addDataset(new Plottable.Dataset(newTwoPointData));
                var allAreas = areaPlot.selections();
                chai_1.assert.strictEqual(allAreas.filter(".line").size(), 2, "2 lines retrieved");
                chai_1.assert.strictEqual(allAreas.filter(".area").size(), 2, "2 areas retrieved");
                div.remove();
            });
            it("retrieves correct selections", function () {
                var twoPointDataset = new Plottable.Dataset([{ x: 0, y: 1 }, { x: 1, y: 2 }]);
                areaPlot.addDataset(twoPointDataset);
                var allAreas = areaPlot.selections([twoPointDataset]);
                chai_1.assert.strictEqual(allAreas.size(), 2, "areas/lines retrieved");
                var selectionData = allAreas.data()[0];
                chai_1.assert.deepEqual(selectionData, twoPointDataset.data(), "new dataset data in selection data");
                div.remove();
            });
        });
        function getPointsInArea(plot, points) {
            var y0Binding = plot.y0();
            var ds = plot.datasets()[0];
            var bottomPoints = points.map(function (d, i) {
                var x = d.x;
                var y0 = y0Binding.accessor(d, i, ds);
                return { x: x, y: y0 };
            }).reverse();
            return points.concat(bottomPoints);
        }
        ;
    });
});
