"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("PiePlot", function () {
        describe("Rendering", function () {
            var div;
            var dataset;
            var data;
            var piePlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                data = [{ value: 5 }, { value: 15 }];
                dataset = new Plottable.Dataset(data);
                piePlot = new Plottable.Plots.Pie();
                piePlot.addDataset(dataset);
                piePlot.sectorValue(function (d) { return d.value; });
                piePlot.renderTo(div);
            });
            it("draws a fill path and a stroke path for each slice", function () {
                var arcPaths = piePlot.content().selectAll(".arc");
                var arcFillPaths = piePlot.content().selectAll(".arc.fill");
                var arcOutlinePaths = piePlot.content().selectAll(".arc.outline");
                chai_1.assert.strictEqual(arcPaths.size(), data.length * 2, "2 paths per datum");
                chai_1.assert.strictEqual(arcFillPaths.size(), data.length, "1 fill path per datum");
                chai_1.assert.strictEqual(arcOutlinePaths.size(), data.length, "1 outline path per datum");
                chai_1.assert.strictEqual(arcFillPaths.style("stroke"), "none", "fill paths have no stroke");
                chai_1.assert.strictEqual(arcOutlinePaths.style("fill"), "none", "outline paths have no fill");
                div.remove();
            });
            it("draws slices proportional in angle to their value", function () {
                var arcPaths = piePlot.content().selectAll(".arc.fill");
                chai_1.assert.strictEqual(arcPaths.size(), 2, "has two sectors");
                var pathString0 = d3.select(arcPaths.node()).attr("d");
                var decomposedPath0 = TestMethods.decomposePath(TestMethods.normalizePath(pathString0));
                var moveCommand0 = decomposedPath0[0];
                chai_1.assert.closeTo(moveCommand0.arguments[0], 0, 1, "starts first arc at top of circle (x)");
                chai_1.assert.operator(moveCommand0.arguments[1], "<", 0, "starts first arc at top of circle (x)");
                var arcCommand0 = decomposedPath0[1];
                chai_1.assert.operator(arcCommand0.arguments[5], ">", 0, "arc ends on the right side of the circle (x)");
                chai_1.assert.closeTo(arcCommand0.arguments[6], 0, 1, "arc ends on the right side of the circle (y)");
                var lineCommand0 = decomposedPath0[2];
                chai_1.assert.closeTo(lineCommand0.arguments[0], 0, 1, "draws line to origin (x)");
                chai_1.assert.closeTo(lineCommand0.arguments[1], 0, 1, "draws line to origin (y)");
                var pathString1 = d3.select(arcPaths.nodes()[1]).attr("d");
                var decomposedPath1 = TestMethods.decomposePath(TestMethods.normalizePath(pathString1));
                var moveCommand1 = decomposedPath1[0];
                chai_1.assert.operator(moveCommand1.arguments[0], ">", 0, "draws line to the right");
                chai_1.assert.closeTo(moveCommand1.arguments[1], 0, 1, "draws line horizontally");
                var arcCommand1 = decomposedPath1[1];
                chai_1.assert.closeTo(arcCommand1.arguments[5], 0, 1, "ends at x origin");
                chai_1.assert.operator(arcCommand1.arguments[6], "<", 0, "ends above 0");
                var lineCommand1 = decomposedPath1[2];
                chai_1.assert.closeTo(lineCommand1.arguments[0], 0, 1, "draws line to origin (x)");
                chai_1.assert.closeTo(lineCommand1.arguments[1], 0, 1, "draws line to origin (y)");
                div.remove();
            });
            it("uses Plottable colors for sectors by default", function () {
                var arcPaths = piePlot.content().selectAll(".arc.fill");
                var plottableColors = new Plottable.Scales.Color().range();
                arcPaths.each(function (d, i) {
                    chai_1.assert.deepEqual(d3.color(d3.select(this).attr("fill")), d3.color(plottableColors[i]), "sector with index " + i + " has the correct fill color");
                });
                div.remove();
            });
            it("updates slices when data changes", function () {
                var originalPathStrings = [];
                piePlot.content().selectAll("path").each(function () {
                    originalPathStrings.push(d3.select(this).attr("d"));
                });
                chai_1.assert.lengthOf(originalPathStrings, data.length * 2, "2 paths per datum");
                var oneMoreSliceData = data.slice();
                oneMoreSliceData.push({ value: 5 });
                dataset.data(oneMoreSliceData);
                var oneMoreSlicePathStrings = [];
                piePlot.content().selectAll("path").each(function () {
                    oneMoreSlicePathStrings.push(d3.select(this).attr("d"));
                });
                chai_1.assert.lengthOf(oneMoreSlicePathStrings, oneMoreSliceData.length * 2, "2 paths per datum");
                originalPathStrings.forEach(function (pathString, index) {
                    chai_1.assert.notStrictEqual(pathString, oneMoreSlicePathStrings[index], "slices were updated when data changed");
                });
                div.remove();
            });
            it("can set innerRadius()", function () {
                var expectedInnerRadius = 5;
                chai_1.assert.strictEqual(piePlot.innerRadius(expectedInnerRadius), piePlot, "setter returns the calling pie plot");
                var arcPaths = piePlot.content().selectAll(".arc.fill");
                chai_1.assert.strictEqual(arcPaths.size(), 2, "has two sectors");
                var decomposedPath0 = TestMethods.decomposePath(TestMethods.normalizePath(d3.select(arcPaths.node()).attr("d")));
                var lineBeforeInnerArcCommand0 = decomposedPath0[2];
                chai_1.assert.closeTo(lineBeforeInnerArcCommand0.arguments[0], expectedInnerRadius, 1, "inner arc starts at correct x");
                chai_1.assert.closeTo(lineBeforeInnerArcCommand0.arguments[1], 0, 1, "inner arc starts at correct y");
                var innerArcCommand0 = decomposedPath0[3];
                chai_1.assert.closeTo(innerArcCommand0.arguments[0], expectedInnerRadius, 1, "inner arc has correct rx");
                chai_1.assert.closeTo(innerArcCommand0.arguments[1], expectedInnerRadius, 1, "inner arc has correct ry");
                chai_1.assert.closeTo(innerArcCommand0.arguments[5], 0, 1, "inner arc ends at correct position (x)");
                chai_1.assert.closeTo(innerArcCommand0.arguments[6], -expectedInnerRadius, 1, "inner arc ends at correct position (y)");
                div.remove();
            });
            it("can set outerRadius()", function () {
                var expectedOuterRadius = 150;
                chai_1.assert.strictEqual(piePlot.outerRadius(function () { return expectedOuterRadius; }), piePlot, "setter returns the calling pie plot");
                var arcPaths = piePlot.content().selectAll(".arc.fill");
                chai_1.assert.strictEqual(arcPaths.size(), 2, "has two sectors");
                var decomposedPath0 = TestMethods.decomposePath(TestMethods.normalizePath(d3.select(arcPaths.node()).attr("d")));
                var lineCommand0 = decomposedPath0[0];
                chai_1.assert.closeTo(lineCommand0.arguments[0], 0, 1, "starts outer arc at correct position (x)");
                chai_1.assert.closeTo(lineCommand0.arguments[1], -expectedOuterRadius, 1, "starts outer arc at correct position (y)");
                var outerArcCommand0 = decomposedPath0[1];
                chai_1.assert.closeTo(outerArcCommand0.arguments[0], expectedOuterRadius, 1, "outer arc has correct rx");
                chai_1.assert.closeTo(outerArcCommand0.arguments[1], expectedOuterRadius, 1, "outer arc has correct ry");
                chai_1.assert.closeTo(outerArcCommand0.arguments[5], expectedOuterRadius, 1, "outer arc ends at correct position (x)");
                chai_1.assert.closeTo(outerArcCommand0.arguments[6], 0, 1, "outer arc ends at correct position (y)");
                div.remove();
            });
            it("can set startAngle()", function () {
                var expectedStartAngle = Math.PI;
                chai_1.assert.strictEqual(piePlot.startAngle(expectedStartAngle), piePlot, "setter returns the calling pie plot");
                var arcPaths = piePlot.content().selectAll(".arc.fill");
                chai_1.assert.strictEqual(arcPaths.size(), 2, "has two sectors");
                var pathString0 = d3.select(arcPaths.node()).attr("d");
                var decomposedPath0 = TestMethods.decomposePath(TestMethods.normalizePath(pathString0));
                var moveCommand0 = decomposedPath0[0];
                chai_1.assert.closeTo(moveCommand0.arguments[0], 0, 1, "starts first arc at the bottom of the circle (x)");
                chai_1.assert.isAbove(moveCommand0.arguments[1], 0, "starts first arc at the bottom of circle (y)");
                var arcCommand0 = decomposedPath0[1];
                chai_1.assert.isBelow(arcCommand0.arguments[5], 0, "arc ends on the left side of the circle (x)");
                chai_1.assert.isAbove(arcCommand0.arguments[6], 0, "arc ends on the bottom side of the circle (y)");
                var lineCommand0 = decomposedPath0[2];
                chai_1.assert.closeTo(lineCommand0.arguments[0], 0, 1, "draws line to origin (x)");
                chai_1.assert.closeTo(lineCommand0.arguments[1], 0, 1, "draws line to origin (y)");
                var pathString1 = d3.select(arcPaths.nodes()[1]).attr("d");
                var decomposedPath1 = TestMethods.decomposePath(TestMethods.normalizePath(pathString1));
                var moveCommand1 = decomposedPath1[0];
                chai_1.assert.isBelow(moveCommand1.arguments[0], 0, "starts second arc at the end of the first arc");
                chai_1.assert.isAbove(moveCommand1.arguments[1], 0, "starts secnond arc at the end of the first arc");
                var arcCommand1 = decomposedPath1[1];
                chai_1.assert.closeTo(arcCommand1.arguments[5], 0, 1, "ends second arc at x origin");
                chai_1.assert.isBelow(arcCommand1.arguments[6], 0, "ends second arc below 0");
                var lineCommand1 = decomposedPath1[2];
                chai_1.assert.closeTo(lineCommand1.arguments[0], 0, 1, "draws line to origin (x)");
                chai_1.assert.closeTo(lineCommand1.arguments[1], 0, 1, "draws line to origin (y)");
                div.remove();
            });
            it("can set endAngle()", function () {
                var expectedEndAngle = Math.PI;
                chai_1.assert.strictEqual(piePlot.endAngle(expectedEndAngle), piePlot, "setter returns the calling pie plot");
                var arcPaths = piePlot.content().selectAll(".arc.fill");
                chai_1.assert.strictEqual(arcPaths.size(), 2, "has two sectors");
                var pathString0 = d3.select(arcPaths.node()).attr("d");
                var decomposedPath0 = TestMethods.decomposePath(TestMethods.normalizePath(pathString0));
                var moveCommand0 = decomposedPath0[0];
                chai_1.assert.closeTo(moveCommand0.arguments[0], 0, 1, "starts first arc at the top of the circle (x)");
                chai_1.assert.isBelow(moveCommand0.arguments[1], 0, "starts first arc at the top of circle (y)");
                var arcCommand0 = decomposedPath0[1];
                chai_1.assert.isAbove(arcCommand0.arguments[5], 0, "arc ends on the right side of the circle (x)");
                chai_1.assert.isBelow(arcCommand0.arguments[6], 0, "arc ends on the right side of the circle (y)");
                var lineCommand0 = decomposedPath0[2];
                chai_1.assert.closeTo(lineCommand0.arguments[0], 0, 1, "draws line to origin (x)");
                chai_1.assert.closeTo(lineCommand0.arguments[1], 0, 1, "draws line to origin (y)");
                var pathString1 = d3.select(arcPaths.nodes()[1]).attr("d");
                var decomposedPath1 = TestMethods.decomposePath(TestMethods.normalizePath(pathString1));
                var moveCommand1 = decomposedPath1[0];
                chai_1.assert.isAbove(moveCommand1.arguments[0], 0, "starts second arc at the top right of the circle");
                chai_1.assert.isBelow(moveCommand1.arguments[1], 0, "starts second arc at the top right of the circle");
                var arcCommand1 = decomposedPath1[1];
                chai_1.assert.closeTo(arcCommand1.arguments[5], 0, 1, "ends second arc at the bottom of the circle (x origin)");
                chai_1.assert.isAbove(arcCommand1.arguments[6], 0, "ends second arc at the bottom of the circle");
                var lineCommand1 = decomposedPath1[2];
                chai_1.assert.closeTo(lineCommand1.arguments[0], 0, 1, "draws line to origin (x)");
                chai_1.assert.closeTo(lineCommand1.arguments[1], 0, 1, "draws line to origin (y)");
                div.remove();
            });
            it("updates slices when data changes and render correctly w.r.t. scale changes", function () {
                var newDataset = new Plottable.Dataset([{ value: 500 }, { value: 600 }]);
                // reset database for this test's sake
                piePlot.datasets([newDataset]);
                var originalStringPaths = [];
                piePlot.content().selectAll("path").each(function () {
                    var pathString = d3.select(this).attr("d");
                    chai_1.assert.notInclude(pathString, "NaN", "original pathString should not contain NaN");
                    originalStringPaths.push(pathString);
                });
                var newData = [{ value: 10 }, { value: 20 }];
                newDataset.data(newData);
                piePlot.content().selectAll("path").each(function (path, index) {
                    var pathString = d3.select(this).attr("d");
                    chai_1.assert.notInclude(pathString, "NaN", "new pathString should not contain NaN");
                    chai_1.assert.notStrictEqual(pathString, originalStringPaths[index], "new pathString should not equal old one");
                });
                div.remove();
            });
        });
        describe("Labels", function () {
            var div;
            var piePlot;
            beforeEach(function () {
                div = TestMethods.generateDiv(500, 500);
                piePlot = new Plottable.Plots.Pie();
                piePlot.sectorValue(function (d) { return d.value; });
                piePlot.labelsEnabled(true);
            });
            it("does not erase or add labels when rendered twice", function () {
                var data = [
                    { value: 1 },
                    { value: 2 },
                    { value: 3 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset);
                piePlot.renderTo(div);
                var labels = piePlot.content().selectAll("text");
                chai_1.assert.strictEqual(labels.size(), data.length, "one label per slice");
                piePlot.render();
                labels = piePlot.content().selectAll("text");
                chai_1.assert.strictEqual(labels.size(), data.length, "one label per slice after re-render()ing");
                div.remove();
            });
            it("updates labels when data changes", function () {
                var data1 = [
                    { value: 1 },
                    { value: 1 },
                    { value: 1 },
                ];
                var dataset = new Plottable.Dataset(data1);
                piePlot.addDataset(dataset);
                piePlot.renderTo(div);
                var labels = piePlot.content().selectAll("text");
                chai_1.assert.strictEqual(labels.size(), data1.length, "one label per datum");
                labels.each(function () {
                    var labelText = d3.select(this).text();
                    chai_1.assert.strictEqual(labelText, "1", "label has correct text");
                });
                var data2 = [
                    { value: 2 },
                    { value: 2 },
                ];
                dataset.data(data2);
                labels = piePlot.content().selectAll("text");
                chai_1.assert.strictEqual(labels.size(), data2.length, "one label per datum");
                labels.each(function () {
                    var labelText = d3.select(this).text();
                    chai_1.assert.strictEqual(labelText, "2", "label text was updated");
                });
                div.remove();
            });
            it("removes labels when they are disabled after rendering", function () {
                var data = [
                    { value: 1 },
                    { value: 1 },
                    { value: 1 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset);
                piePlot.renderTo(div);
                var labels = piePlot.content().selectAll("text");
                chai_1.assert.strictEqual(labels.size(), data.length, "one label per datum");
                piePlot.labelsEnabled(false);
                labels = piePlot.content().selectAll("text");
                chai_1.assert.strictEqual(labels.size(), 0, "labels were removed");
                div.remove();
            });
            it("hides labels if slices are too small", function () {
                var data = [
                    { key: "A", value: 1 },
                    { key: "B", value: 50 },
                    { key: "C", value: 1 },
                    { key: "D", value: 50 },
                    { key: "E", value: 1 },
                    { key: "F", value: 50 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset);
                piePlot.renderTo(div);
                var labelGs = piePlot.content().select(".label-area").selectAll(".label-area > g");
                labelGs.each(function (d, i) {
                    var visibility = d3.select(this).style("visibility");
                    if (data[i].value === 1) {
                        chai_1.assert.strictEqual(visibility, "hidden", "label " + i + " hidden when slice is too small");
                    }
                    else {
                        chai_1.assert.include(["visible", "inherit"], visibility, "label with index " + i + " shown when slice is appropriately sized");
                    }
                });
                div.remove();
            });
            it("uses its formatter to format labels", function () {
                var data = [
                    { value: 5 },
                    { value: 15 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset);
                piePlot.labelFormatter(function (n) { return n + " m"; });
                piePlot.renderTo(div);
                var texts = div.selectAll("text").nodes().map(function (n) { return d3.select(n).text(); });
                chai_1.assert.lengthOf(texts, 2, "both labels are drawn");
                chai_1.assert.strictEqual(texts[0], "5 m", "The formatter was used to format the label for slice index 0");
                chai_1.assert.strictEqual(texts[1], "15 m", "The formatter was used to format the label for slice index 1");
                div.remove();
            });
            it("hides labels if they don't fit", function () {
                var data = [
                    { value: 1 },
                    { value: 50 },
                    { value: 1 },
                    { value: 50 },
                    { value: 1 },
                    { value: 50 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset);
                piePlot.renderTo(div);
                var texts = div.selectAll("text");
                chai_1.assert.strictEqual(texts.size(), data.length, "One label is rendered for each piece of data");
                texts.each(function (d, i) {
                    var visibility = d3.select(this).style("visibility");
                    if (i % 2 === 0) {
                        chai_1.assert.strictEqual(visibility, "hidden", "label " + i + " hidden when slice is too small");
                    }
                    else {
                        chai_1.assert.include(["visible", "inherit"], visibility, "label " + i + " shown when slice is appropriately sized");
                    }
                });
                div.remove();
            });
            it("hides labels outside of the render area", function () {
                var data = [
                    { value: 5000 },
                    { value: 5000 },
                    { value: 5000 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset).outerRadius(500);
                piePlot.renderTo(div);
                var texts = div.selectAll("text");
                chai_1.assert.strictEqual(texts.size(), data.length, "One label is rendered for each piece of data");
                texts.each(function (d, i) {
                    var visibility = d3.select(this).style("visibility");
                    if (i === 1) {
                        chai_1.assert.strictEqual(visibility, "hidden", "label " + i + " hidden when cut off by the lower margin");
                    }
                    else {
                        chai_1.assert.include(["visible", "inherit"], visibility, "label " + i + " shown when in the renderArea");
                    }
                });
                div.remove();
            });
            it("does not show labels for invalid data", function () {
                var data = [
                    { value: 1 },
                    { value: "value" },
                    { value: 2 },
                ];
                var dataset = new Plottable.Dataset(data);
                piePlot.addDataset(dataset);
                piePlot.renderTo(div);
                var texts = div.selectAll("text").nodes().map(function (n) { return d3.select(n).text(); });
                chai_1.assert.lengthOf(texts, 2, "One label is rendered for each valid datum");
                chai_1.assert.strictEqual(texts[0], "1", "Label for the first valid datum is shown");
                chai_1.assert.strictEqual(texts[1], "2", "Label for the second valid datum is shown");
                div.remove();
            });
        });
        describe("Selections", function () {
            var div;
            var dataset;
            var data;
            var piePlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                data = [{ value: 5 }, { value: 15 }];
                dataset = new Plottable.Dataset(data);
                piePlot = new Plottable.Plots.Pie();
                piePlot.addDataset(dataset);
                piePlot.sectorValue(function (d) { return d.value; });
                piePlot.renderTo(div);
            });
            it("retrieves all dataset selections when no arguments are specified", function () {
                var allSectors = piePlot.selections();
                chai_1.assert.strictEqual(allSectors.size(), data.length * 2, "all sectors retrieved");
                chai_1.assert.strictEqual(allSectors.filter(".fill").size(), 2, "each sector has a fill path");
                chai_1.assert.strictEqual(allSectors.filter(".outline").size(), 2, "each sector has an outline path");
                div.remove();
            });
            it("retrieves correct selections", function () {
                var allSectors = piePlot.selections([dataset]);
                chai_1.assert.strictEqual(allSectors.size(), data.length * 2, "all sectors retrieved");
                chai_1.assert.strictEqual(allSectors.filter(".fill").size(), 2, "each sector has a fill path");
                chai_1.assert.strictEqual(allSectors.filter(".outline").size(), 2, "each sector has an outline path");
                chai_1.assert.includeMembers(allSectors.data(), data, "dataset data in selection data");
                div.remove();
            });
            it("skips invalid Datasets", function () {
                var allSectors = piePlot.selections([new Plottable.Dataset([])]);
                chai_1.assert.strictEqual(allSectors.size(), 0, "no sectors retrieved");
                div.remove();
            });
        });
        describe("Entities", function () {
            var div;
            var dataset;
            var data = [{ value: 5 }, { value: 15 }];
            var piePlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                dataset = new Plottable.Dataset();
                piePlot = new Plottable.Plots.Pie();
                dataset.data(data);
                piePlot.addDataset(dataset);
                piePlot.sectorValue(function (d) { return d.value; });
                piePlot.renderTo(div);
            });
            it("returns both a fill path and a stroke path in each Entity's selection", function () {
                var entities = piePlot.entities();
                chai_1.assert.lengthOf(entities, data.length, "returned one Entity per datum");
                entities.forEach(function (entity) {
                    chai_1.assert.isNotNull(entity.selection, "entity has a fill selection");
                    chai_1.assert.isNotNull(entity.strokeSelection, "entity has a stroke selection");
                });
                div.remove();
            });
            it("retrieves the entity under each given point with entitiesAt()", function () {
                var OUTER_RADIUS = 200;
                piePlot.outerRadius(OUTER_RADIUS);
                /* tslint:disable no-shadowed-variable */
                var data = [
                    { value: 500 },
                    { value: 5 },
                    { value: 5 },
                    { value: 5 },
                    { value: 5 }
                ];
                /* tslint:enable no-shadowed-variable */
                dataset.data(data);
                var totalValue = data.map(function (d) { return d.value; }).reduce(function (previous, current) { return previous + current; });
                var runningTotal = 0;
                var clickAngles = data.map(function (d) {
                    var angle = (runningTotal + d.value / 2) / totalValue * 2 * Math.PI;
                    runningTotal += d.value;
                    return angle;
                });
                var clicks = clickAngles.map(function (angle) {
                    return {
                        x: OUTER_RADIUS + Math.sin(angle),
                        y: OUTER_RADIUS - Math.cos(angle),
                    };
                });
                clicks.forEach(function (point, i) {
                    var entities = piePlot.entitiesAt(point);
                    chai_1.assert.lengthOf(entities, 1, "exactly one entity is selected");
                    TestMethods.assertPlotEntitiesEqual(entities[0], piePlot.entities()[i], "the Entity with index " + i + " is selected when clicking at [" + point.x + ", " + point.y + "]");
                });
                var entities = piePlot.entitiesAt({ x: 0, y: 0 });
                chai_1.assert.lengthOf(entities, 0, "no entities returned");
                div.remove();
            });
            it("returns nothing for points within innerRadius() or outside of outerRadius() with entitiesAt()", function () {
                var INNER_RADIUS = 100;
                piePlot.innerRadius(INNER_RADIUS);
                var clickInCenter = {
                    x: 200,
                    y: 200,
                };
                var entitiesInCenter = piePlot.entitiesAt(clickInCenter);
                chai_1.assert.lengthOf(entitiesInCenter, 0, "no entities returned when clicking inside innerRadius()");
                var OUTER_RADIUS = 150;
                piePlot.outerRadius(OUTER_RADIUS);
                var clickBetweenRadii = {
                    x: 200,
                    y: 200 + (INNER_RADIUS + OUTER_RADIUS) / 2,
                };
                var entitiesBetweenRadii = piePlot.entitiesAt(clickBetweenRadii);
                TestMethods.assertPlotEntitiesEqual(entitiesBetweenRadii[0], piePlot.entities()[1], "retrieved the correct entity");
                var clickOutsideOuterRadius = {
                    x: 200,
                    y: 200 + OUTER_RADIUS + 1,
                };
                var entitiesOutsideOuterRadius = piePlot.entitiesAt(clickOutsideOuterRadius);
                chai_1.assert.lengthOf(entitiesOutsideOuterRadius, 0, "no entities returned when clicking outside outerRadius()");
                div.remove();
            });
            it("retrieves undefined for entitiesNearest when no entities rendered", function () {
                piePlot.datasets([new Plottable.Dataset([])]);
                piePlot.renderTo(div);
                var closest = piePlot.entityNearest({
                    x: piePlot.width() / 2,
                    y: piePlot.height() / 2,
                });
                chai_1.assert.strictEqual(closest, undefined, "no datum has been retrieved");
                div.remove();
            });
        });
        describe("Fail safe tests", function () {
            it("renders correctly with no Datasets", function () {
                var DIV_WIDTH = 400;
                var DIV_HEIGHT = 400;
                var div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var piePlot = new Plottable.Plots.Pie();
                piePlot.sectorValue(function (d) { return d.value; });
                chai_1.assert.doesNotThrow(function () { return piePlot.renderTo(div); }, Error);
                chai_1.assert.strictEqual(piePlot.width(), DIV_WIDTH, "was allocated width");
                chai_1.assert.strictEqual(piePlot.height(), DIV_HEIGHT, "was allocated height");
                div.remove();
            });
            it("does not render slices for undefined, NaN, strings, or negative numbers", function () {
                var div = TestMethods.generateDiv();
                var dataWithBadValue = [
                    { v: 1 },
                    { v: undefined },
                    { v: 1 },
                    { v: NaN },
                    { v: 1 },
                    { v: "Bad String" },
                    { v: 1 },
                    { v: -100 },
                ];
                var piePlot = new Plottable.Plots.Pie();
                piePlot.addDataset(new Plottable.Dataset(dataWithBadValue));
                piePlot.sectorValue(function (d) { return d.v; });
                piePlot.renderTo(div);
                var elementsDrawnSel = piePlot.content().selectAll(".arc.fill");
                chai_1.assert.strictEqual(elementsDrawnSel.size(), 4, "There should be exactly 4 slices in the pie chart, representing the valid values");
                chai_1.assert.lengthOf(piePlot.entities(), 4, "there should be exactly 4 entities, representing the valid values");
                for (var i = 0; i < 4; i++) {
                    var startAngle = piePlot._startAngles[i];
                    var endAngle = piePlot._endAngles[i];
                    chai_1.assert.closeTo(endAngle - startAngle, Math.PI / 2, 0.001, "slice with index " + i + " is a quarter of the pie");
                }
                div.remove();
            });
        });
    });
});
