"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("RectanglePlot", function () {
        describe("rendering", function () {
            it("renders rectangles with the input x1, x2, y1, y2 edges", function () {
                var data = [
                    { x: 0, y: 0, x2: 1, y2: 1 },
                    { x: 1, y: 1, x2: 2, y2: 2 },
                    { x: 2, y: 2, x2: 3, y2: 3 },
                    { x: 3, y: 3, x2: 4, y2: 4 },
                    { x: 4, y: 4, x2: 5, y2: 5 },
                ];
                var div = TestMethods.generateDiv();
                var xScale = new Plottable.Scales.Linear();
                var yScale = new Plottable.Scales.Linear();
                var plot = new Plottable.Plots.Rectangle();
                var xAccessor = function (d) { return d.x; };
                var yAccessor = function (d) { return d.y; };
                var x2Accessor = function (d) { return d.x2; };
                var y2Accessor = function (d) { return d.y2; };
                plot.x(xAccessor, xScale).x2(x2Accessor);
                plot.y(yAccessor, yScale).y2(y2Accessor);
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var rects = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(rects.size(), data.length, "one rectangle per datum");
                rects.each(function (d, i) {
                    var rect = d3.select(this);
                    chai_1.assert.closeTo(TestMethods.numAttr(rect, "height"), Math.abs(yScale.scale(yAccessor(d)) - yScale.scale(y2Accessor(d))), window.Pixel_CloseTo_Requirement, "rect " + i + " height is correct");
                    chai_1.assert.closeTo(TestMethods.numAttr(rect, "width"), Math.abs(xScale.scale(xAccessor(d)) - xScale.scale(x2Accessor(d))), window.Pixel_CloseTo_Requirement, "rect " + i + " width is correct");
                    chai_1.assert.closeTo(TestMethods.numAttr(rect, "x"), Math.min(xScale.scale(xAccessor(d)), xScale.scale(x2Accessor(d))), window.Pixel_CloseTo_Requirement, "rect " + i + " x coordinate is correct");
                    chai_1.assert.closeTo(TestMethods.numAttr(rect, "y"), Math.min(yScale.scale(yAccessor(d)), yScale.scale(y2Accessor(d))), window.Pixel_CloseTo_Requirement, "rect " + i + " y coordinate is correct");
                });
                div.remove();
            });
        });
        describe("retreiving entities", function () {
            var div;
            var xScale;
            var yScale;
            var plot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Rectangle();
                plot.x(function (d) { return d.x; }, xScale).x2(function (d) { return d.x2; });
                plot.y(function (d) { return d.y; }, yScale).y2(function (d) { return d.y2; });
            });
            function createNonIntersectingRectangleData(count) {
                return Plottable.Utils.Math.range(0, count).map(function (rectangleNumber) {
                    return {
                        x: rectangleNumber,
                        y: rectangleNumber,
                        x2: rectangleNumber + 1,
                        y2: rectangleNumber + 1,
                    };
                });
            }
            function createIntersectingRectangleData(count) {
                return Plottable.Utils.Math.range(0, count).map(function (rectangleNumber) {
                    return {
                        x: rectangleNumber,
                        y: rectangleNumber,
                        x2: rectangleNumber + 1.5,
                        y2: rectangleNumber + 1.5,
                    };
                });
            }
            it("retrieves the correct entity under a point", function () {
                var data = createNonIntersectingRectangleData(5);
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var entities = plot.entitiesAt({ x: (xScale.scale(data[2].x) + xScale.scale(data[3].x)) / 2,
                    y: (yScale.scale(data[2].y) + yScale.scale(data[3].y)) / 2, });
                chai_1.assert.lengthOf(entities, 1, "found only one entity when querying a point inside a rectangle");
                chai_1.assert.strictEqual(entities[0].index, 2, "entity retrieved is at index 2");
                div.remove();
            });
            it("retrieves correct entities under a point when rectangles intersect", function () {
                var data = createIntersectingRectangleData(2);
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var intersectingEntities = plot.entitiesAt({ x: xScale.scale(data[1].x), y: yScale.scale(data[1].y) });
                chai_1.assert.lengthOf(intersectingEntities, 2, "two entities when querying a point in intersection");
                chai_1.assert.strictEqual(intersectingEntities[0].index, 0, "entity retrieved is at index 0");
                chai_1.assert.strictEqual(intersectingEntities[1].index, 1, "entity retrieved is at index 1");
                var nonIntersectingEntities = plot.entitiesAt({ x: xScale.scale(data[1].x2), y: yScale.scale(data[1].y2) });
                chai_1.assert.lengthOf(nonIntersectingEntities, 1, "found only one entity when querying a point inside the second rectangle");
                chai_1.assert.strictEqual(nonIntersectingEntities[0].index, 1, "entity retrieved is at index 1");
                nonIntersectingEntities = plot.entitiesAt({ x: xScale.scale(data[0].x), y: yScale.scale(data[1].y) });
                chai_1.assert.lengthOf(nonIntersectingEntities, 1, "found only one entity when querying a point inside the first rectangle");
                chai_1.assert.strictEqual(nonIntersectingEntities[0].index, 0, "entity retrieved is at index 0");
                div.remove();
            });
            it("retrieves the entities that intersect with the bounding box", function () {
                var data = createNonIntersectingRectangleData(5);
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var entities = plot.entitiesIn({
                    topLeft: { x: (xScale.scale(data[1].x) + xScale.scale(data[2].x)) / 2,
                        y: (yScale.scale(data[2].y) + yScale.scale(data[3].y)) / 2, },
                    bottomRight: { x: (xScale.scale(data[2].x) + xScale.scale(data[3].x)) / 2,
                        y: (yScale.scale(data[1].y) + yScale.scale(data[2].y)) / 2, },
                });
                chai_1.assert.lengthOf(entities, 2, "retrieved 2 entities intersect with the box");
                chai_1.assert.strictEqual(entities[0].index, 1, "the entity of index 1 is retrieved");
                chai_1.assert.strictEqual(entities[1].index, 2, "the entity of index 2 is retrieved");
                div.remove();
            });
            it("retrieves the entities that intersect with the given ranges", function () {
                var data = createNonIntersectingRectangleData(5);
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var entities = plot.entitiesIn({
                    min: (xScale.scale(data[1].x) + xScale.scale(data[2].x)) / 2,
                    max: (xScale.scale(data[2].x) + xScale.scale(data[3].x)) / 2,
                }, {
                    min: (yScale.scale(data[2].y) + yScale.scale(data[3].y)) / 2,
                    max: (yScale.scale(data[1].y) + yScale.scale(data[2].y)) / 2,
                });
                chai_1.assert.lengthOf(entities, 2, "retrieved 2 entities intersect with the box");
                chai_1.assert.strictEqual(entities[0].index, 1, "the entity of index 1 is retrieved");
                chai_1.assert.strictEqual(entities[1].index, 2, "the entity of index 2 is retrieved");
                div.remove();
            });
            it("retrieves undefined from entityNearest when no entities are rendered", function () {
                plot.addDataset(new Plottable.Dataset([]));
                plot.renderTo(div);
                var closest = plot.entityNearest({
                    x: plot.width() / 2,
                    y: plot.height() / 2,
                });
                chai_1.assert.strictEqual(closest, undefined, "no datum has been retrieved");
                div.remove();
            });
        });
        describe("autoranging the x and y scales", function () {
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv();
            });
            it("adjusts the xScale domain with respect to the yScale domain when autorangeMode is set to x", function () {
                var data = [
                    { y: "A", x: 0, x2: 1 },
                    { y: "B", x: 1, x2: 2 },
                ];
                var xScale = new Plottable.Scales.Linear();
                var yScale = new Plottable.Scales.Category();
                xScale.padProportion(0);
                var plot = new Plottable.Plots.Rectangle();
                plot.x(function (d) { return d.x; }, xScale);
                plot.x2(function (d) { return d.x2; });
                plot.y(function (d) { return d.y; }, yScale);
                plot.addDataset(new Plottable.Dataset(data));
                plot.autorangeMode("x");
                plot.renderTo(div);
                chai_1.assert.deepEqual(xScale.domain(), [0, 2], "x domain includes both visible segments");
                yScale.domain(["A"]);
                chai_1.assert.deepEqual(xScale.domain(), [0, 1], "x domain includes only the visible segment (first)");
                yScale.domain(["B"]);
                chai_1.assert.deepEqual(xScale.domain(), [1, 2], "x domain includes only the visible segment (second)");
                div.remove();
            });
            it("adjusts the yScale domain with respect to the xScale domain when autorangeMode is set to y", function () {
                var data = [
                    { x: "A", y: 0, y2: 1 },
                    { x: "B", y: 1, y2: 2 },
                ];
                var xScale = new Plottable.Scales.Category();
                var yScale = new Plottable.Scales.Linear();
                yScale.padProportion(0);
                var plot = new Plottable.Plots.Rectangle();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
                plot.y2(function (d) { return d.y2; });
                plot.addDataset(new Plottable.Dataset(data));
                plot.autorangeMode("y");
                plot.renderTo(div);
                chai_1.assert.deepEqual(yScale.domain(), [0, 2], "y domain includes both visible segments");
                xScale.domain(["A"]);
                chai_1.assert.deepEqual(yScale.domain(), [0, 1], "y domain includes only the visible segment (first)");
                xScale.domain(["B"]);
                chai_1.assert.deepEqual(yScale.domain(), [1, 2], "y domain includes only the visible segment (second)");
                div.remove();
            });
        });
        describe("using invalid data", function () {
            it("does not draw rectangles for data points containing NaN", function () {
                var div = TestMethods.generateDiv();
                var data = [
                    { x: "A", y: 1, y2: 2 },
                    { x: "B", y: 2, y2: 3 },
                    { x: "C", y: 3, y2: NaN },
                ];
                var xScale = new Plottable.Scales.Category();
                var yScale = new Plottable.Scales.Linear();
                var plot = new Plottable.Plots.Rectangle();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
                plot.y2(function (d) { return d.y2; });
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var rectangles = plot.selections();
                chai_1.assert.strictEqual(rectangles.size(), data.length - 1, "rectangle per valid datum");
                rectangles.each(function (d, i) {
                    var rect = d3.select(this);
                    chai_1.assert.isFalse(Plottable.Utils.Math.isNaN(TestMethods.numAttr(rect, "x")), "x is not NaN for rect " + i);
                    chai_1.assert.isFalse(Plottable.Utils.Math.isNaN(TestMethods.numAttr(rect, "y")), "y is not NaN for rect " + i);
                    chai_1.assert.isFalse(Plottable.Utils.Math.isNaN(TestMethods.numAttr(rect, "height")), "height is not NaN for rect " + i);
                    chai_1.assert.isFalse(Plottable.Utils.Math.isNaN(TestMethods.numAttr(rect, "width")), "width is not NaN for rect " + i);
                });
                div.remove();
            });
        });
        describe("using category scales", function () {
            var div;
            var xScale;
            var yScale;
            var plot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Category();
                plot = new Plottable.Plots.Rectangle();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
            });
            it("renders rectangles with the correct x, y, width, and ehgiht", function () {
                var data = [
                    { x: "A", y: "U" },
                    { x: "B", y: "U" },
                    { x: "A", y: "V" },
                    { x: "B", y: "V" },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var rects = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(rects.size(), data.length, "rect for each datum");
                rects.each(function (d, i) {
                    var rect = d3.select(this);
                    var rectCenterX = TestMethods.numAttr(rect, "x") + TestMethods.numAttr(rect, "width") / 2;
                    var rectCenterY = TestMethods.numAttr(rect, "y") + TestMethods.numAttr(rect, "height") / 2;
                    chai_1.assert.closeTo(xScale.scale(d.x), rectCenterX, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in x is scaled version of input data");
                    chai_1.assert.closeTo(yScale.scale(d.y), rectCenterY, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in y is scaled version of input data");
                });
                div.remove();
            });
            it("renders rectangles when data is set after construction", function () {
                var dataset = new Plottable.Dataset();
                plot.addDataset(dataset);
                plot.renderTo(div);
                var data = [
                    { x: "X", y: "Z" },
                    { x: "Y", y: "T" },
                ];
                dataset.data(data);
                var rects = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(rects.size(), data.length, "rect for each datum");
                rects.each(function (d, i) {
                    var rect = d3.select(this);
                    var rectCenterX = TestMethods.numAttr(rect, "x") + TestMethods.numAttr(rect, "width") / 2;
                    var rectCenterY = TestMethods.numAttr(rect, "y") + TestMethods.numAttr(rect, "height") / 2;
                    chai_1.assert.closeTo(xScale.scale(d.x), rectCenterX, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in x is scaled version of input data");
                    chai_1.assert.closeTo(yScale.scale(d.y), rectCenterY, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in y is scaled version of input data");
                });
                div.remove();
            });
            it("renders rectangles even when there isn't data for every spot", function () {
                var data = [
                    { x: "A", y: "W" },
                    { x: "B", y: "X" },
                    { x: "C", y: "Y" },
                    { x: "D", y: "Z" },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                var rects = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(rects.size(), data.length, "rect for each datum");
                rects.each(function (d, i) {
                    var rect = d3.select(this);
                    var rectCenterX = TestMethods.numAttr(rect, "x") + TestMethods.numAttr(rect, "width") / 2;
                    var rectCenterY = TestMethods.numAttr(rect, "y") + TestMethods.numAttr(rect, "height") / 2;
                    chai_1.assert.closeTo(xScale.scale(d.x), rectCenterX, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in x is scaled version of input data");
                    chai_1.assert.closeTo(yScale.scale(d.y), rectCenterY, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in y is scaled version of input data");
                });
                div.remove();
            });
            it("renders rectangles in the correct x and y locations even with a reversed y domain", function () {
                var data = [
                    { x: "A", y: "U" },
                    { x: "B", y: "U" },
                    { x: "A", y: "V" },
                    { x: "B", y: "V" },
                ];
                plot.addDataset(new Plottable.Dataset(data));
                plot.renderTo(div);
                yScale.domain(yScale.domain().reverse());
                var rects = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(rects.size(), data.length, "rect for each datum");
                rects.each(function (d, i) {
                    var rect = d3.select(this);
                    var rectCenterX = TestMethods.numAttr(rect, "x") + TestMethods.numAttr(rect, "width") / 2;
                    var rectCenterY = TestMethods.numAttr(rect, "y") + TestMethods.numAttr(rect, "height") / 2;
                    chai_1.assert.closeTo(xScale.scale(d.x), rectCenterX, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in x is scaled version of input data");
                    chai_1.assert.closeTo(yScale.scale(d.y), rectCenterY, window.Pixel_CloseTo_Requirement, "center of rect " + i + " in y is scaled version of input data");
                });
                div.remove();
            });
        });
        describe("retrieving D3 Selections from the plot", function () {
            var div;
            var xScale;
            var yScale;
            var plot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Category();
                plot = new Plottable.Plots.Rectangle();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
                var data = [
                    { x: "A", y: "U" },
                    { x: "B", y: "U" },
                    { x: "A", y: "V" },
                    { x: "B", y: "V" },
                ];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                plot.renderTo(div);
            });
            it("retrieves all selections with no args", function () {
                var allCells = plot.selections();
                chai_1.assert.strictEqual(allCells.size(), plot.datasets()[0].data().length, "rect for each datum");
                var selectionData = allCells.data();
                chai_1.assert.includeMembers(selectionData, plot.datasets()[0].data(), "data in selection data");
                div.remove();
            });
            it("retrieves correct selections", function () {
                var allCells = plot.selections([plot.datasets()[0]]);
                chai_1.assert.strictEqual(allCells.size(), plot.datasets()[0].data().length, "rect for each datum");
                var selectionData = allCells.data();
                chai_1.assert.includeMembers(selectionData, plot.datasets()[0].data(), "data in selection data");
                div.remove();
            });
            it("skips invalid Datasets", function () {
                var dummyDataset = new Plottable.Dataset([]);
                var allCells = plot.selections([plot.datasets()[0], dummyDataset]);
                chai_1.assert.strictEqual(allCells.size(), plot.datasets()[0].data().length, "rect for each datum");
                var selectionData = allCells.data();
                chai_1.assert.includeMembers(selectionData, plot.datasets()[0].data(), "data in selection data");
                div.remove();
            });
        });
        describe("labelling plot elements", function () {
            var plot;
            var xScale;
            var yScale;
            var div;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Rectangle();
                var data = [
                    { x: 0, y: 0, x2: 1, y2: 1, val: "1" },
                    { x: 0, y: 1, x2: 1, y2: 2, val: "2" },
                ];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                plot.x(function (d) { return d.x; }, xScale)
                    .y(function (d) { return d.y; }, yScale)
                    .x2(function (d) { return d.x2; })
                    .y2(function (d) { return d.y2; })
                    .label(function (d) { return d.val; });
            });
            it("does not display rectangle labels by default", function () {
                plot.renderTo(div);
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), 0, "no labels are drawn by default");
                div.remove();
            });
            it("renders correct text for the labels", function () {
                plot.renderTo(div);
                plot.labelsEnabled(true);
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), 2, "all labels are drawn");
                texts.each(function (d, i) {
                    var textString = d3.select(this).text();
                    chai_1.assert.strictEqual(textString, plot.datasets()[0].data()[i].val, "label " + i + " is rendered");
                });
                div.remove();
            });
            it("hides labels when rectangles do not offer enough width", function () {
                var constrainedWidth = 150;
                div.style("width", constrainedWidth + "px");
                plot.renderTo(div);
                plot.labelsEnabled(true);
                plot.label(function () { return "a really really really long string"; });
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), 0, "labels not drawn if not enough width");
                div.remove();
            });
            it("hides labels when rectangles do not offer enough height", function () {
                var constrainedHeight = 30;
                div.style("height", constrainedHeight + "px");
                plot.renderTo(div);
                plot.labelsEnabled(true);
                plot.label(function (d) { return d.val; });
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), 0, "labels not drawn if not enough height");
                div.remove();
            });
            it("updates labels on dataset change", function () {
                plot.renderTo(div);
                plot.labelsEnabled(true);
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), plot.datasets()[0].data().length, "all labels are drawn");
                var data2 = [{ x: 0, y: 0, x2: 1, y2: 1, val: "5" }];
                plot.datasets()[0].data(data2);
                var updatedTexts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(updatedTexts.size(), data2.length, "label for each datum");
                updatedTexts.each(function (d, i) {
                    var textString = d3.select(this).text();
                    chai_1.assert.strictEqual(textString, data2[i].val, "new label drawn");
                });
                div.remove();
            });
            it("hides labels cut off by edges", function () {
                plot.labelsEnabled(true);
                var xDomainMin = 1;
                var xDomainMax = 4;
                var data = Plottable.Utils.Math.range(xDomainMin - 1, xDomainMax + 1, 0.5).map(function (d) {
                    return { x: d, y: 2, x2: d + 0.5, y2: 3, val: d.toString() };
                });
                plot.datasets()[0].data(data);
                xScale.domain([xDomainMin, xDomainMax]);
                plot.renderTo(div);
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), (xDomainMax - xDomainMin) / 0.5, "labels inside the edges are drawn");
                texts.each(function (d, i) {
                    var value = parseFloat(d3.select(this).text());
                    chai_1.assert.operator(value, ">=", xDomainMin, "label drawn inside left edge");
                    chai_1.assert.operator(value, "<=", xDomainMax, "label drawn inside right edge");
                });
                div.remove();
            });
            it("hides labels cut off by other rectangles", function () {
                plot.labelsEnabled(true);
                var constantValue = 5;
                var overlappingRectangleData = Plottable.Utils.Math.range(0, 4).map(function (index) {
                    return { x: constantValue, y: constantValue, x2: constantValue + 2, y2: constantValue + 2, val: index.toString() };
                });
                plot.datasets()[0].data(overlappingRectangleData);
                plot.renderTo(div);
                var texts = plot.content().selectAll("text");
                chai_1.assert.strictEqual(texts.size(), 1, "only top most label is rendered");
                div.remove();
            });
        });
    });
});
