"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var Mocks = require("../mocks");
var TestMethods = require("../testMethods");
var domUtils_1 = require("../../src/utils/domUtils");
describe("Plots", function () {
    describe("ScatterPlot", function () {
        describe("Basic Rendering", function () {
            var DIV_WIDTH = 400;
            var DIV_HEIGHT = 400;
            var div;
            var plot;
            var xScale;
            var yScale;
            beforeEach(function () {
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Scatter();
                plot.x(function (d) { return d.x; }, xScale)
                    .y(function (d) { return d.y; }, yScale);
            });
            afterEach(function () {
                if (this.currentTest.state === "passed") {
                    plot.destroy();
                    div.remove();
                }
            });
            it("renders correctly with no data", function () {
                chai_1.assert.doesNotThrow(function () { return plot.renderTo(div); }, Error);
                chai_1.assert.strictEqual(plot.width(), DIV_WIDTH, "was allocated width");
                chai_1.assert.strictEqual(plot.height(), DIV_HEIGHT, "was allocated height");
            });
            it("is initialized correctly", function () {
                var dataset = new Plottable.Dataset([{ x: 0, y: 0 }]);
                plot.addDataset(dataset);
                chai_1.assert.isDefined(plot.attr("fill"), "default color is defined");
                chai_1.assert.isDefined(plot.animator(Plottable.Plots.Animator.MAIN), "main animator is defined");
                chai_1.assert.isDefined(plot.size(), "size() is defined");
                chai_1.assert.isDefined(plot.symbol(), "symbol() is defined");
            });
            it("initially draws the points with a size of 0 when resetting", function () {
                var data = [
                    { x: 0, y: 0 },
                ];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                var symbolFactory = Plottable.SymbolFactories.square();
                plot.symbol(function () { return symbolFactory; });
                plot.animator(Plottable.Plots.Animator.MAIN, new Mocks.NoOpAnimator());
                plot.animated(true);
                plot.renderTo(div);
                var pathString = plot.content().select("path").attr("d");
                var zeroSizePathString = symbolFactory(0);
                var normalizingPath = plot.content().append("path"); // generated and normalized paths don't align on all browsers
                var expectedPathString = normalizingPath.attr("d", zeroSizePathString).attr("d");
                chai_1.assert.strictEqual(pathString, expectedPathString, "path string is initialized with the correct symbol and a size of 0");
            });
            it.skip("correctly handles NaN, undefined, Infinity, and non-number x and y values", function () {
                var data = [
                    { x: 0.0, y: 0.0 },
                    { x: 0.2, y: 0.2 },
                    { x: 0.4, y: 0.4 },
                    { x: 0.6, y: 0.6 },
                    { x: 0.8, y: 0.8 },
                ];
                var dataset = new Plottable.Dataset(data);
                plot.addDataset(dataset);
                plot.renderTo(div);
                chai_1.assert.strictEqual(plot.selections().size(), 5, "all 5 data points are drawn");
                var dataWithError = data.slice();
                dataWithError[2] = { x: 0.4, y: NaN };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the y values is NaN");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned  when one of the y values is NaN");
                dataWithError = data.slice();
                dataWithError[2] = { x: NaN, y: 0.4 };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the x values is NaN");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the x values is NaN");
                dataWithError = data.slice();
                dataWithError[2] = { x: 0.4, y: undefined };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the y values is undefined");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the y values is undefined");
                dataWithError = data.slice();
                dataWithError[2] = { x: undefined, y: 0.4 };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the x values is undefined");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the x values is undefined");
                dataWithError = data.slice();
                dataWithError[2] = { x: 0.4, y: Infinity };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the y values is Infinity");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the y values is Infinity");
                dataWithError = data.slice();
                dataWithError[2] = { x: -Infinity, y: 0.4 };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the x values is -Infinity");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the x values is -Infinity");
                dataWithError = data.slice();
                dataWithError[2] = { x: 0.4, y: "12" };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the y values is \"12\"");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the y values is \"12\"");
                dataWithError = data.slice();
                dataWithError[2] = { x: "abc", y: 0.4 };
                dataset.data(dataWithError);
                chai_1.assert.strictEqual(plot.selections().size(), 4, "only 4 data points are drawn when one of the x values is \"abc\"");
                chai_1.assert.lengthOf(plot.entities(), 4, "only 4 Entities returned when one of the x values is \"abc\"");
            });
        });
        describe("The Accessors properly access data, index and Dataset", function () {
            var div;
            var plot;
            var dataset;
            var data;
            beforeEach(function () {
                var DIV_WIDTH = 400;
                var DIV_HEIGHT = 400;
                div = TestMethods.generateDiv(DIV_WIDTH, DIV_HEIGHT);
                var xScale = new Plottable.Scales.Linear().domain([0, DIV_WIDTH]);
                var yScale = new Plottable.Scales.Linear().domain([0, DIV_HEIGHT]);
                var metadata = { foo: 10, bar: 20 };
                data = [{ x: 0, y: 0 }, { x: 1, y: 1 }];
                dataset = new Plottable.Dataset(data, metadata);
                plot = new Plottable.Plots.Scatter();
                plot.x(function (d, i, _dataset) { return d.x + i * dataset.metadata().foo; }, xScale);
                plot.y(function (d, i, _dataset) { return dataset.metadata().bar; }, yScale);
                plot.addDataset(dataset);
            });
            it("renders symbols correctly", function () {
                plot.renderTo(div);
                var symbols = plot.selections();
                chai_1.assert.strictEqual(symbols.size(), data.length, "exactly 2 symbols are rendered");
                var c1 = d3.select(symbols.node());
                var c2 = d3.select(symbols.nodes()[1]);
                var c1Position = domUtils_1.getTranslateValues(c1);
                var c2Position = domUtils_1.getTranslateValues(c2);
                chai_1.assert.closeTo(c1Position[0], 0, 0.01, "first symbol cx is correct");
                chai_1.assert.closeTo(c1Position[1], 380, 0.01, "first symbol cy is correct");
                chai_1.assert.closeTo(c2Position[0], 11, 0.01, "second symbol cx is correct");
                chai_1.assert.closeTo(c2Position[1], 380, 0.01, "second symbol cy is correct");
                div.remove();
            });
            it("renders symbols correctly after data change", function () {
                plot.renderTo(div);
                var changedData = [{ x: 2, y: 2 }, { x: 4, y: 4 }];
                dataset.data(changedData);
                var symbols = plot.selections();
                chai_1.assert.strictEqual(symbols.size(), data.length, "exactly 2 symbols are rendered");
                var c1 = d3.select(symbols.node());
                var c2 = d3.select(symbols.nodes()[1]);
                var c1Position = domUtils_1.getTranslateValues(c1);
                var c2Position = domUtils_1.getTranslateValues(c2);
                chai_1.assert.closeTo(c1Position[0], 2, 0.01, "first symbol cx is correct after data change");
                chai_1.assert.closeTo(c1Position[1], 380, 0.01, "first symbol cy is correct after data change");
                chai_1.assert.closeTo(c2Position[0], 14, 0.01, "second symbol cx is correct after data change");
                chai_1.assert.closeTo(c2Position[1], 380, 0.01, "second symbol cy is correct after data change");
                div.remove();
            });
            it("renders symbols correctly after metadata change", function () {
                plot.renderTo(div);
                var changedMetadata = { foo: 0, bar: 0 };
                dataset.metadata(changedMetadata);
                var symbols = plot.selections();
                chai_1.assert.strictEqual(symbols.size(), 2, "exactly 2 symbols are rendered");
                var c1 = d3.select(symbols.node());
                var c2 = d3.select(symbols.nodes()[1]);
                var c1Position = domUtils_1.getTranslateValues(c1);
                var c2Position = domUtils_1.getTranslateValues(c2);
                chai_1.assert.closeTo(c1Position[0], 0, 0.01, "first symbol cx is correct after metadata change");
                chai_1.assert.closeTo(c1Position[1], 400, 0.01, "first symbol cy is correct after metadata change");
                chai_1.assert.closeTo(c2Position[0], 1, 0.01, "second symbol cx is correct after metadata change");
                chai_1.assert.closeTo(c2Position[1], 400, 0.01, "second symbol cy is correct after metadata change");
                div.remove();
            });
            it("sets size() correctly", function () {
                chai_1.assert.strictEqual(plot.size().accessor(data[0], 0, dataset), 6, "first symbol size is initialized");
                chai_1.assert.strictEqual(plot.size().accessor(data[1], 1, dataset), 6, "second symbol size is initialized");
                chai_1.assert.strictEqual(plot.size(10), plot, "setting size() returns calling Plot.Scatter");
                chai_1.assert.strictEqual(plot.size().accessor(data[0], 0, dataset), 10, "first symbol size is set to a constant");
                chai_1.assert.strictEqual(plot.size().accessor(data[1], 1, dataset), 10, "second symbol size is set to a constant");
                chai_1.assert.strictEqual(plot.size(function (d, i) { return i + d.x * d.y; }), plot, "setting size() returns calling Plot.Scatter");
                chai_1.assert.strictEqual(plot.size().accessor(data[0], 0, dataset), 0, "first symbol size is calculated correctly");
                chai_1.assert.strictEqual(plot.size().accessor(data[1], 1, dataset), 2, "second symbol size is calculated correctly");
                div.remove();
            });
            it("sets symbol() correctly", function () {
                var circleSymbolFactory = Plottable.SymbolFactories.circle();
                var squareSymbolFactory = Plottable.SymbolFactories.square();
                chai_1.assert.strictEqual(plot.symbol(function () { return circleSymbolFactory; }), plot, "setting symbol() returns calling Plot.Scatter");
                chai_1.assert.deepEqual(plot.symbol().accessor(data[0], 0, dataset), circleSymbolFactory, "first symbol SymbolFactory is set correctly");
                chai_1.assert.deepEqual(plot.symbol().accessor(data[1], 1, dataset), circleSymbolFactory, "second symbol SymbolFactory is set correctly");
                chai_1.assert.strictEqual(plot.symbol(function (d, i) { return i === 0 ? squareSymbolFactory : circleSymbolFactory; }), plot, "setting symbol() returns calling Plot.Scatter");
                chai_1.assert.deepEqual(plot.symbol().accessor(data[0], 0, dataset), squareSymbolFactory, "first symbol SymbolFactory is set correctly");
                chai_1.assert.deepEqual(plot.symbol().accessor(data[1], 1, dataset), circleSymbolFactory, "second symbol SymbolFactory is set correctly");
                div.remove();
            });
        });
        describe("Selections", function () {
            var div;
            var plot;
            var dataset;
            var dataset2;
            var xScale;
            var yScale;
            var data;
            var data2;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Linear();
                data = [{ x: 0, y: 0 }, { x: 1, y: 1 }];
                data2 = [{ x: 1, y: 2 }, { x: 3, y: 4 }];
                dataset = new Plottable.Dataset(data);
                dataset2 = new Plottable.Dataset(data2);
                plot = new Plottable.Plots.Scatter();
                plot.x(function (d) { return d.x; }, xScale)
                    .y(function (d) { return d.y; }, yScale)
                    .addDataset(dataset)
                    .addDataset(dataset2);
            });
            it("selects all points in all datasets", function () {
                plot.renderTo(div);
                var allCircles = plot.selections();
                chai_1.assert.strictEqual(allCircles.size(), 4, "all circles retrieved");
                var selectionData = allCircles.data();
                chai_1.assert.includeMembers(selectionData, data, "first dataset data in selection data");
                chai_1.assert.includeMembers(selectionData, data2, "second dataset data in selection data");
                div.remove();
            });
            it("selects the closest data point", function () {
                plot.renderTo(div);
                var diff = 10;
                var closest = plot.entityNearest({
                    x: xScale.scale(0) + diff,
                    y: yScale.scale(0) + diff,
                });
                chai_1.assert.deepEqual(closest.datum, dataset.data()[0], "correct datum has been retrieved");
                div.remove();
            });
            it("ignores off-plot data points when retrieving the nearest Entity", function () {
                plot.renderTo(div);
                yScale.domain([0, 1.9]);
                var closest = plot.entityNearest({
                    x: xScale.scale(1),
                    y: xScale.scale(2),
                });
                chai_1.assert.deepEqual(closest.datum, dataset.data()[1], "correct datum has been retrieved");
                div.remove();
            });
            it("retrieves undefined from entityNearest when no scatter points are in view", function () {
                yScale.domain([-2, -1]);
                xScale.domain([-2, -1]);
                plot.renderTo(div);
                var closest = plot.entityNearest({
                    x: plot.width() / 2,
                    y: plot.height() / 2,
                });
                chai_1.assert.strictEqual(closest, undefined, "no datum has been retrieved");
                div.remove();
            });
            it("can retrieve Entities in a certain range", function () {
                plot.renderTo(div);
                var entities = plot.entitiesIn({
                    min: xScale.scale(1),
                    max: xScale.scale(1),
                }, {
                    min: yScale.scale(1),
                    max: yScale.scale(1),
                });
                chai_1.assert.lengthOf(entities, 1, "only one Entity has been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 1, y: 1 }, "correct datum has been retrieved");
                div.remove();
            });
            it("does not return Entities whose center lies outside the range", function () {
                plot.renderTo(div);
                var entities = plot.entitiesIn({
                    min: xScale.scale(1.001),
                    max: xScale.scale(1.001),
                }, {
                    min: yScale.scale(1.001),
                    max: yScale.scale(1.001),
                });
                chai_1.assert.lengthOf(entities, 0, "no Entities retrieved");
                div.remove();
            });
            it("can retrieve Entities in a certain bounds", function () {
                plot.renderTo(div);
                var entities = plot.entitiesIn({
                    topLeft: {
                        x: xScale.scale(1),
                        y: yScale.scale(1),
                    },
                    bottomRight: {
                        x: xScale.scale(1),
                        y: yScale.scale(1),
                    },
                });
                chai_1.assert.lengthOf(entities, 1, "only one Entity has been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 1, y: 1 }, "correct datum has been retrieved");
                div.remove();
            });
            it("can retrieve Entities centered at a given Point", function () {
                plot.renderTo(div);
                var entities = plot.entitiesAt({
                    x: xScale.scale(1),
                    y: yScale.scale(1),
                });
                chai_1.assert.lengthOf(entities, 1, "only one Entity has been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 1, y: 1 }, "correct datum has been retrieved");
                div.remove();
            });
            it("determines whether an Entity contains a given Point by its position and size", function () {
                var initialSize = 10;
                var initialRadius = initialSize / 2;
                plot.size(initialSize);
                plot.renderTo(div);
                var entities = plot.entitiesAt({
                    x: xScale.scale(1) + initialRadius,
                    y: yScale.scale(1) - initialRadius,
                });
                chai_1.assert.lengthOf(entities, 1, "only one Entity has been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 1, y: 1 }, "correct datum has been retrieved");
                plot.size(initialSize - 1);
                entities = plot.entitiesAt({
                    x: xScale.scale(1) + initialRadius,
                    y: yScale.scale(1) - initialRadius,
                });
                chai_1.assert.lengthOf(entities, 0, "none of the Entities is retrieved");
                div.remove();
            });
            it("returns all Entities containing a given Point across all Datasets", function () {
                dataset2.data([{ x: 0, y: 1 }, { x: 200, y: 200 }]);
                plot.renderTo(div);
                var entities = plot.entitiesAt({
                    x: xScale.scale(0.5),
                    y: yScale.scale(0.5),
                });
                chai_1.assert.lengthOf(entities, 3, "all 3 Entities containing the Point have been retrieved");
                chai_1.assert.deepEqual(entities[0].datum, { x: 0, y: 0 }, "correct datum has been retrieved");
                chai_1.assert.deepEqual(entities[1].datum, { x: 1, y: 1 }, "correct datum has been retrieved");
                chai_1.assert.deepEqual(entities[2].datum, { x: 0, y: 1 }, "correct datum has been retrieved");
                div.remove();
            });
        });
    });
});
