"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("StackedAreaPlot", function () {
        describe("setting the x property", function () {
            var div;
            var stackedAreaPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                var data1 = [
                    { x: 1 },
                    { x: 3 },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                dataset1.metadata({ bar: 7 });
                var data2 = [
                    { x: 1 },
                    { x: 3 },
                ];
                var dataset2 = new Plottable.Dataset(data2);
                dataset2.metadata({ bar: 10 });
                stackedAreaPlot = new Plottable.Plots.StackedArea();
                stackedAreaPlot.addDataset(dataset1);
                stackedAreaPlot.addDataset(dataset2);
                stackedAreaPlot.y(0, new Plottable.Scales.Linear());
            });
            it("can set the x property accessor to a constant value", function () {
                var constantValue = 10;
                chai_1.assert.strictEqual(stackedAreaPlot.x(constantValue), stackedAreaPlot, "setter returns calling object");
                stackedAreaPlot.renderTo(div);
                var stackedAreas = stackedAreaPlot.content().selectAll("path");
                chai_1.assert.strictEqual(stackedAreas.size(), stackedAreaPlot.datasets().length, "same number of area selections as datasets");
                stackedAreas.each(function (data, i) {
                    var stackedAreaSelection = d3.select(this);
                    var areaVertices = TestMethods.areaVertices(stackedAreaSelection);
                    var areaXs = areaVertices.map(function (areaVertex) { return areaVertex.x; }).slice(0, -2);
                    data.forEach(function (datum, index) {
                        chai_1.assert.closeTo(areaXs[index], constantValue, window.Pixel_CloseTo_Requirement, "x pixel value at index " + i + " correctly set");
                    });
                    var areaEdgeXs = areaVertices.map(function (areaVertex) { return areaVertex.x; }).slice(-2).reverse();
                    chai_1.assert.closeTo(areaEdgeXs[0], constantValue, window.Pixel_CloseTo_Requirement, "left edge x pixel value correctly set");
                    chai_1.assert.closeTo(areaEdgeXs[1], constantValue, window.Pixel_CloseTo_Requirement, "right edge x pixel value correctly set");
                });
                stackedAreaPlot.destroy();
                div.remove();
            });
            it("can set the x property accessor to be dependent on the input data", function () {
                var accessor = function (d, i, ds) { return d.x * 100 + i * 10 + ds.metadata().bar; };
                chai_1.assert.strictEqual(stackedAreaPlot.x(accessor), stackedAreaPlot, "setter returns calling object");
                chai_1.assert.strictEqual(stackedAreaPlot.x().accessor, accessor, "property set for datum");
                stackedAreaPlot.renderTo(div);
                var stackedAreas = stackedAreaPlot.content().selectAll("path");
                chai_1.assert.strictEqual(stackedAreas.size(), stackedAreaPlot.datasets().length, "same number of area selections as datasets");
                stackedAreas.each(function (data, i) {
                    var stackedAreaSelection = d3.select(this);
                    var areaVertices = TestMethods.areaVertices(stackedAreaSelection);
                    var areaXs = areaVertices.map(function (areaVertex) { return areaVertex.x; }).slice(0, -2);
                    data.forEach(function (datum, datumIndex) {
                        var x = accessor(datum, datumIndex, stackedAreaPlot.datasets()[i]);
                        chai_1.assert.closeTo(areaXs[datumIndex], x, window.Pixel_CloseTo_Requirement, "x pixel value at index " + i + " correctly set");
                    });
                    var areaEdgeXs = areaVertices.map(function (areaVertex) { return areaVertex.x; }).slice(-2).reverse();
                    chai_1.assert.closeTo(areaEdgeXs[0], accessor(data[0], 0, stackedAreaPlot.datasets()[i]), window.Pixel_CloseTo_Requirement, "left edge x pixel value correctly set");
                    chai_1.assert.closeTo(areaEdgeXs[1], accessor(data[data.length - 1], data.length - 1, stackedAreaPlot.datasets()[i]), window.Pixel_CloseTo_Requirement, "right edge x pixel value correctly set");
                });
                stackedAreaPlot.destroy();
                div.remove();
            });
            it("can set the x property scale", function () {
                var accessor = function (d) { return d.x; };
                var linearScale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(stackedAreaPlot.x(accessor, linearScale), stackedAreaPlot, "setter returns calling object");
                chai_1.assert.strictEqual(stackedAreaPlot.x().accessor, accessor, "property set for datum");
                stackedAreaPlot.renderTo(div);
                var stackedAreas = stackedAreaPlot.content().selectAll("path");
                chai_1.assert.strictEqual(stackedAreas.size(), stackedAreaPlot.datasets().length, "same number of area selections as datasets");
                stackedAreas.each(function (data, i) {
                    var stackedAreaSelection = d3.select(this);
                    var areaVertices = TestMethods.areaVertices(stackedAreaSelection);
                    var areaXs = areaVertices.map(function (areaVertex) { return areaVertex.x; }).slice(0, -2);
                    data.forEach(function (datum, index) {
                        var x = linearScale.scale(accessor(datum));
                        chai_1.assert.closeTo(areaXs[index], x, window.Pixel_CloseTo_Requirement, "x pixel value at index " + i + " correctly set");
                    });
                    var areaEdgeXs = areaVertices.map(function (areaVertex) { return areaVertex.x; }).slice(-2).reverse();
                    chai_1.assert.closeTo(areaEdgeXs[0], linearScale.scale(accessor(data[0])), window.Pixel_CloseTo_Requirement, "left edge x pixel value correctly set");
                    chai_1.assert.closeTo(areaEdgeXs[1], linearScale.scale(accessor(data[data.length - 1])), window.Pixel_CloseTo_Requirement, "right edge x pixel value correctly set");
                });
                stackedAreaPlot.destroy();
                div.remove();
            });
        });
        describe("setting the y property", function () {
            var div;
            var stackedAreaPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                var data1 = [
                    { y: 1 },
                    { y: 3 },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                dataset1.metadata({ bar: 7 });
                var data2 = [
                    { y: 5 },
                    { y: 3 },
                ];
                var dataset2 = new Plottable.Dataset(data2);
                dataset2.metadata({ bar: 10 });
                stackedAreaPlot = new Plottable.Plots.StackedArea();
                stackedAreaPlot.addDataset(dataset1);
                stackedAreaPlot.addDataset(dataset2);
                stackedAreaPlot.x(function (d, i) { return i; });
            });
            function calculateStackedYs(yAccessor) {
                var stackedYDataArray = [];
                stackedAreaPlot.datasets().forEach(function (dataset, datasetIndex, datasets) {
                    var stackedYData = dataset.data().map(function (d, i) { return yAccessor(d, i, dataset); });
                    if (datasetIndex === 0) {
                        stackedYDataArray[datasetIndex] = stackedYData;
                        return;
                    }
                    stackedYDataArray[datasetIndex] = dataset.data().map(function (d, i) {
                        return yAccessor(d, i, dataset) + stackedYDataArray[datasetIndex - 1][i];
                    });
                });
                return stackedYDataArray;
            }
            it("can set to a constant value", function () {
                var constantValue = 10;
                chai_1.assert.strictEqual(stackedAreaPlot.y(constantValue), stackedAreaPlot, "setter returns calling object");
                stackedAreaPlot.destroy();
                div.remove();
            });
            it("can set to be dependent on the input data", function () {
                var accessor = function (d, i, ds) { return d.y * 100 + i * 10 + ds.metadata().bar; };
                chai_1.assert.strictEqual(stackedAreaPlot.y(accessor), stackedAreaPlot, "setter returns calling object");
                chai_1.assert.strictEqual(stackedAreaPlot.y().accessor, accessor, "accessor set");
                stackedAreaPlot.destroy();
                div.remove();
            });
            it("can set the property scale", function () {
                var accessor = function (d) { return d.y; };
                var linearScale = new Plottable.Scales.Linear();
                chai_1.assert.strictEqual(stackedAreaPlot.y(accessor, linearScale), stackedAreaPlot, "setter returns calling object");
                chai_1.assert.strictEqual(stackedAreaPlot.y().accessor, accessor, "accessor set");
                stackedAreaPlot.renderTo(div);
                var stackedAreas = stackedAreaPlot.content().selectAll("path");
                chai_1.assert.strictEqual(stackedAreas.size(), stackedAreaPlot.datasets().length, "same number of area selections as datasets");
                var stackedYs = calculateStackedYs(accessor);
                stackedAreas.each(function (data, i) {
                    var stackedAreaSelection = d3.select(this);
                    var areaVertices = TestMethods.areaVertices(stackedAreaSelection);
                    var areaYs = areaVertices.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                    data.forEach(function (datum, datumIndex) {
                        var y = linearScale.scale(stackedYs[i][datumIndex]);
                        chai_1.assert.closeTo(areaYs[datumIndex], y, window.Pixel_CloseTo_Requirement, "y pixel value at index " + i + " correctly set");
                    });
                    var areaEdgeYs = areaVertices.map(function (areaVertex) { return areaVertex.y; }).slice(-2).reverse();
                    if (i === 0) {
                        chai_1.assert.closeTo(areaEdgeYs[0], linearScale.scale(0), window.Pixel_CloseTo_Requirement, "left edge y pixel value correctly set");
                        chai_1.assert.closeTo(areaEdgeYs[1], linearScale.scale(0), window.Pixel_CloseTo_Requirement, "right edge y pixel value correctly set");
                    }
                    else {
                        chai_1.assert.closeTo(areaEdgeYs[0], linearScale.scale(stackedYs[i - 1][0]), window.Pixel_CloseTo_Requirement, "left edge y pixel value correctly set");
                        chai_1.assert.closeTo(areaEdgeYs[1], linearScale.scale(stackedYs[i - 1][data.length - 1]), window.Pixel_CloseTo_Requirement, "right edge y pixel value correctly set");
                    }
                });
                stackedAreaPlot.destroy();
                div.remove();
            });
        });
        describe("setting the y0 property", function () {
            var stackedAreaPlot;
            beforeEach(function () {
                var data1 = [
                    { foo: 1 },
                    { foo: 3 },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                dataset1.metadata({ bar: 7 });
                var data2 = [
                    { foo: 1 },
                    { foo: 3 },
                ];
                var dataset2 = new Plottable.Dataset(data2);
                dataset2.metadata({ bar: 10 });
                stackedAreaPlot = new Plottable.Plots.StackedArea();
                stackedAreaPlot.addDataset(dataset1);
                stackedAreaPlot.addDataset(dataset2);
                stackedAreaPlot.x(function (d, i) { return i; });
                stackedAreaPlot.y(function (d, i) { return i; });
            });
            it("defaults to the 0 constant value", function () {
                chai_1.assert.strictEqual(stackedAreaPlot.y0().accessor(null, 0, null), 0, "property set to 0");
                stackedAreaPlot.destroy();
            });
            it("can set the y0 property accessor to a constant value", function () {
                var constantValue = 10;
                stackedAreaPlot.y0(constantValue);
                chai_1.assert.strictEqual(stackedAreaPlot.y0().accessor(null, 0, null), constantValue, "property set to constant value");
                stackedAreaPlot.destroy();
            });
            it("can set the y0 property accessor to be dependent on the input data", function () {
                var accessor = function (d, i, ds) { return d.foo * 100 + i * 10 + ds.metadata().bar; };
                stackedAreaPlot.y0(accessor);
                chai_1.assert.strictEqual(stackedAreaPlot.y0().accessor, accessor, "property set for datum");
                stackedAreaPlot.destroy();
            });
        });
        describe("rendering on edge case scenarios", function () {
            it("renders nothing when no data", function () {
                var div = TestMethods.generateDiv();
                var stackedAreaPlot = new Plottable.Plots.StackedArea();
                stackedAreaPlot.x(function () { return null; });
                // HACKHACK https://github.com/palantir/plottable/issues/2712 y scale must be set.
                stackedAreaPlot.y(function () { return null; }, new Plottable.Scales.Linear());
                stackedAreaPlot.renderTo(div);
                chai_1.assert.strictEqual(stackedAreaPlot.selections().size(), 0, "no areas rendered");
                stackedAreaPlot.destroy();
                div.remove();
            });
            it("coerces strings to numbers for calculating offsets", function () {
                var div = TestMethods.generateDiv();
                var data0 = [
                    { x: 2, y: "2" },
                    { x: 3, y: 2 },
                ];
                var data1 = [
                    { x: 2, y: "0" },
                    { x: 3, y: "2" },
                ];
                var data2 = [
                    { x: 2, y: 2 },
                    { x: 3, y: 2 },
                ];
                var xScale = new Plottable.Scales.Linear();
                var yScale = new Plottable.Scales.Linear();
                var plot = new Plottable.Plots.StackedArea();
                var dataset0 = new Plottable.Dataset(data0);
                plot.addDataset(dataset0);
                var dataset1 = new Plottable.Dataset(data1);
                plot.addDataset(dataset1);
                var dataset2 = new Plottable.Dataset(data2);
                plot.addDataset(dataset2);
                plot.x(function (d) { return d.x; }, xScale).y(function (d) { return d.y; }, yScale);
                plot.renderTo(div);
                var stackedAreaSelections = plot.content().selectAll("path");
                var stackedAreaSelection0 = d3.select(stackedAreaSelections.node());
                var stackedAreaSelection1 = d3.select(stackedAreaSelections.nodes()[1]);
                var stackedAreaSelection2 = d3.select(stackedAreaSelections.nodes()[2]);
                var areaVertices0 = TestMethods.areaVertices(stackedAreaSelection0);
                var areaYs0 = areaVertices0.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                var areaVertices1 = TestMethods.areaVertices(stackedAreaSelection1);
                var areaYs1 = areaVertices1.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                var areaVertices2 = TestMethods.areaVertices(stackedAreaSelection2);
                var areaYs2 = areaVertices2.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                chai_1.assert.closeTo(areaYs0[0], yScale.scale(data0[0].y), window.Pixel_CloseTo_Requirement, "dataset0 should have no offset");
                chai_1.assert.closeTo(areaYs1[0], yScale.scale(data0[0].y), window.Pixel_CloseTo_Requirement, "dataset1 should be offset with previous datasets");
                chai_1.assert.closeTo(areaYs2[0], yScale.scale(data2[0].y + parseFloat(data1[0].y) + parseFloat(data0[0].y)), window.Pixel_CloseTo_Requirement, "dataset2 should be offset with previous datasets");
                plot.destroy();
                div.remove();
            });
            it("coerces null to 0 for calculating offsets", function () {
                var div = TestMethods.generateDiv();
                var data0 = [
                    { x: 2, y: 2 },
                    { x: 3, y: 2 },
                ];
                var data1 = [
                    { x: 2, y: null },
                    { x: 3, y: 2 },
                ];
                var data2 = [
                    { x: 2, y: 2 },
                    { x: 3, y: 2 },
                ];
                var xScale = new Plottable.Scales.Linear();
                var yScale = new Plottable.Scales.Linear();
                var plot = new Plottable.Plots.StackedArea();
                var dataset0 = new Plottable.Dataset(data0);
                plot.addDataset(dataset0);
                var dataset1 = new Plottable.Dataset(data1);
                plot.addDataset(dataset1);
                var dataset2 = new Plottable.Dataset(data2);
                plot.addDataset(dataset2);
                plot.x(function (d) { return d.x; }, xScale).y(function (d) { return d.y; }, yScale);
                plot.renderTo(div);
                var stackedAreaSelections = plot.content().selectAll("path");
                var stackedAreaSelection0 = d3.select(stackedAreaSelections.node());
                var stackedAreaSelection1 = d3.select(stackedAreaSelections.nodes()[1]);
                var stackedAreaSelection2 = d3.select(stackedAreaSelections.nodes()[2]);
                var areaVertices0 = TestMethods.areaVertices(stackedAreaSelection0);
                var areaYs0 = areaVertices0.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                var areaVertices1 = TestMethods.areaVertices(stackedAreaSelection1);
                var areaYs1 = areaVertices1.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                var areaVertices2 = TestMethods.areaVertices(stackedAreaSelection2);
                var areaYs2 = areaVertices2.map(function (areaVertex) { return areaVertex.y; }).slice(0, -2);
                chai_1.assert.closeTo(areaYs0[0], yScale.scale(data0[0].y), window.Pixel_CloseTo_Requirement, "dataset0 should have no offset");
                chai_1.assert.closeTo(areaYs1[0], yScale.scale(data0[0].y), window.Pixel_CloseTo_Requirement, "dataset1 should be offset with dataset0");
                chai_1.assert.closeTo(areaYs2[0], yScale.scale(data2[0].y + data0[0].y), window.Pixel_CloseTo_Requirement, "dataset2 should be offset with only dataset0");
                plot.destroy();
                div.remove();
            });
        });
        describe("downsampling", function () {
            var stackedAreaPlot;
            beforeEach(function () {
                stackedAreaPlot = new Plottable.Plots.StackedArea();
            });
            it("disables downsampling", function () {
                chai_1.assert.strictEqual(stackedAreaPlot.downsamplingEnabled(), false, "downsampling is disabled by default");
                stackedAreaPlot.downsamplingEnabled(true);
                chai_1.assert.strictEqual(stackedAreaPlot.downsamplingEnabled(), false, "downsampling will not be enabled by user");
            });
        });
        describe("stacking order", function () {
            var div;
            var xScale;
            var stackedAreaPlot;
            var datas = [
                [1, 2, 3],
                [17, 13, 11],
            ];
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Category();
                stackedAreaPlot = new Plottable.Plots.StackedArea()
                    .addDataset(new Plottable.Dataset(datas[0]))
                    .addDataset(new Plottable.Dataset(datas[1]))
                    .x(function (d, i) { return "" + i; }, xScale)
                    .y(function (d) { return d; }, new Plottable.Scales.Linear())
                    .renderTo(div);
            });
            it("stacks bottomup by default", function () {
                var areas = stackedAreaPlot.content().selectAll("path");
                var area0 = areas.filter(function (d) { return d === datas[0]; });
                var area1 = areas.filter(function (d) { return d === datas[1]; });
                var areaYs0 = TestMethods.areaVertices(area0).map(function (areaVertex) { return areaVertex.y; });
                var areaYs1 = TestMethods.areaVertices(area1).map(function (areaVertex) { return areaVertex.y; });
                // note that higher y value means toward bottom of screen
                chai_1.assert.operator(areaYs0[0], ">", areaYs1[0], "first series below second");
            });
            it("stacks topdown", function () {
                stackedAreaPlot.stackingOrder("topdown");
                var areas = stackedAreaPlot.content().selectAll("path");
                var area0 = areas.filter(function (d) { return d === datas[0]; });
                var area1 = areas.filter(function (d) { return d === datas[1]; });
                var areaYs0 = TestMethods.areaVertices(area0).map(function (areaVertex) { return areaVertex.y; });
                var areaYs1 = TestMethods.areaVertices(area1).map(function (areaVertex) { return areaVertex.y; });
                ;
                // note that higher y value means toward bottom of screen
                chai_1.assert.operator(areaYs0[0], "<", areaYs1[0], "first series atop second");
            });
            afterEach(function () {
                stackedAreaPlot.destroy();
                div.remove();
            });
        });
    });
});
