"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
var barPlot_1 = require("../../src/plots/barPlot");
describe("Plots", function () {
    describe("Vertical Stacked Bar Plot", function () {
        describe("rendering using positive data", function () {
            var div;
            var xScale;
            var yScale;
            var stackedBarPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Linear();
                var data1 = [
                    { x: "A", y: 1 },
                    { x: "B", y: 2 },
                ];
                var data2 = [
                    { x: "A", y: 2 },
                    { x: "B", y: 1 },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                var dataset2 = new Plottable.Dataset(data2);
                stackedBarPlot = new Plottable.Plots.StackedBar();
                stackedBarPlot.addDataset(dataset1);
                stackedBarPlot.addDataset(dataset2);
                stackedBarPlot.x(function (d) { return d.x; }, xScale);
                stackedBarPlot.y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
            });
            afterEach(function () {
                stackedBarPlot.destroy();
                div.remove();
            });
            it("renders rects offset by previous values", function () {
                var bars = stackedBarPlot.content().selectAll("rect");
                var dataLength = stackedBarPlot.datasets()[0].data().length;
                var dataCount = stackedBarPlot.datasets().length * dataLength;
                chai_1.assert.strictEqual(bars.size(), dataCount, "same number of bars as data");
                var calculateStackedYs = function (yAccessor) {
                    var stackedYDataArray = [];
                    stackedBarPlot.datasets().forEach(function (dataset, datasetIndex) {
                        var yData = dataset.data().map(function (d, i) { return yAccessor(d, i, dataset); });
                        if (datasetIndex === 0) {
                            stackedYDataArray[datasetIndex] = yData;
                            return;
                        }
                        stackedYDataArray[datasetIndex] = dataset.data().map(function (d, i) {
                            return yData[i] + stackedYDataArray[datasetIndex - 1][i];
                        });
                    });
                    return stackedYDataArray;
                };
                var stackedYs = calculateStackedYs(function (d) { return d.y; });
                bars.each(function (d, i) {
                    var bar = d3.select(this);
                    var datasetIndex = Math.floor(i / dataLength);
                    var datumIndex = i % dataLength;
                    chai_1.assert.closeTo(TestMethods.numAttr(bar, "y"), yScale.scale(stackedYs[datasetIndex][datumIndex]), window.Pixel_CloseTo_Requirement, "y attribute offset set correctly");
                });
            });
            // HACKHACK #2795: correct off-bar label logic to be implemented
            it("doesn't show any off-bar labels", function () {
                stackedBarPlot.labelsEnabled(true);
                yScale.domain([0, 30]);
                var offBarLabels = stackedBarPlot.content().selectAll(".off-bar-label");
                chai_1.assert.operator(offBarLabels.size(), ">", 0, "some off-bar labels are drawn");
                offBarLabels.each(function (d, i) {
                    chai_1.assert.isTrue(d3.select(this).style("visibility") === "hidden", "off-bar label " + i + " is hidden");
                });
            });
            it("shows stacked bar labels", function () {
                stackedBarPlot.labelsEnabled(true);
                yScale.domain([0, 30]);
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 2);
                var text = [];
                stackedBarLabels.each(function (d) {
                    text.push(d3.select(this).text());
                });
                chai_1.assert.deepEqual(["3", "3"], text);
            });
            it("shows stacked bar labels for date axes", function () {
                // remove existing stacked bar plot
                stackedBarPlot.destroy();
                var dateString1 = "2013-09-02T00:00:00.000Z";
                var dateString2 = "2013-09-03T00:00:00.000Z";
                var data1 = [
                    { x: new Date(dateString1), y: 1 },
                    { x: new Date(dateString2), y: 2 },
                ];
                var data2 = [
                    { x: new Date(dateString1), y: 2 },
                    { x: new Date(dateString2), y: 1 },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                var dataset2 = new Plottable.Dataset(data2);
                var stackedBar = new Plottable.Plots.StackedBar();
                stackedBar.addDataset(dataset1);
                stackedBar.addDataset(dataset2);
                var xScale = new Plottable.Scales.Time();
                stackedBar.x(function (d) { return d.x; }, xScale);
                stackedBar.y(function (d) { return d.y; }, yScale);
                stackedBar.renderTo(div);
                stackedBar.labelsEnabled(true);
                yScale.domain([0, 30]);
                var stackedBarLabels = stackedBar.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 2);
                var text = [];
                stackedBarLabels.each(function (d) {
                    text.push(d3.select(this).text());
                });
                chai_1.assert.deepEqual(["3", "3"], text);
                stackedBar.destroy();
            });
            it("doesn't show stacked bar labels when columns are too tall", function () {
                stackedBarPlot.labelsEnabled(true);
                yScale.domain([0, 3]);
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 0);
            });
            it("doesn't show stacked bar labels when columns are too narrow", function () {
                stackedBarPlot.labelsEnabled(true);
                xScale.range([0, 20]);
                xScale.domain(xScale.domain());
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 0);
            });
            it("considers lying within a bar's y-range to mean it is closest", function () {
                var d0 = stackedBarPlot.datasets()[0].data()[0];
                var d1 = stackedBarPlot.datasets()[1].data()[0];
                var closestEntity = stackedBarPlot.entityNearest({ x: 0, y: yScale.scale(d0.y) + 1 });
                chai_1.assert.strictEqual(closestEntity.datum, d0, "bottom bar is closest when within its range");
                closestEntity = stackedBarPlot.entityNearest({ x: 0, y: yScale.scale(d0.y) - 1 });
                chai_1.assert.strictEqual(closestEntity.datum, d1, "top bar is closest when within its range");
            });
        });
        describe("rendering using negative data", function () {
            var div;
            var xScale;
            var yScale;
            var stackedBarPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Linear();
                var data1 = [
                    { x: "A", y: -1 },
                    { x: "B", y: -2 },
                ];
                var data2 = [
                    { x: "A", y: -2 },
                    { x: "B", y: -1 },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                var dataset2 = new Plottable.Dataset(data2);
                stackedBarPlot = new Plottable.Plots.StackedBar();
                stackedBarPlot.addDataset(dataset1);
                stackedBarPlot.addDataset(dataset2);
                stackedBarPlot.x(function (d) { return d.x; }, xScale);
                stackedBarPlot.y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
            });
            afterEach(function () {
                stackedBarPlot.destroy();
                div.remove();
            });
            it("shows stacked bar labels", function () {
                yScale.domain([3, -3]);
                stackedBarPlot.labelsEnabled(true);
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 2);
                var text = [];
                stackedBarLabels.each(function (d) {
                    text.push(d3.select(this).text());
                });
                chai_1.assert.deepEqual(["-3", "-3"], text);
            });
            it("doesn't show stacked bar labels when columns are too tall", function () {
                stackedBarPlot.labelsEnabled(true);
                yScale.domain([-3, 0]);
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 0);
            });
            it("doesn't show stacked bar labels when columns are too narrow", function () {
                stackedBarPlot.labelsEnabled(true);
                xScale.range([0, 40]);
                xScale.domain(xScale.domain());
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 0);
            });
            it("renders rects offset by previous values", function () {
                var bars = stackedBarPlot.content().selectAll("rect");
                var dataLength = stackedBarPlot.datasets()[0].data().length;
                var dataCount = stackedBarPlot.datasets().length * dataLength;
                chai_1.assert.strictEqual(bars.size(), dataCount, "same number of bars as data");
                var calculateStackedYs = function (yAccessor) {
                    var stackedYDataArray = [];
                    stackedBarPlot.datasets().forEach(function (dataset, datasetIndex) {
                        var yData = dataset.data().map(function (d, i) { return yAccessor(d, i, dataset); });
                        if (datasetIndex === 0) {
                            stackedYDataArray[datasetIndex] = dataset.data().map(function () { return 0; });
                        }
                        stackedYDataArray[datasetIndex + 1] = dataset.data().map(function (d, i) {
                            return yData[i] + stackedYDataArray[datasetIndex][i];
                        });
                    });
                    return stackedYDataArray;
                };
                var stackedYs = calculateStackedYs(function (d) { return d.y; });
                bars.each(function (d, i) {
                    var bar = d3.select(this);
                    var datasetIndex = Math.floor(i / dataLength);
                    var datumIndex = i % dataLength;
                    chai_1.assert.closeTo(TestMethods.numAttr(bar, "y"), yScale.scale(stackedYs[datasetIndex][datumIndex]), window.Pixel_CloseTo_Requirement, "y attribute offset set correctly");
                });
            });
        });
        describe("non-overlapping datasets", function () {
            var div;
            var xScale;
            var stackedBarPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Category();
                var yScale = new Plottable.Scales.Linear();
                var data1 = [
                    { x: "A", y: 1 },
                    { x: "B", y: 2 },
                    { x: "C", y: 1 },
                ];
                var data2 = [
                    { x: "A", y: 2 },
                    { x: "B", y: 3 },
                ];
                var data3 = [
                    { x: "B", y: 1 },
                    { x: "C", y: 7 },
                ];
                stackedBarPlot = new Plottable.Plots.StackedBar();
                stackedBarPlot.addDataset(new Plottable.Dataset(data1));
                stackedBarPlot.addDataset(new Plottable.Dataset(data2));
                stackedBarPlot.addDataset(new Plottable.Dataset(data3));
                stackedBarPlot.x(function (d) { return d.x; }, xScale);
                stackedBarPlot.y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
            });
            afterEach(function () {
                stackedBarPlot.destroy();
                div.remove();
            });
            it("draws bars at specified x location and stacks in order of datasets", function () {
                var bars = stackedBarPlot.content().selectAll("rect");
                var datumCount = stackedBarPlot.datasets().reduce(function (prev, curr) { return prev + curr.data().length; }, 0);
                chai_1.assert.strictEqual(bars.size(), datumCount, "draws a bar for each datum");
                var domain = xScale.domain();
                domain.forEach(function (value) {
                    var domainBarPairs = d3.pairs(bars.filter(function (d) { return d.x === value; }).nodes());
                    domainBarPairs.forEach(function (aBarPair) {
                        chai_1.assert.closeTo(TestMethods.numAttr(d3.select(aBarPair[0]), "x"), TestMethods.numAttr(d3.select(aBarPair[1]), "x"), window.Pixel_CloseTo_Requirement, "bars at same x position");
                        chai_1.assert.operator(TestMethods.numAttr(d3.select(aBarPair[0]), "y"), ">", TestMethods.numAttr(d3.select(aBarPair[1]), "y"), "previous dataset bar under second");
                    });
                });
            });
            it("can be use reverse stacking order", function () {
                // change stacking order
                stackedBarPlot.stackingOrder("topdown");
                var bars = stackedBarPlot.content().selectAll("rect");
                var datumCount = stackedBarPlot.datasets().reduce(function (prev, curr) { return prev + curr.data().length; }, 0);
                chai_1.assert.strictEqual(bars.size(), datumCount, "draws a bar for each datum");
                var domain = xScale.domain();
                domain.forEach(function (value) {
                    var domainBarPairs = d3.pairs(bars.filter(function (d) { return d.x === value; }).nodes());
                    domainBarPairs.forEach(function (aBarPair) {
                        chai_1.assert.closeTo(TestMethods.numAttr(d3.select(aBarPair[0]), "x"), TestMethods.numAttr(d3.select(aBarPair[1]), "x"), window.Pixel_CloseTo_Requirement, "bars at same x position");
                        chai_1.assert.operator(TestMethods.numAttr(d3.select(aBarPair[0]), "y"), "<", TestMethods.numAttr(d3.select(aBarPair[1]), "y"), "previous dataset bar above second");
                    });
                });
            });
        });
        describe("fail safe tests", function () {
            it("should default to 0 when calculating stack offsets with non-numbers", function () {
                var div = TestMethods.generateDiv();
                var stringData = [
                    { x: "A", y: "s" },
                ];
                var nullData = [
                    { x: "A", y: null },
                ];
                var undefinedData = [
                    { x: "A", y: undefined },
                ];
                var naNData = [
                    { x: "A", y: NaN },
                ];
                var validData = [
                    { x: "A", y: 1 },
                ];
                var xScale = new Plottable.Scales.Category();
                var yScale = new Plottable.Scales.Linear();
                var stackedBarPlot = new Plottable.Plots.StackedBar();
                var ds1 = new Plottable.Dataset(stringData);
                var ds2 = new Plottable.Dataset(nullData);
                var ds3 = new Plottable.Dataset(undefinedData);
                var ds4 = new Plottable.Dataset(naNData);
                var ds5 = new Plottable.Dataset(validData);
                stackedBarPlot.addDataset(ds1);
                stackedBarPlot.addDataset(ds2);
                stackedBarPlot.addDataset(ds3);
                stackedBarPlot.addDataset(ds4);
                stackedBarPlot.addDataset(ds5);
                stackedBarPlot.x(function (d) { return d.x; }, xScale).y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
                var validBar = stackedBarPlot.content().selectAll("rect").filter(function (d) { return d.y === 1; });
                chai_1.assert.closeTo(TestMethods.numAttr(validBar, "y"), yScale.scale(1), window.Pixel_CloseTo_Requirement, "bar stacks from 0");
                stackedBarPlot.destroy();
                div.remove();
            });
        });
    });
    describe("Horizontal Stacked Bar Plot", function () {
        describe("rendering using positive data", function () {
            var div;
            var xScale;
            var yScale;
            var stackedBarPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Category();
                var data1 = [
                    { x: 1, y: "A" },
                    { x: 2, y: "B" },
                ];
                var data2 = [
                    { x: 2, y: "A" },
                    { x: 1, y: "B" },
                ];
                var dataset1 = new Plottable.Dataset(data1);
                var dataset2 = new Plottable.Dataset(data2);
                stackedBarPlot = new Plottable.Plots.StackedBar(barPlot_1.BarOrientation.horizontal);
                stackedBarPlot.addDataset(dataset1);
                stackedBarPlot.addDataset(dataset2);
                stackedBarPlot.x(function (d) { return d.x; }, xScale);
                stackedBarPlot.y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
            });
            afterEach(function () {
                stackedBarPlot.destroy();
                div.remove();
            });
            it("shows stacked bar labels", function () {
                xScale.domain([0, 30]);
                stackedBarPlot.labelsEnabled(true);
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 2);
                var text = [];
                stackedBarLabels.each(function (d) {
                    text.push(d3.select(this).text());
                });
                chai_1.assert.deepEqual(["3", "3"], text);
            });
            it("doesn't show stacked bar labels when columns are too tall", function () {
                xScale.domain([0, 3]);
                stackedBarPlot.labelsEnabled(true);
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 0);
            });
            it("doesn't show stacked bar labels when columns are too narrow", function () {
                stackedBarPlot.labelsEnabled(true);
                yScale.range([0, 40]);
                yScale.domain(yScale.domain());
                var stackedBarLabels = stackedBarPlot.content().selectAll(".stacked-bar-label");
                chai_1.assert.strictEqual(stackedBarLabels.size(), 0);
            });
            it("renders rects offset by previous values", function () {
                var bars = stackedBarPlot.content().selectAll("rect");
                var dataLength = stackedBarPlot.datasets()[0].data().length;
                var dataCount = stackedBarPlot.datasets().length * dataLength;
                chai_1.assert.strictEqual(bars.size(), dataCount, "same number of bars as data");
                var calculateStackedXs = function (xAccessor) {
                    var stackedXDataArray = [];
                    stackedBarPlot.datasets().forEach(function (dataset, datasetIndex) {
                        var xData = dataset.data().map(function (d, i) { return xAccessor(d, i, dataset); });
                        if (datasetIndex === 0) {
                            stackedXDataArray[datasetIndex] = dataset.data().map(function () { return 0; });
                        }
                        stackedXDataArray[datasetIndex + 1] = dataset.data().map(function (d, i) {
                            return xData[i] + stackedXDataArray[datasetIndex][i];
                        });
                    });
                    return stackedXDataArray;
                };
                var stackedXs = calculateStackedXs(function (d) { return d.x; });
                bars.each(function (d, i) {
                    var bar = d3.select(this);
                    var datasetIndex = Math.floor(i / dataLength);
                    var datumIndex = i % dataLength;
                    chai_1.assert.closeTo(TestMethods.numAttr(bar, "x"), xScale.scale(stackedXs[datasetIndex][datumIndex]), window.Pixel_CloseTo_Requirement, "x attribute offset correctly");
                });
            });
            // HACKHACK #2795: correct off-bar label logic to be implemented
            it("doesn't show any off-bar labels", function () {
                stackedBarPlot.labelsEnabled(true);
                xScale.domain([0, 50]);
                var offBarLabels = stackedBarPlot.content().selectAll(".off-bar-label");
                chai_1.assert.operator(offBarLabels.size(), ">", 0, "some off-bar labels are drawn");
                offBarLabels.each(function (d, i) {
                    chai_1.assert.strictEqual(d3.select(this).style("visibility"), "hidden", "off-bar label " + i + " is hidden");
                });
            });
        });
        describe("non-overlapping datasets", function () {
            var div;
            var yScale;
            var stackedBarPlot;
            beforeEach(function () {
                div = TestMethods.generateDiv();
                var xScale = new Plottable.Scales.Linear();
                yScale = new Plottable.Scales.Category();
                var data1 = [
                    { y: "A", x: 1 },
                    { y: "B", x: 2 },
                    { y: "C", x: 1 },
                ];
                var data2 = [
                    { y: "A", x: 2 },
                    { y: "B", x: 3 },
                ];
                var data3 = [
                    { y: "B", x: 1 },
                    { y: "C", x: 7 },
                ];
                stackedBarPlot = new Plottable.Plots.StackedBar(barPlot_1.BarOrientation.horizontal);
                stackedBarPlot.addDataset(new Plottable.Dataset(data1));
                stackedBarPlot.addDataset(new Plottable.Dataset(data2));
                stackedBarPlot.addDataset(new Plottable.Dataset(data3));
                stackedBarPlot.x(function (d) { return d.x; }, xScale);
                stackedBarPlot.y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
            });
            afterEach(function () {
                stackedBarPlot.destroy();
                div.remove();
            });
            it("draws bars at specified y location and stacks in order of datasets", function () {
                var bars = stackedBarPlot.content().selectAll("rect");
                var datumCount = stackedBarPlot.datasets().reduce(function (prev, curr) { return prev + curr.data().length; }, 0);
                chai_1.assert.strictEqual(bars.size(), datumCount, "draws a bar for each datum");
                var domain = yScale.domain();
                domain.forEach(function (value) {
                    var domainBarPairs = d3.pairs(bars.filter(function (d) { return d.y === value; }).nodes());
                    domainBarPairs.forEach(function (aBarPair) {
                        chai_1.assert.closeTo(TestMethods.numAttr(d3.select(aBarPair[0]), "y"), TestMethods.numAttr(d3.select(aBarPair[1]), "y"), window.Pixel_CloseTo_Requirement, "bars at same x position");
                        chai_1.assert.operator(TestMethods.numAttr(d3.select(aBarPair[0]), "x"), "<", TestMethods.numAttr(d3.select(aBarPair[1]), "x"), "previous dataset bar under second");
                    });
                });
            });
        });
        describe("fail safe tests", function () {
            it("should default to 0 when calculating stack offsets with non-numbers", function () {
                var div = TestMethods.generateDiv();
                var stringData = [
                    { x: "s", y: "A" },
                ];
                var nullData = [
                    { x: null, y: "A" },
                ];
                var undefinedData = [
                    { x: undefined, y: "A" },
                ];
                var naNData = [
                    { x: NaN, y: "A" },
                ];
                var validData = [
                    { x: 1, y: "A" },
                ];
                var xScale = new Plottable.Scales.Linear();
                var yScale = new Plottable.Scales.Category();
                var stackedBarPlot = new Plottable.Plots.StackedBar(barPlot_1.BarOrientation.horizontal);
                var ds1 = new Plottable.Dataset(stringData);
                var ds2 = new Plottable.Dataset(nullData);
                var ds3 = new Plottable.Dataset(undefinedData);
                var ds4 = new Plottable.Dataset(naNData);
                var ds5 = new Plottable.Dataset(validData);
                stackedBarPlot.addDataset(ds1);
                stackedBarPlot.addDataset(ds2);
                stackedBarPlot.addDataset(ds3);
                stackedBarPlot.addDataset(ds4);
                stackedBarPlot.addDataset(ds5);
                stackedBarPlot.x(function (d) { return d.x; }, xScale).y(function (d) { return d.y; }, yScale);
                stackedBarPlot.renderTo(div);
                var validBar = stackedBarPlot.content().selectAll("rect").filter(function (d) { return d.x === 1; });
                chai_1.assert.closeTo(TestMethods.numAttr(validBar, "x"), xScale.scale(0), window.Pixel_CloseTo_Requirement, "bar stacks from 0");
                stackedBarPlot.destroy();
                div.remove();
            });
        });
    });
});
