"use strict";
var d3 = require("d3");
var chai_1 = require("chai");
var Plottable = require("../../src");
var TestMethods = require("../testMethods");
describe("Plots", function () {
    describe("Waterfall", function () {
        describe("rendering growth bars", function () {
            var numAttr = TestMethods.numAttr;
            var div;
            var dataset;
            var xScale;
            var yScale;
            var plot;
            var growthBarData = [
                { x: "A", y: 0 },
                { x: "B", y: 5 },
                { x: "C", y: 10 },
                { x: "D", y: 100 },
            ];
            var growthClass = "waterfall-growth";
            beforeEach(function () {
                div = TestMethods.generateDiv();
                dataset = new Plottable.Dataset(growthBarData);
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Waterfall();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
                plot.total(function (d, i) { return i === 0; });
                plot.addDataset(dataset);
                plot.renderTo(div);
            });
            it("classes growth bars", function () {
                var bars = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(bars.size(), growthBarData.length, "same number of bars as data points");
                bars.each(function (d, i) {
                    if (i === 0) {
                        return;
                    }
                    var bar = d3.select(this);
                    chai_1.assert.isTrue(bar.classed(growthClass), "bar classed as growth bar");
                });
                plot.destroy();
                div.remove();
            });
            it("places bars at current sum", function () {
                var bars = plot.content().selectAll("rect." + growthClass);
                chai_1.assert.strictEqual(bars.size(), growthBarData.length - 1, "all bars are growth except for first");
                var yAccessor = plot.y().accessor;
                var sum = 0;
                bars.each(function (d, i) {
                    var dataY = yAccessor(d, i, dataset);
                    var bar = d3.select(this);
                    chai_1.assert.closeTo(numAttr(bar, "y") + numAttr(bar, "height"), yScale.scale(sum), window.Pixel_CloseTo_Requirement, "growth bar bottom at previous sum");
                    sum += dataY;
                    chai_1.assert.closeTo(numAttr(bar, "y"), yScale.scale(sum), window.Pixel_CloseTo_Requirement, "growth bar top at final sum");
                });
                plot.destroy();
                div.remove();
            });
        });
        describe("rendering decline bars", function () {
            var numAttr = TestMethods.numAttr;
            var div;
            var dataset;
            var xScale;
            var yScale;
            var plot;
            var declineBarData = [
                { x: "A", y: 0 },
                { x: "B", y: -5 },
                { x: "C", y: -25 },
                { x: "D", y: -10 },
                { x: "E", y: -15 },
            ];
            var declineClass = "waterfall-decline";
            beforeEach(function () {
                div = TestMethods.generateDiv();
                dataset = new Plottable.Dataset(declineBarData);
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Waterfall();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
                plot.total(function (d, i) { return i === 0; });
                plot.addDataset(dataset);
                plot.renderTo(div);
            });
            it("classes decline bars", function () {
                var bars = plot.content().selectAll("rect");
                chai_1.assert.strictEqual(bars.size(), declineBarData.length, "same number of bars as data points");
                bars.each(function (d, i) {
                    if (i === 0) {
                        return;
                    }
                    var bar = d3.select(this);
                    chai_1.assert.isTrue(bar.classed(declineClass), "bar classed as decline bars");
                });
                plot.destroy();
                div.remove();
            });
            it("places bars at current sum", function () {
                var bars = plot.content().selectAll("rect." + declineClass);
                chai_1.assert.strictEqual(bars.size(), declineBarData.length - 1, "all bars are decline except for first");
                var yAccessor = plot.y().accessor;
                var sum = 0;
                bars.each(function (d, i) {
                    var dataY = yAccessor(d, i, dataset);
                    var bar = d3.select(this);
                    chai_1.assert.closeTo(numAttr(bar, "y"), yScale.scale(sum), window.Pixel_CloseTo_Requirement, "growth bar top at previous sum");
                    sum += dataY;
                    chai_1.assert.closeTo(numAttr(bar, "y") + numAttr(bar, "height"), yScale.scale(sum), window.Pixel_CloseTo_Requirement, "growth bar bottom at final sum");
                });
                plot.destroy();
                div.remove();
            });
        });
        describe("denoting total bars", function () {
            var div;
            var dataset;
            var xScale;
            var yScale;
            var plot;
            var data = [
                { x: "A", y: 20, t: true },
                { x: "B", y: 5, t: false },
                { x: "C", y: 25, t: true },
                { x: "D", y: -10, t: false },
                { x: "E", y: 15, t: true },
            ];
            var totalClass = "waterfall-total";
            beforeEach(function () {
                div = TestMethods.generateDiv();
                dataset = new Plottable.Dataset(data);
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Waterfall();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
            });
            it("can set the total property", function () {
                var accessor = function (d) { return d.t; };
                chai_1.assert.strictEqual(plot.total(accessor), plot, "setter returns calling object");
                plot.addDataset(dataset);
                plot.renderTo(div);
                var bars = plot.content().selectAll("rect").filter(function (d) { return accessor(d); });
                bars.each(function (d) {
                    var totalBar = d3.select(this);
                    chai_1.assert.isTrue(totalBar.classed(totalClass));
                });
                plot.destroy();
                div.remove();
            });
            it("can get the total property", function () {
                var accessor = function (d) { return d.t === "total"; };
                plot.total(accessor);
                chai_1.assert.strictEqual(plot.total().accessor, accessor, "can get if connectors are enabled");
                plot.destroy();
                div.remove();
            });
        });
        describe("enabling connectors", function () {
            var numAttr = TestMethods.numAttr;
            var div;
            var dataset;
            var xScale;
            var yScale;
            var plot;
            var data = [
                { x: "A", y: 20, t: true },
                { x: "B", y: 5, t: false },
                { x: "C", y: 25, t: true },
                { x: "D", y: -10, t: false },
                { x: "E", y: 15, t: true },
            ];
            beforeEach(function () {
                div = TestMethods.generateDiv();
                dataset = new Plottable.Dataset(data);
                xScale = new Plottable.Scales.Category();
                yScale = new Plottable.Scales.Linear();
                plot = new Plottable.Plots.Waterfall();
                plot.x(function (d) { return d.x; }, xScale);
                plot.y(function (d) { return d.y; }, yScale);
                plot.total(function (d) { return d.t; });
                plot.addDataset(dataset);
            });
            it("can set if connectors are enabled", function () {
                chai_1.assert.isFalse(plot.connectorsEnabled(), "no connectors by default");
                chai_1.assert.strictEqual(plot.connectorsEnabled(true), plot, "setter returns calling object");
                plot.renderTo(div);
                var bars = plot.content().selectAll("rect");
                var connectors = plot.content().selectAll("line.connector");
                chai_1.assert.strictEqual(bars.size(), connectors.size() + 1, "there is one more bar than number of connectors");
                connectors.each(function (datum, index) {
                    var connector = d3.select(this);
                    var bar = d3.select(bars.nodes()[index]);
                    var connectorOnBottom = bar.classed("waterfall-decline");
                    if (connectorOnBottom) {
                        chai_1.assert.closeTo(numAttr(connector, "y1"), numAttr(bar, "y") + numAttr(bar, "height"), window.Pixel_CloseTo_Requirement, "connector on declining bar at bottom");
                    }
                    else {
                        chai_1.assert.closeTo(numAttr(connector, "y1"), numAttr(bar, "y"), window.Pixel_CloseTo_Requirement, "connector on non-declining bar at top");
                    }
                    chai_1.assert.strictEqual(numAttr(connector, "y1"), numAttr(connector, "y2"), "connector stays at same height");
                });
                plot.destroy();
                div.remove();
            });
            it("can get if connectors are enabled", function () {
                var connectorsEnabled = true;
                plot.connectorsEnabled(connectorsEnabled);
                chai_1.assert.strictEqual(plot.connectorsEnabled(), connectorsEnabled, "can get if connectors are enabled");
                plot.destroy();
                div.remove();
            });
        });
    });
});
