"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Scales", function () {
    describe("TimeScale tests", function () {
        describe("Basic Usage", function () {
            var scale;
            beforeEach(function () {
                scale = new Plottable.Scales.Time();
            });
            it("calculates the correct extent", function () {
                var date1 = new Date(2015, 2, 25, 19, 0, 0);
                var date2 = new Date(2015, 2, 24, 19, 0, 0);
                var date3 = new Date(2015, 2, 25, 19, 0, 0);
                var date4 = new Date(2015, 2, 26, 19, 0, 0);
                var values = [date1, date2, date3, date4];
                var computedExtent = scale.extentOfValues(values);
                chai_1.assert.deepEqual(computedExtent, [date2, date4], "The extent is the miminum and the maximum value in the domain");
            });
            it.skip("ignores invalid values when calculating the extent", function () {
                var expectedExtent = [new Date("2015-06-05"), new Date("2015-06-04")];
                var arrayWithBadValues = [
                    null,
                    NaN,
                    undefined,
                    Infinity,
                    -Infinity,
                    "a string",
                    0,
                    expectedExtent[0],
                    expectedExtent[1],
                ];
                var extent = scale.extentOfValues(arrayWithBadValues);
                chai_1.assert.strictEqual(extent[0].getTime(), expectedExtent[0].getTime(), "returned correct min");
                chai_1.assert.strictEqual(extent[1].getTime(), expectedExtent[1].getTime(), "returned correct max");
            });
            it("can be padded", function () {
                scale.padProportion(0);
                var unpaddedDomain = scale.domain();
                scale.addIncludedValuesProvider(function () { return unpaddedDomain; });
                scale.padProportion(0.1);
                chai_1.assert.operator(scale.domain()[0].getTime(), "<", unpaddedDomain[0].getTime(), "left side of domain was padded");
                chai_1.assert.operator(scale.domain()[1].getTime(), ">", unpaddedDomain[1].getTime(), "right side of domain was padded");
            });
            it("has a consistent default domain", function () {
                var scale2 = new Plottable.Scales.Time();
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), scale2.domain()[0].getTime(), "both scales have the same default left side of domain");
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), scale2.domain()[1].getTime(), "both scales have the same default right side of domain");
            });
            it("respects padding exceptions", function () {
                var minValue = new Date(2000, 5, 4);
                var maxValue = new Date(2000, 5, 6);
                scale.addIncludedValuesProvider(function () { return [minValue, maxValue]; });
                chai_1.assert.operator(scale.domain()[0].getTime(), "<", minValue.getTime(), "left side of domain is normally padded");
                chai_1.assert.operator(scale.domain()[1].getTime(), ">", maxValue.getTime(), "right side of domain is normally padded");
                scale.addPaddingExceptionsProvider(function () { return [minValue]; });
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), minValue.getTime(), "left side of domain isn't padded if it matches the exception");
                var maxValuePaddingException = function () { return [maxValue]; };
                scale.addPaddingExceptionsProvider(maxValuePaddingException);
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), maxValue.getTime(), "right side of domain isn't padded if it matches the exception");
                scale.removePaddingExceptionsProvider(maxValuePaddingException);
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), minValue.getTime(), "left side of domain still isn't padded");
                chai_1.assert.operator(scale.domain()[1].getTime(), ">", maxValue.getTime(), "right side of the domain is padded again because exception was removed");
            });
            it("can't set reversed domain", function () {
                chai_1.assert.throws(function () { return scale.domain([new Date("1986-10-26"), new Date("1985-11-05")]); }, "chronological");
            });
            it("produces correct number of ticks", function () {
                // 100 year span
                scale.domain([new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2100, 0, 1, 0, 0, 0, 0)]);
                var yearTicks = scale.tickInterval(Plottable.TimeInterval.year);
                chai_1.assert.strictEqual(yearTicks.length, 101, "generated correct number of ticks for 100 year span, every year");
                // 1 year span
                scale.domain([new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 11, 31, 0, 0, 0, 0)]);
                var monthTicks = scale.tickInterval(Plottable.TimeInterval.month);
                chai_1.assert.strictEqual(monthTicks.length, 12, "generated correct number of ticks for 1 year span, every month");
                var threeMonthTicks = scale.tickInterval(Plottable.TimeInterval.month, 3);
                chai_1.assert.strictEqual(threeMonthTicks.length, 4, "generated correct number of ticks for 1 year span, every 3 months");
                // 1 month span
                scale.domain([new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 1, 1, 0, 0, 0, 0)]);
                var dayTicks = scale.tickInterval(Plottable.TimeInterval.day);
                chai_1.assert.strictEqual(dayTicks.length, 32, "generated correct number of ticks for 1 month span, every day");
                // 1 day span
                scale.domain([new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 23, 0, 0, 0)]);
                var hourTicks = scale.tickInterval(Plottable.TimeInterval.hour);
                chai_1.assert.strictEqual(hourTicks.length, 24, "generated correct number of ticks for 1 day span, every hour");
                // 1 hour span
                scale.domain([new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 1, 0, 0, 0)]);
                var minuteTicks = scale.tickInterval(Plottable.TimeInterval.minute);
                chai_1.assert.strictEqual(minuteTicks.length, 61, "generated correct number of ticks for 1 hour span, every minute");
                var tenMinuteTicks = scale.tickInterval(Plottable.TimeInterval.minute, 10);
                chai_1.assert.strictEqual(tenMinuteTicks.length, 7, "generated correct number of ticks for 1 hour span, every 10 minutes");
                // 1 minute span
                scale.domain([new Date(2000, 0, 1, 0, 0, 0, 0), new Date(2000, 0, 1, 0, 1, 0, 0)]);
                var secondTicks = scale.tickInterval(Plottable.TimeInterval.second);
                chai_1.assert.strictEqual(secondTicks.length, 61, "generated correct number of ticks for 1 minute span, every second");
            });
        });
        describe("Automatic Domain Calculation", function () {
            it("expands single value domains to [value - 1, value + 1] when autoDomain()-ing", function () {
                var scale = new Plottable.Scales.Time();
                var singleValue = new Date(2000, 5, 5);
                var dayBefore = new Date(2000, 5, 4);
                var dayAfter = new Date(2000, 5, 6);
                scale.addIncludedValuesProvider(function () { return [singleValue]; });
                scale.autoDomain();
                var domain = scale.domain();
                chai_1.assert.strictEqual(domain[0].getTime(), dayBefore.getTime(), "left side of domain was expaded by one day");
                chai_1.assert.strictEqual(domain[1].getTime(), dayAfter.getTime(), "right side of domain was expaded by one day");
            });
            it("doesn't lock up if a zero-width domain is set while there are value providers", function () {
                var scale = new Plottable.Scales.Time();
                scale.padProportion(0.1);
                var provider = function () { return [new Date(2000, 5, 5), new Date(2000, 5, 6)]; };
                scale.addIncludedValuesProvider(provider);
                scale.autoDomain();
                var originalAutoDomain = scale.domain();
                scale.domain([new Date(0), new Date(0)]);
                scale.autoDomain();
                var domainAfter = scale.domain();
                var numberize = function (d) { return d.getTime(); };
                chai_1.assert.deepEqual(domainAfter.map(numberize), originalAutoDomain.map(numberize), "autodomained as expected");
            });
        });
        describe("Domain constraints with domainMin() and domainMax()", function () {
            var scale;
            beforeEach(function () {
                scale = new Plottable.Scales.Time();
            });
            it("can force the minimum of the domain with domainMin()", function () {
                scale.padProportion(0);
                var requestedDomain = [new Date("2015-05-01"), new Date("2015-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var minBelowBottom = new Date("2015-04-01");
                chai_1.assert.strictEqual(scale.domainMin(minBelowBottom), scale, "the scale is returned by the setter");
                chai_1.assert.strictEqual(scale.domainMin().getTime(), minBelowBottom.getTime(), "can get domainMin()");
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), minBelowBottom.getTime(), "lower end of domain was set by domainMin()");
                var minInMiddle = new Date("2015-06-01");
                scale.domainMin(minInMiddle);
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), minInMiddle.getTime(), "lower end was set even if requested value cuts off some data");
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "calling autoDomain() overrides domainMin()");
                chai_1.assert.strictEqual(scale.domainMin().getTime(), scale.domain()[0].getTime(), "returns autoDomain()-ed min value after autoDomain()-ing");
                var minEqualTop = scale.domain()[1];
                var nextDay = new Date("2015-07-02");
                scale.domainMin(minEqualTop);
                var domain = scale.domain();
                chai_1.assert.strictEqual(domain[0].getTime(), minEqualTop.getTime(), "lower end was set even if requested value is >= autoDomain()-ed max");
                chai_1.assert.strictEqual(domain[1].getTime(), nextDay.getTime(), "upper end is set one day later");
            });
            it("does not change domainMin() after adding more included values providers", function () {
                scale.padProportion(0);
                var requestedDomain = [new Date("2015-05-01"), new Date("2015-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var minInMiddle = new Date("2015-06-01");
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), requestedDomain[0].getTime(), "domain minimum is the minimum value");
                scale.domainMin(minInMiddle);
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), minInMiddle.getTime(), "can set domain minimum with domainMin()");
                var requestedDomain2 = [new Date("2014-05-01"), new Date("2016-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain2; });
                chai_1.assert.strictEqual(scale.domain()[0].getTime(), minInMiddle.getTime(), "adding IncludedValuesProvider doesn't change domainMin()");
            });
            it("can force the maximum of the domain with domainMax()", function () {
                scale.padProportion(0);
                var requestedDomain = [new Date("2015-05-01"), new Date("2015-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var maxAboveTop = new Date("2015-08-01");
                chai_1.assert.strictEqual(scale.domainMax(maxAboveTop), scale, "the scale is returned by the setter");
                chai_1.assert.strictEqual(scale.domainMax().getTime(), maxAboveTop.getTime(), "can get domainMax()");
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), maxAboveTop.getTime(), "upper end of domain was set by domainMax()");
                var maxInMiddle = new Date("2015-06-01");
                scale.domainMax(maxInMiddle);
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), maxInMiddle.getTime(), "upper end was set even if requested value cuts off some data");
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "calling autoDomain() overrides domainMax()");
                chai_1.assert.strictEqual(scale.domainMax().getTime(), scale.domain()[1].getTime(), "returns autoDomain()-ed max value after autoDomain()-ing");
                var maxEqualBottom = new Date("2015-05-01");
                var dayBefore = new Date("2015-04-30");
                scale.domainMax(maxEqualBottom);
                var domain = scale.domain();
                chai_1.assert.strictEqual(domain[1].getTime(), maxEqualBottom.getTime(), "upper end was set even if requested value is <= autoDomain()-ed min");
                chai_1.assert.strictEqual(domain[0].getTime(), dayBefore.getTime(), "lower end is set one day before");
            });
            it("does not change domainMax() after adding more included values providers", function () {
                scale.padProportion(0);
                var requestedDomain = [new Date("2015-05-01"), new Date("2015-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var maxInMiddle = new Date("2015-06-01");
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), requestedDomain[1].getTime(), "domain maximum is the maximum value");
                scale.domainMax(maxInMiddle);
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), maxInMiddle.getTime(), "can set domain maximum with domainMax()");
                var requestedDomain2 = [new Date("2014-05-01"), new Date("2016-07-01")];
                scale.addIncludedValuesProvider(function (_scale) { return requestedDomain2; });
                chai_1.assert.strictEqual(scale.domain()[1].getTime(), maxInMiddle.getTime(), "adding another ExtentsProvider doesn't change domainMax()");
            });
            it("can force the domain by using domainMin() and domainMax() together", function () {
                scale.padProportion(0);
                var requestedDomain = [new Date("2015-05-01"), new Date("2015-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var desiredMin = new Date("2015-04-01");
                var desiredMax = new Date("2015-06-01");
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "there are no initial constraings");
                scale.domainMin(desiredMin);
                scale.domainMax(desiredMax);
                chai_1.assert.deepEqual(scale.domain(), [desiredMin, desiredMax], "setting domainMin() and domainMax() sets the domain");
                scale.autoDomain();
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "autoDomaining overwrites the domain constraints");
            });
            it.skip("cannot invert the domain using domainMin() or domainMax()", function () {
                scale.padProportion(0);
                var requestedDomain = [new Date("2015-05-01"), new Date("2015-07-01")];
                scale.addIncludedValuesProvider(function () { return requestedDomain; });
                var bigMin = new Date("2015-08-01");
                var smallMax = new Date("2015-04-01");
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "initial domain is set");
                scale.domainMin(bigMin);
                chai_1.assert.throws(function () { return scale.domainMax(smallMax); }, Error, "domain values must be in chronological order", "cannot invert the domain using domainMax()");
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "scale domain did not change");
                scale.domainMax(smallMax);
                chai_1.assert.throws(function () { return scale.domainMin(bigMin); }, Error, "domain values must be in chronological order", "cannot invert the domain using domainMin()");
                chai_1.assert.deepEqual(scale.domain(), requestedDomain, "scale domain did not change");
            });
        });
    });
});
