"use strict";
var chai_1 = require("chai");
var Plottable = require("../../src");
describe("Utils.Methods", function () {
    it("inRange()", function () {
        chai_1.assert.isTrue(Plottable.Utils.Math.inRange(0, -1, 1), "basic functionality works");
        chai_1.assert.isTrue(Plottable.Utils.Math.inRange(0, 0, 1), "it is a closed interval");
        chai_1.assert.isTrue(!Plottable.Utils.Math.inRange(0, 1, 2), "returns false when false");
    });
    describe("max() and min()", function () {
        var max = Plottable.Utils.Math.max;
        var min = Plottable.Utils.Math.min;
        var today = new Date();
        it("return the default value if max() or min() can't be computed", function () {
            var minValue = 1;
            var maxValue = 5;
            var defaultValue = 3;
            var goodArray = [
                [minValue],
                [maxValue],
            ];
            // bad array is technically of type number[][], but subarrays are empty!
            var badArray = [
                [],
                [],
            ];
            var accessor = function (arr) { return arr[0]; };
            chai_1.assert.strictEqual(min(goodArray, accessor, defaultValue), minValue, "min(): minimum value is returned in good case");
            chai_1.assert.strictEqual(min(badArray, accessor, defaultValue), defaultValue, "min(): default value is returned in bad case");
            chai_1.assert.strictEqual(max(goodArray, accessor, defaultValue), maxValue, "max(): maximum value is returned in good case");
            chai_1.assert.strictEqual(max(badArray, accessor, defaultValue), defaultValue, "max(): default value is returned in bad case");
        });
        it("max() and min() work on numbers", function () {
            var alist = [1, 2, 3, 4, 5];
            var dbl = function (x) { return x * 2; };
            var dblIndexOffset = function (x, i) { return x * 2 - i; };
            var numToDate = function (x) {
                var t = new Date(today.getTime());
                t.setDate(today.getDate() + x);
                return t;
            };
            chai_1.assert.deepEqual(max(alist, 99), 5, "max ignores default on non-empty array");
            chai_1.assert.deepEqual(max(alist, dbl, 0), 10, "max applies function appropriately");
            chai_1.assert.deepEqual(max(alist, dblIndexOffset, 5), 6, "max applies function with index");
            chai_1.assert.deepEqual(max(alist, numToDate, today), numToDate(5), "max applies non-numeric function appropriately");
            chai_1.assert.deepEqual(max([], 10), 10, "works as intended with default value");
            chai_1.assert.deepEqual(max([], dbl, 5), 5, "default value works with function");
            chai_1.assert.deepEqual(max([], numToDate, today), today, "default non-numeric value works with non-numeric function");
            chai_1.assert.deepEqual(min(alist, 0), 1, "min works for basic list");
            chai_1.assert.deepEqual(min(alist, dbl, 0), 2, "min works with function arg");
            chai_1.assert.deepEqual(min(alist, dblIndexOffset, 0), 2, "min works with function index arg");
            chai_1.assert.deepEqual(min(alist, numToDate, today), numToDate(1), "min works with non-numeric function arg");
            chai_1.assert.deepEqual(min([], dbl, 5), 5, "min accepts custom default and function");
            chai_1.assert.deepEqual(min([], numToDate, today), today, "min accepts non-numeric default and function");
        });
        it("max() and min() work on strings", function () {
            var strings = ["a", "bb", "ccc", "ddd"];
            chai_1.assert.deepEqual(max(strings, function (s) { return s.length; }, 0), 3, "works on arrays of non-numbers with a function");
            chai_1.assert.deepEqual(max([], function (s) { return s.length; }, 5), 5, "defaults work even with non-number function type");
        });
        it("max() and min() work on dates", function () {
            var tomorrow = new Date(today.getTime());
            tomorrow.setDate(today.getDate() + 1);
            var dayAfterTomorrow = new Date(today.getTime());
            dayAfterTomorrow.setDate(today.getDate() + 2);
            var dates = [today, tomorrow, dayAfterTomorrow, null];
            chai_1.assert.deepEqual(min(dates, dayAfterTomorrow), today, "works on arrays of non-numeric values but comparable");
            chai_1.assert.deepEqual(max(dates, today), dayAfterTomorrow, "works on arrays of non-number values but comparable");
            chai_1.assert.deepEqual(max([null], today), today, "returns default value if passed array of null values");
            chai_1.assert.deepEqual(max([], today), today, "returns default value if passed empty");
        });
    });
    it("isNaN()", function () {
        var isNaN = Plottable.Utils.Math.isNaN;
        chai_1.assert.isTrue(isNaN(NaN), "Only NaN should pass the isNaN check");
        chai_1.assert.isFalse(isNaN(undefined), "undefined should fail the isNaN check");
        chai_1.assert.isFalse(isNaN(null), "null should fail the isNaN check");
        chai_1.assert.isFalse(isNaN(Infinity), "Infinity should fail the isNaN check");
        chai_1.assert.isFalse(isNaN(1), "numbers should fail the isNaN check");
        chai_1.assert.isFalse(isNaN(0), "0 should fail the isNaN check");
        chai_1.assert.isFalse(isNaN("foo"), "strings should fail the isNaN check");
        chai_1.assert.isFalse(isNaN(""), "empty strings should fail the isNaN check");
        chai_1.assert.isFalse(isNaN({}), "empty Objects should fail the isNaN check");
    });
    it("isValidNumber()", function () {
        var isValidNumber = Plottable.Utils.Math.isValidNumber;
        chai_1.assert.isTrue(isValidNumber(0), "(0 is a valid number");
        chai_1.assert.isTrue(isValidNumber(1), "(1 is a valid number");
        chai_1.assert.isTrue(isValidNumber(-1), "(-1 is a valid number");
        chai_1.assert.isTrue(isValidNumber(0.1), "(0.1 is a valid number");
        chai_1.assert.isFalse(isValidNumber(null), "(null is not a valid number");
        chai_1.assert.isFalse(isValidNumber(NaN), "(NaN is not a valid number");
        chai_1.assert.isFalse(isValidNumber(undefined), "(undefined is not a valid number");
        chai_1.assert.isFalse(isValidNumber(Infinity), "(Infinity is not a valid number");
        chai_1.assert.isFalse(isValidNumber(-Infinity), "(-Infinity is not a valid number");
        chai_1.assert.isFalse(isValidNumber("number"), "('number' is not a valid number");
        chai_1.assert.isFalse(isValidNumber("string"), "('string' is not a valid number");
        chai_1.assert.isFalse(isValidNumber("0"), "('0' is not a valid number");
        chai_1.assert.isFalse(isValidNumber("1"), "('1' is not a valid number");
        chai_1.assert.isFalse(isValidNumber([]), "([] is not a valid number");
        chai_1.assert.isFalse(isValidNumber([1]), "([1] is not a valid number");
        chai_1.assert.isFalse(isValidNumber({}), "({} is not a valid number");
        chai_1.assert.isFalse(isValidNumber({ 1: 1 }), "({1: 1} is not a valid number");
    });
    it("range()", function () {
        var start = 0;
        var end = 6;
        var range = Plottable.Utils.Math.range(start, end);
        chai_1.assert.deepEqual(range, [0, 1, 2, 3, 4, 5], "all entries has been generated");
        range = Plottable.Utils.Math.range(start, end, 2);
        chai_1.assert.deepEqual(range, [0, 2, 4], "all entries has been generated");
        range = Plottable.Utils.Math.range(start, end, 11);
        chai_1.assert.deepEqual(range, [0], "all entries has been generated");
        chai_1.assert.throws(function () { return Plottable.Utils.Math.range(start, end, 0); }, "step cannot be 0");
        range = Plottable.Utils.Math.range(start, end, -1);
        chai_1.assert.lengthOf(range, 0, "no entries because of invalid step");
        range = Plottable.Utils.Math.range(end, start, -1);
        chai_1.assert.deepEqual(range, [6, 5, 4, 3, 2, 1], "all entries has been generated");
        range = Plottable.Utils.Math.range(-2, 2);
        chai_1.assert.deepEqual(range, [-2, -1, 0, 1], "all entries has been generated range crossing 0");
        range = Plottable.Utils.Math.range(0.2, 4);
        chai_1.assert.deepEqual(range, [0.2, 1.2, 2.2, 3.2], "all entries has been generated with float start");
        range = Plottable.Utils.Math.range(0.6, 2.2, 0.5);
        chai_1.assert.deepEqual(range, [0.6, 1.1, 1.6, 2.1], "all entries has been generated with float step");
    });
});
